/*
 * Copyright Staffan Gimåker 2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Config.hh"
#include "MessageHub.hh"


using namespace peekabot;


Config::Config()
{
}


Config::~Config()
{
}


bool Config::has(const std::string &key) const
{
    boost::recursive_mutex::scoped_lock lock(m_mutex);

    return ( m_generated.has(key) ||
             m_overrides.has(key) ||
             m_persistent.has(key) ||
             m_defaults.has(key) );
}


void Config::load(const std::string &filename)
{
    boost::recursive_mutex::scoped_lock lock(m_mutex);

    try
    {
        m_persistent.load_ini(filename);
    }
    catch(std::exception &e)
    {
        TheMessageHub::instance().publish(
            ERROR_MESSAGE,
            "Failed to load configuration: %s", e.what());
    }
}


void Config::save(const std::string &filename) const
{
    boost::recursive_mutex::scoped_lock lock(m_mutex);

    try
    {
        m_persistent.save_ini(filename);
    }
    catch(std::exception &e)
    {
        TheMessageHub::instance().publish(
            ERROR_MESSAGE,
            "Failed to save configuration: %s", e.what());
    }
}


Plist &Config::get_generated()
{
    return m_generated;
}


const Plist &Config::get_generated() const
{
    return m_generated;
}


Plist &Config::get_defaults()
{
    return m_defaults;
}


const Plist &Config::get_defaults() const
{
    return m_defaults;
}


LexicalMap &Config::get_overrides()
{
    return m_overrides;
}


const LexicalMap &Config::get_overrides() const
{
    return m_overrides;
}


LexicalMap &Config::get_persistent()
{
    return m_persistent;
}


const LexicalMap &Config::get_persistent() const
{
    return m_persistent;
}
