/*
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

/**
 * \file Contains definitions for global constants.
 */

#include <boost/format.hpp>

#include "Protocol.hh"
#include "Types.hh"

using namespace peekabot;


// -------------- Protocol.hh --------------


/**
 * \brief The peekabot network transport protocol version.
 */
const uint32_t peekabot::protocol::PROTOCOL_VERSION = 3;


/**
 * \brief The unique ID sent at the beginning of authentication requests.
 */
const uint32_t peekabot::protocol::UNIQUE_ID = 0x7065656B;

/**
 * \brief Authentication timeout length in milliseconds.
 */
const unsigned int peekabot::protocol::AUTHENTICATION_TIMEOUT = 10000;


// ------------- Types.hh --------------


/**
 * \brief peekabot major version.
 */
const uint8_t peekabot::version::PEEKABOT_VERSION_MAJOR    = 0;

/**
 * \brief peekabot minor version.
 */
const uint8_t peekabot::version::PEEKABOT_VERSION_MINOR    = 6;

/**
 * \brief peekabot version revision.
 */
const uint8_t peekabot::version::PEEKABOT_VERSION_REVISION = 0;


const uint32_t peekabot::version::PEEKABOT_VERSION = (
    (peekabot::version::PEEKABOT_VERSION_MAJOR<<16) | 
    (peekabot::version::PEEKABOT_VERSION_MINOR<<8)  | 
    (peekabot::version::PEEKABOT_VERSION_REVISION));


const std::string peekabot::version::PEEKABOT_VERSION_STRING = (
    boost::format("%1%.%2%.%3%") %  
    (int)peekabot::version::PEEKABOT_VERSION_MAJOR %
    (int)peekabot::version::PEEKABOT_VERSION_MINOR %
    (int)peekabot::version::PEEKABOT_VERSION_REVISION).str();

/**
 * \brief The least recent version of peekabot this peekabot version
 * is backwards compatible with.
 *
 * If any actions are changed or removed between versions, this should be 
 * updated accordingly!
 */
const uint32_t peekabot::version::PEEKABOT_COMPATIBLE_VERSION = (
    (0<<16) | 
    (6<<8)  | 
    0);





/** 
 *\brief Constant setting the number of visibility layers supported.
 */
const uint32_t peekabot::NUMBER_OF_LAYERS = 16;
