/*
 * Copyright Staffan Gimåker 2007-2008.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef __PEEKABOT_GENERIC_RESULT_HH
#define __PEEKABOT_GENERIC_RESULT_HH


#include "ActionResult.hh"


namespace peekabot
{

    /**
     * \internal
     *
     * \brief An action result class that can be used to transmit basic 
     * peekabot types.
     *
     * The types supported are listed in \c GenericResult.cc, but typically
     * include anything that can directly be handled by \c Serialization- and \c
     * DeserializationBuffer. Matrices, strings and (most) numeric types, for
     * example.
     */
    template<class T>
    class GenericResult : public ActionResult
    {
        PEEKABOT_SERIALIZABLE

    public:
        typedef T ResultType;

        GenericResult(const ResultType &x) throw() : m_x(x) {}

        virtual ~GenericResult() throw() {}


        virtual boost::any get_result() throw()
        {
            return m_x;
        }

        /*virtual ResultTypeID get_result_type() const throw()
        {
            return guid();
        }

        virtual void serialize_state(
            SerializationBuffer &buf) const throw()
        {
            buf.write(m_x);
        }

        virtual void deserialize_state(
            DeserializationBuffer &buf) throw(std::exception)
        {
            buf.read(m_x);
            }

        static void register_result_type(ResultTypeID _guid)
        {
            guid() = _guid;
            TheActionResultFactory::instance().register_product(
                guid(), &create);
                }*/


    private:
        GenericResult() throw() {}

        virtual void save(SerializationInterface &ar) const
        {
            ar << m_x;
        }

        virtual void load(DeserializationInterface &ar)
        {
            ar >> m_x;
        }

        /*static ResultTypeID &guid() throw()
        {
            static ResultTypeID s_guid = static_cast<ResultTypeID>(-1);
            return s_guid;
        }

        static ActionResult *create() throw()
        {
            return new GenericResult();
        }*/

    private:

        ResultType m_x;
    };


}


#endif // __PEEKABOT_GENERIC_RESULT_HH
