/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "IDTranslationTable.hh"
#include "SceneObject.hh"


using namespace peekabot;


// --------------------- IDTranslationTable implementation ---------------------

void IDTranslationTable::add_node_translation_entry(
    ObjectID object_id, NodeID node_id) throw()
{
    m_node_id_map.insert(std::make_pair(object_id, node_id));
}


void IDTranslationTable::remove_node_translation_entry(
    ObjectID object_id) throw()
{
    m_node_id_map.erase(object_id);
}

void IDTranslationTable::remove_node_translation_entries(
    SceneObject *subtree) throw()
{
    remove_node_translation_entry(subtree->get_object_id());
    
    SceneObject::ChildContainer &children = subtree->get_children();
    for( SceneObject::ChildContainer::iterator it = children.begin();
         it != children.end(); ++it )
    {
        remove_node_translation_entries(*it);
    }
}

NodeID IDTranslationTable::get_node_id(
    ObjectID object_id) const throw()
{
    NodeIDMap::const_iterator it = m_node_id_map.find(object_id);
    
    if( it != m_node_id_map.end() )
        return it->second;
    else
        return -1; /// \todo We should report errors somehow. Throw?
}


void IDTranslationTable::add_primitive_translation_entry(
    ObjectID object_id, PrimitiveID primitive_id) throw()
{
    m_primitive_id_map.insert(std::make_pair(object_id, primitive_id));
}


void IDTranslationTable::remove_primitive_translation_entry(
    ObjectID object_id) throw()
{
    m_primitive_id_map.erase(object_id);
}

void IDTranslationTable::remove_primitive_translation_entries(
    SceneObject *subtree) throw()
{
    remove_primitive_translation_entry(subtree->get_object_id());
    
    SceneObject::ChildContainer &children = subtree->get_children();
    for( SceneObject::ChildContainer::iterator it = children.begin();
         it != children.end(); ++it )
    {
        remove_primitive_translation_entries(*it);
    }
}

PrimitiveID IDTranslationTable::get_primitive_id(
    ObjectID object_id) const throw()
{
    PrimitiveIDMap::const_iterator it = m_primitive_id_map.find(object_id);
    
    if( it != m_primitive_id_map.end() )
        return it->second;
    else
        return -1; /// \todo We should report errors somehow. Throw?
}
