/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_ID_TRANSLATION_TABLE_HH
#define __PEEKABOT_ID_TRANSLATION_TABLE_HH


#include <map>
#include "Types.hh"
#include "renderer/IDTypes.hh"
#include "Singleton.hh"


namespace peekabot
{
    class SceneObject;


    class IDTranslationTable
    {
    public:
        void add_node_translation_entry(ObjectID object_id, NodeID node_id) throw();
        
        void remove_node_translation_entry(ObjectID object_id) throw();
        
        void remove_node_translation_entries(SceneObject *subtree) throw();
        
        NodeID get_node_id(ObjectID object_id) const throw();
        
        
        void add_primitive_translation_entry(ObjectID object_id, PrimitiveID primitive_id) throw();
        
        void remove_primitive_translation_entry(ObjectID object_id) throw();
        
        void remove_primitive_translation_entries(SceneObject *subtree) throw();
        
        PrimitiveID get_primitive_id(ObjectID object_id) const throw();
        
    private:
        // For mapping from ObjectIDs to NodeIDs
        typedef std::map<ObjectID, NodeID> NodeIDMap;
        NodeIDMap m_node_id_map;
        
        // For mapping from ObjectIDs to PrimitiveIDs
        typedef std::map<ObjectID, PrimitiveID> PrimitiveIDMap;
        PrimitiveIDMap m_primitive_id_map;
    };


    typedef Singleton<IDTranslationTable> TheIDTranslationTable;

}


#endif // __PEEKABOT_ID_TRANSLATION_TABLE_HH
