/*
 * Copyright Staffan Gimåker 2007-2008.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <set>
#include <boost/scoped_ptr.hpp>

#include "Init.hh"
#include "Types.hh"
#include "Sockets.hh"
#include "serialization/SerializableFactory.hh"
#include "serialization/Containers.hh"
#include "TypesIO.hh"
#include "VectorIO.hh"
#include "MatrixIO.hh"
#include "ActionRegistry.hh"

//
// ActionResult headers:
//
#include "GenericResult.hh"


using namespace peekabot;


namespace
{
    /** 
     * \brief Keeps track of how many times init() was called.
     *
     * Calling init() and cleanup() increments and decrements it by one 
     * respectively. When \c init_count goes from and to zero actual init 
     * and clean up is performed.
     */
    int init_count = 0;

    void register_action_results()
    {
        int uuid = 10000;

        TheSerializableFactory::instance().register_class<GenericResult<Matrix4ru> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<Vector4ru> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<Vector3ru> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<uint8_t> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<int8_t> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<uint16_t> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<int16_t> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<uint32_t> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<int32_t> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<uint64_t> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<int64_t> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<float> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<double> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<std::string> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<RGBColor> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<Opacity> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<bool> >(++uuid);
        TheSerializableFactory::instance().register_class<GenericResult<std::vector<std::string> > >(++uuid);
    }
}




void peekabot::init()
{
    if( init_count == 0 )
    {
        register_actions();
        register_action_results();
        
        sockets::sockets_init();
    }

    ++init_count;
}


void peekabot::cleanup()
{
    if( init_count > 0 )
    {
        --init_count;

        if( init_count == 0 )
        {
            sockets::sockets_cleanup();
        
            TheSerializableFactory::instance().deregister_all();
        }
    }
}
