/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_LABEL_HH
#define __PEEKABOT_LABEL_HH


#include <string>
#include "SceneObject.hh"
#include "ScalableObject.hh"
#include "XMLHandler.hh"
#include "HandlerInformer.hh"


namespace peekabot
{
    class ScopedHandler;

    /**
     * \internal
     *
     * \brief A text label primitives, for showing orientable, scalable labels.
     */
    class Label : public SceneObject,
                  public ScalableObject
    {
    public:
        Label() throw();

        Label(ScopedHandler *handler) throw();
        
        virtual void accept(ObjectVisitor* visitor) throw();
        
        void set_text(const std::string &text) throw();

        const std::string &get_text() const throw();

        void set_alignment(TextAlignment alignment) throw();

        TextAlignment get_alignment() const throw();

        /// \name Signals
        /// @{

        typedef boost::signal<
            void (Label *label, const std::string &text)> OnTextSetSignal;
        
        OnTextSetSignal m_on_text_set;

        typedef boost::signal<
            void (Label *label, TextAlignment alignment)> OnAlignmentSetSignal;
        
        OnAlignmentSetSignal m_on_alignment_set;

        /// @}

    private:
        /** 
         * \brief Element start handler for XML parsing.
         */
        static void start_handler(
            const std::string &name,
            XMLHandler::AttributeMap &attributes,
            ScopedHandler *handler) throw();

        void text_start_handler(
            const std::string &name,
            XMLHandler::AttributeMap &attributes,
            ScopedHandler *handler) throw();

        void text_cdata_handler(
            const std::string &cdata,
            ScopedHandler *handler) throw();

        void align_start_handler(
            const std::string &name,
            XMLHandler::AttributeMap &attributes,
            ScopedHandler *handler) throw();

        void align_cdata_handler(
            const std::string &cdata,
            ScopedHandler *handler);

    private:
        static HandlerInformer ms_handler_informer;

        std::string m_text;

        TextAlignment m_alignment;
    };
}

#endif // __PEEKABOT_LABEL_HH
