/*
 * Copyright Anders Boberg 2006.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PEEKABOT_OBJECT_VISITOR_HH_INCLUDED
#define PEEKABOT_OBJECT_VISITOR_HH_INCLUDED

namespace peekabot
{
    class SceneObject;
    class CameraObject;
    class ModelObject;
    class SphereObject;
    class GroupObject;
    class CubeObject;
    class PolygonObject;
    class CylinderObject;
    class CircleObject;
    class PolyLineObject;
    class SensorObject;
    class PointSetObject;
    class Hinge;
    class Slider;
    class GridObject;
    class PointCloud;
    class LineCloud;
    class OccupancyGrid2D;
    class Label;


    /**
     * \internal
     *
     * \brief Interface for visitors of SceneObject derived classes.
     *
     * Each class that inherits from SceneObject should implement its own 
     * accept() method, the sole purpose of which is to call the visit() method 
     * of a visitor object implementing this interface and provide its \c this 
     * pointer as the parameter. This is a very convenient way of performing 
     * implicit typecasts.
     */
    class ObjectVisitor
    {
    public:
        virtual ~ObjectVisitor() {}

        /** \brief Visit method for CameraObject.
         */
        virtual void visit(CameraObject* camera_object) = 0;



        /** \brief Visit method for ModelObject.
         */
        virtual void visit(ModelObject* model_object) = 0;



        /** \brief Visit method for SphereObject.
         */
        virtual void visit(SphereObject* sphere_object) = 0;



        /** \brief Visit method for GroupObject.
         */
        virtual void visit(GroupObject* group_object) = 0;



        /** \brief Visit method for CubeObject.
         */
        virtual void visit(CubeObject* cube_object) = 0;


        /** \brief Visit method for PlaneObject.
         */
        virtual void visit(PolygonObject* polygon_object) = 0;



        /** \brief Visit method for CylinderObject.
         */
        virtual void visit(CylinderObject* cylinder_object) = 0;



        /** \brief Visit method for CircleObject.
         */
        virtual void visit(CircleObject* circle_object) = 0;

        /** \brief Visit method for PolyLineObject.
         */
        virtual void visit(PolyLineObject* poly_line_object) = 0;



        /** \brief Visit method for SensorObject.
         */
        virtual void visit(SensorObject* sensor_object) = 0;

        /** \brief Visit method for PointSetObject.
         */
        virtual void visit(PointSetObject* object) = 0;

        virtual void visit(Hinge* object) = 0;

        virtual void visit(Slider* object) = 0;

        virtual void visit(GridObject* object) = 0;

        virtual void visit(PointCloud* object) = 0;

        virtual void visit(LineCloud* object) = 0;

        virtual void visit(OccupancyGrid2D* object) = 0;

        virtual void visit(Label* object) = 0;
    };

}


#endif // PEEKABOT_OBJECT_VISITOR_HH_INCLUDED
