/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <boost/bind.hpp>
#include "ScopedHandler.hh"
#include "ScopedMap.hh"
#include "OccupancyGrid2D.hh"


using namespace peekabot;


/*HandlerInformer OccupancyGrid2D::ms_handler_informer(
    "occupancy_grid_2d", &OccupancyGrid2D::start_handler);*/


OccupancyGrid2D::OccupancyGrid2D(
    float cell_size, 
    const RGBColor &free_color,
    const RGBColor &occupied_color) throw()
    : SceneObject("occupancy_grid_2d"),
      m_cell_size(cell_size),
      m_free_color(free_color),
      m_occupied_color(occupied_color)
{
}


OccupancyGrid2D::OccupancyGrid2D(ScopedHandler *handler)
    : SceneObject("occupancy_grid_2d", handler),
      ScalableObject(handler)
{
    // Register the file tag start handler
    /*handler->add_start_handler(
        "file",        
        boost::bind(&OccupancyGrid2D::file_start_handler,
        this, _1, _2, _3));*/
}


void OccupancyGrid2D::accept(ObjectVisitor *visitor) throw()
{
    visitor->visit(this);
}


float OccupancyGrid2D::get_cell_size() const throw()
{
    return m_cell_size;
}


const RGBColor &OccupancyGrid2D::get_free_color() const throw()
{
    return m_free_color;
}


const RGBColor &OccupancyGrid2D::get_occupied_color() const throw()
{
    return m_occupied_color;
}


void OccupancyGrid2D::set_cell(const Vector2f &x, float belief) throw()
{
    std::vector<std::pair<Vector2f, float> > cells;
    cells.push_back(std::make_pair(x, belief));
    set_cells(cells);
}


void OccupancyGrid2D::set_cells(const std::vector<std::pair<Vector2f, float> > &cells) throw()
{
    m_on_cells_set(this, cells);
}


void OccupancyGrid2D::start_handler(
    const std::string &name,
    XMLHandler::AttributeMap &attributes,
    ScopedHandler *handler) throw()
{
    // Creating a scene object by passing a ScopedHandler will cause it to
    // enter a new scope with all registered tag start handlers plus any
    // specific handlers for SceneObject properties which are registered
    // by the parent constructor.
    SceneObject* tmp = new OccupancyGrid2D(handler);
    
    // Set the new object as the current object
    ScopedMap & variables = handler->get_variables();
    variables.push_variable("current_object", tmp);
}
