/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_POINT_SET_OBJECT_HH
#define __PEEKABOT_POINT_SET_OBJECT_HH

#include <queue>

#include "Types.hh"
#include "Vector.hh"
#include "SceneObject.hh"

namespace peekabot
{

    /**
     * \internal
     *
     * \brief Class representing cameras in the scene.
     *
     * Since the Renderer renders the scene as if filmed from a PointSetObject, 
     * the position and rotation of the camera specify how the scene should 
     * be rendered. Other properties which affect the rendition, such as the 
     * field of view, the fill model and the view mode, are also stored in a 
     * PointSetObject.
     *
     */
    class PointSetObject : public SceneObject
    {
    public:
        std::queue<std::pair<Vector3ru, RGBColor> > m_new_points;

        PointSetObject() throw();

        virtual ~PointSetObject() throw();

        void add_point(const Vector3ru &p, const RGBColor &color) throw();


        /** \brief Method for accepting \c ObjectVisitor objects.
         *
         * This method performs an automatic type cast by calling the 
         * \c visit() method of the visitor, allowing it to perform 
         * type-dependent task.
         */
        virtual void accept(ObjectVisitor* visitor) throw();
    };

}


#endif // __PEEKABOT_POINT_SET_OBJECT_HH
