/*
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PEEKABOT_PSEUDONYM_CATALOG_HH_INCLUDED
#define PEEKABOT_PSEUDONYM_CATALOG_HH_INCLUDED


#include <set>
#include <map>
#include <stdexcept>
#include "Types.hh"


namespace peekabot
{
    class SceneObject;

    namespace atl
    {
        class ClientConnection;
    }

    class PseudonymCatalog
    {
    public:
        PseudonymCatalog(SceneObject *root) throw();

        ~PseudonymCatalog();
        
        ObjectID translate_pseudonym(SourceID source, ObjectID id) const throw();

        void add_pseudonym(SourceID source, ObjectID id, ObjectID real_id) 
            throw(std::runtime_error);

        // id = pseudonym id
        void remove_pseudonym(SourceID source, ObjectID id) 
            throw(std::runtime_error);


    private:
        void remove_pseudonyms_for_object(ObjectID real_id) throw();

        void remove_pseudonyms_for_client(SourceID source) throw();


        void on_detached(SceneObject *object) throw();

        void on_child_attached(SceneObject *object) throw();

        void on_deleted(SceneObject *object) throw();

        
        void on_client_connected(atl::ClientConnection *conn) throw();

    private:
        typedef std::map<ObjectID, ObjectID> PseudoToReal;
        typedef std::multimap<ObjectID, ObjectID> RealToPseudo;

        struct PseudoBimap
        {
            PseudoToReal m_pseudo_to_real;
            RealToPseudo m_real_to_pseudo;
        };

        typedef std::map<SourceID, PseudoBimap> SourceMap;
        SourceMap m_source_map;


        std::set<SceneObject *> m_tracked;
    };
}


#endif // PEEKABOT_PSEUDONYM_CATALOG_HH_INCLUDED
