/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <sstream>
#include <boost/bind.hpp>

#include "ScalableObject.hh"
#include "ScopedHandler.hh"

using namespace peekabot;


ScalableObject::ScalableObject() throw()
    : m_x_scale(1), m_y_scale(1), m_z_scale(1)
{
}


real_unit ScalableObject::get_x_scale() const throw()
{
    return m_x_scale;
}

real_unit ScalableObject::get_y_scale() const throw()
{
    return m_y_scale;
}


real_unit ScalableObject::get_z_scale() const throw()
{
    return m_z_scale;
}


void ScalableObject::set_scale(
    real_unit x_scale, real_unit y_scale, real_unit z_scale) throw()
{
    // First check that the object isn't read-only
    /*if(is_read_only())
      throw AccessDenied();*/

    //m_on_set_scale(this);

    m_x_scale = x_scale;
    m_y_scale = y_scale;
    m_z_scale = z_scale;

    m_on_scale_set(this, x_scale, y_scale, z_scale);
}


void ScalableObject::set_x_scale(real_unit x_scale) throw()
{
    set_scale(x_scale, m_y_scale, m_z_scale);
}


void ScalableObject::set_y_scale(real_unit y_scale) throw()
{
    set_scale(m_x_scale, y_scale, m_z_scale);
}


void ScalableObject::set_z_scale(real_unit z_scale) throw()
{
    set_scale(m_x_scale, m_y_scale, z_scale);
}

ScalableObject::ScalableObject(ScopedHandler *handler)
    : m_x_scale(1), m_y_scale(1), m_z_scale(1)
{
    // Register the <scale> tag start handler
    handler->add_start_handler(
        "scale",        
        boost::bind(&ScalableObject::scale_start_handler, this, _1, _2, _3));    
}

void ScalableObject::scale_start_handler(
    const std::string & name,
    ScopedHandler::AttributeMap &attributes,
    ScopedHandler *handler) throw()
{
    ScopedHandler::TagScope scope;
    
    scope.cdata_functor = boost::bind(
        &ScalableObject::scale_cdata_handler, this, _1, _2);
    
    handler->enter_scope(scope);    
}

void ScalableObject::scale_cdata_handler(
    const std::string &cdata,
    ScopedHandler *handler) throw()
{
    // Read scaling scalars
    std::stringstream s(cdata);
    real_unit x, y, z;
    s >> x >> y >> z;

    set_x_scale(x);
    set_y_scale(y);
    set_z_scale(z);
}

