/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <boost/bind.hpp>

#include "Sensor.hh"


using namespace peekabot;


//
// ----- XML handler methods ------
//


void Sensor::params_start_handler(
    const std::string &name,
    XMLHandler::AttributeMap &attributes,
    ScopedHandler *handler)
{
    ScopedHandler::TagScope scope;

    // Add tag handlers to the scope
    scope.start_functors["bool"] = boost::bind(
        &Sensor::bool_start_handler, this, _1, _2, _3);

    scope.start_functors["int"] = boost::bind(
        &Sensor::int_start_handler, this, _1, _2, _3);

    scope.start_functors["float"] = boost::bind(
        &Sensor::float_start_handler, this, _1, _2, _3);

    scope.start_functors["string"] = boost::bind(
        &Sensor::string_start_handler, this, _1, _2, _3);

    scope.start_functors["color"] = boost::bind(
        &Sensor::color_start_handler, this, _1, _2, _3);

    handler->enter_scope(scope);
}


void Sensor::bool_start_handler(
    const std::string &name,
    XMLHandler::AttributeMap &attributes,
    ScopedHandler *handler)
{
    // Extract the param name and overwrite the param_name variable
    ScopedHandler::TagScope scope;
    scope.cdata_functor = boost::bind(
        &Sensor::bool_cdata_handler, this, _1, _2, 
        attributes["name"]);
    handler->enter_scope(scope);
}


void Sensor::bool_cdata_handler(
    const std::string &cdata, ScopedHandler *handler,
    const std::string &param_name)
{  
    if( cdata == "false" || cdata == "0" )
        set_param<bool>(param_name, false);
    else
        set_param<bool>(param_name, true);
}


void Sensor::int_start_handler(
    const std::string &name,
    XMLHandler::AttributeMap &attributes,
    ScopedHandler *handler)
{
    // Extract the param name and overwrite the param_name variable
    ScopedHandler::TagScope scope;
    scope.cdata_functor = boost::bind(
        &Sensor::int_cdata_handler, this, _1, _2, 
        attributes["name"]);
    handler->enter_scope(scope);
}


void Sensor::int_cdata_handler(
    const std::string &cdata, ScopedHandler *handler,
    const std::string &param_name)
{      
    set_param<int>(param_name, atoi(cdata.c_str()));
}


void Sensor::float_start_handler(
    const std::string &name,
    XMLHandler::AttributeMap &attributes,
    ScopedHandler *handler)
{
    // Extract the param name and overwrite the param_name variable
    ScopedHandler::TagScope scope;
    scope.cdata_functor = boost::bind(
        &Sensor::float_cdata_handler, this, _1, _2, 
        attributes["name"]);
    handler->enter_scope(scope);
}


void Sensor::float_cdata_handler(
    const std::string &cdata, ScopedHandler *handler,
    const std::string &param_name)
{    
    set_param<float>(param_name, static_cast<float>(atof(cdata.c_str())));
}

            
void Sensor::string_start_handler(
    const std::string &name,
    XMLHandler::AttributeMap &attributes,
    ScopedHandler *handler)
{
    // Extract the param name and overwrite the param_name variable
    ScopedHandler::TagScope scope;
    scope.cdata_functor = boost::bind(
        &Sensor::string_cdata_handler, this, _1, _2, 
        attributes["name"]);
    handler->enter_scope(scope);
}


void Sensor::string_cdata_handler(
    const std::string &cdata, ScopedHandler *handler,
    const std::string &param_name)
{
    set_param<std::string>(param_name, cdata);
}


void Sensor::color_start_handler(
    const std::string &name,
    XMLHandler::AttributeMap &attributes,
    ScopedHandler *handler)
{
    const std::string &param_name = attributes["name"];

    float r = static_cast<float>(atof( attributes["r"].c_str() ));
    float g = static_cast<float>(atof( attributes["g"].c_str() ));
    float b = static_cast<float>(atof( attributes["b"].c_str() ));

    set_param<RGBColor>(param_name, RGBColor(r, g, b));

    handler->enter_scope();
}
