/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_SENSOR_OBJECT_HH
#define __PEEKABOT_SENSOR_OBJECT_HH


#include <string>
#include <boost/shared_ptr.hpp>

#include "SceneObject.hh"
#include "serialization/DeserializationInterface.hh"
#include "renderer/IDTypes.hh"
#include "HandlerInformer.hh"
#include "Sensor.hh"


namespace peekabot
{

    /**
     * \internal
     *
     * \brief Class representing robot sensors.
     *
     * This is the class that is referred to when clients send sensor data.
     */
    class SensorObject : public SceneObject
    {
    public:
        SensorObject(const std::string &sensor_type);

        SensorObject(
            ScopedHandler *handler, 
            const std::string &sensor_type,
            const std::string &params_template);

        virtual ~SensorObject() throw();
        
        /**
         * \brief Method for accepting \c ObjectVisitor objects.
         *
         * This method performs an automatic type cast by calling the \c visit()
         * method of the visitor, allowing it to perform type-dependent task.
         */
        virtual void accept(ObjectVisitor* visitor) throw();

        const std::string &get_sensor_type() const throw();

        void set_sensor_data(DeserializationInterface &sensor_data);

    private:
        static void start_handler(
            const std::string &name,
            XMLHandler::AttributeMap &attributes,
            ScopedHandler *handler);

        /**
         * \brief Called when the \c sensor end tag is encountered. Inititates
         * sensor interpreter parameter validation.
         */
        void end_handler(const std::string &name, ScopedHandler* handler);

    private:
        std::string m_sensor_type;

        boost::shared_ptr<Sensor> m_data_interpreter;

        static HandlerInformer ms_handler_informer;
    };


}

#endif // __PEEKABOT_SENSOR_OBJECT_HH
