/*
 * Copyright Staffan Gimåker 2006-2007, 2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PEEKABOT_SERVER_ACTUATOR_HH_INCLUDED
#define PEEKABOT_SERVER_ACTUATOR_HH_INCLUDED


#include <queue>
#include <boost/thread/thread.hpp>
#include <boost/thread/condition.hpp>
#include <boost/thread/recursive_mutex.hpp>

#include "Types.hh"
#include "Action.hh"
#include "Singleton.hh"
#include "Configuration.hh"


namespace peekabot
{
    class PseudonymCatalog;



    class ServerActuator
    {
        PseudonymCatalog *m_pseudonyms;

        volatile bool m_stop_signal;

        // 0 when not running
        boost::thread *m_thread;

        int m_action_batch_size;

    private:
        std::queue<std::pair<Action *, SourceID> > m_queue;

        boost::recursive_mutex m_mutex;

        boost::condition m_queue_cond;


        // Returns true if any GUI actions were processed
        bool process_actions(bool &do_redraw) throw();

        void run() throw();

        void handle_inbound_action(Action *action, SourceID source) throw();


    public:
        ServerActuator() throw();

        virtual ~ServerActuator() throw();


        bool start() throw();

        bool stop() throw();

        bool is_running() throw();
    };


    typedef Singleton<ServerActuator> TheServerActuator;
}


#endif // PEEKABOT_SERVER_ACTUATOR_HH_INCLUDED
