/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <fstream>
#include <algorithm>

#include "SnapshotWriter.hh"
#include "MessageHub.hh"
#include "Bytesex.hh"
#include <png++/png.hpp>


using namespace peekabot;


SnapshotWriter::SnapshotWriter(
    const std::string &filename)
    : m_filename(filename)
{
}


SnapshotWriter::~SnapshotWriter() throw()
{
}


void SnapshotWriter::collect_frame(int w, int h, const uint8_t *rgb_data) throw()
{
    if( w < 1 || h < 1 )
        return;

    assert( rgb_data != 0 );

    try
    {
        png::image< png::rgb_pixel > image(w, h);

        for( int y = 0; y < h; ++y )
        {
            for( int x = 0; x < w; ++x )
            {
                image[h-y-1][x] = png::rgb_pixel(
                    rgb_data[3*(w*y + x)+0], 
                    rgb_data[3*(w*y + x)+1], 
                    rgb_data[3*(w*y + x)+2]);
            }
        }

        image.write(m_filename.c_str());

        TheMessageHub::instance().publish(
            INFO_MESSAGE,
            "Snapshot saved to " + m_filename);
    }
    catch(std::exception &e)
    {
        TheMessageHub::instance().publish(
            ERROR_MESSAGE,
            "Failed to save snapshot: %s", e.what());
    }
    catch(...)
    {
        TheMessageHub::instance().publish(
            ERROR_MESSAGE,
            "Failed to save snapshot: unknown exception caught");
    }
}
