/*
 * Copyright Staffan Gimåker 2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PEEKABOT_UPLOAD_CACHE_HH_INCLUDED
#define PEEKABOT_UPLOAD_CACHE_HH_INCLUDED


#include <map>
#include <string>
#include <boost/utility.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/random/mersenne_twister.hpp>

#include "Singleton.hh"


namespace peekabot
{
    class UploadCache : public boost::noncopyable
    {
    public:
        UploadCache();

        ~UploadCache();

        void add(const std::string &key, const void *data, std::size_t n);

        void remove(const std::string &key);

        bool has_key(const std::string &key) const;

        void clear();

        boost::filesystem::path expand_path(
            const std::string &path) const;

    private:
        static boost::filesystem::path upload_directory();

        std::string generate_filename(const std::string &key) const;

    private:
        typedef std::map<std::string, std::string> KeyFilenameMap;

        KeyFilenameMap m_files;

        mutable boost::mt19937 m_rng;
    };

    typedef Singleton<UploadCache, singleton::StaticSingleton> TheUploadCache;
}


#endif // PEEKABOT_UPLOAD_CACHE_HH_INCLUDED
