/*
 * Copyright Staffan Gimåker 2007-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_ACTION_RESULT_CONTAINER_HH_INCLUDED
#define PEEKABOT_ACTION_RESULT_CONTAINER_HH_INCLUDED


#include <string>
#include <stdexcept>
#include <boost/shared_ptr.hpp>
#include "../Types.hh"
#include "../Action.hh"
#include "../ActionResult.hh"


namespace peekabot
{
    /**
     * \brief Stores an \c ActionResult, of any type that can be 
     * serialized, deserialized and constructed by the the ActionResultFactory.
     */
    class ActionResultContainer : public Action
    {
        PEEKABOT_SERIALIZABLE

    public:
        ActionResultContainer(
            uint32_t request_id, boost::shared_ptr<ActionResult> result) throw();

        virtual ~ActionResultContainer() throw();

        virtual Action *clone() const;

        virtual void execute(
            ClientExecutionContext *context) 
            const throw(std::exception);

        boost::shared_ptr<ActionResult> get_action_result() throw();
       
    private:
        ActionResultContainer() throw();

        ActionResultContainer(const ActionResultContainer &x) throw();

    protected:
        virtual void save(SerializationInterface &ar) const;

        virtual void load(DeserializationInterface &ar);

    private:
        uint32_t m_request_id;

        boost::shared_ptr<ActionResult> m_action_result;
    };
}


// PEEKABOT_REGISTER_SERIALIZABLE 30


#endif // PEEKABOT_ACTION_RESULT_CONTAINER_HH_INCLUDED
