/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_ADD_GENERIC_HH_INCLUDED
#define PEEKABOT_ADD_GENERIC_HH_INCLUDED


#include <string>
#include <stdexcept>
#include "../Types.hh"
#include "../Action.hh"
#include "../PathIdentifier.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#endif


namespace peekabot
{
    template<class ObjectType>
    class AddGeneric : public Action
    {
        PEEKABOT_SERIALIZABLE

    public:
        AddGeneric(const PathIdentifier &path_ident,
                   NameConflictPolicy conflict_policy,
                   bool register_pseudonym,
                   ObjectID pseudonym_id) throw()
            : m_path_ident(path_ident),
              m_conflict_policy(conflict_policy),
              m_register_pseudonym(register_pseudonym),
              m_pseudonym_id(pseudonym_id)
        {
        }

        virtual ~AddGeneric() throw() {}

        virtual Action *clone() const
        {
            return new AddGeneric(m_path_ident, m_conflict_policy,
                                  m_register_pseudonym, m_pseudonym_id);
        }

        virtual void execute(
            ServerExecutionContext *context)
            const throw(std::exception)
        {
#ifdef __PEEKABOT_SERVER
            SceneObject *parent = m_path_ident.get_parent(context);
            std::string name = m_path_ident.get_leaf_name(context);

            // Apply ALIAS_ON_CONFLICT policy if applicable
            if(m_conflict_policy == ALIAS_ON_CONFLICT &&
               context->aliasing_policy_applied<ObjectType>(parent, name,
                                                            m_pseudonym_id,
                                                            m_register_pseudonym))
                return;

            ObjectType *ptr = new ObjectType();

            try
            {
                ptr->set_name(name);
                parent->attach(ptr, m_conflict_policy);
            }
            catch(...)
            {
                delete ptr;
                throw;
            }

            if( m_register_pseudonym )
                context->register_pseudonym(ptr->get_object_id(), m_pseudonym_id);
#endif
        }

    private:
        AddGeneric() throw() {}

    protected:
        virtual void save(SerializationInterface &ar) const
        {
            ar << m_path_ident << m_conflict_policy
               << m_register_pseudonym << m_pseudonym_id;
        }

        virtual void load(DeserializationInterface &ar)
        {
            ar >> m_path_ident >> m_conflict_policy
               >> m_register_pseudonym >> m_pseudonym_id;
        }

    private:
        PathIdentifier m_path_ident;
        NameConflictPolicy m_conflict_policy;
        bool m_register_pseudonym;
        ObjectID m_pseudonym_id;
    };
}


#endif // PEEKABOT_ADD_GENERIC_HH_INCLUDED
