/*
 * Copyright Staffan Gimåker 2007-2008.
 * Copyright Anders Boberg 2007.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "AddModel.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../ModelObject.hh"
#endif


using namespace peekabot;


AddModel::AddModel() throw()
{
}


AddModel::AddModel(
    const PathIdentifier &path_ident,
    const std::string &filename,
    NameConflictPolicy conflict_policy,
    bool register_pseudonym,
    ObjectID pseudonym_id) throw()
    : m_path_ident(path_ident),
      m_filename(filename),
      m_conflict_policy(conflict_policy),
      m_register_pseudonym(register_pseudonym),
      m_pseudonym_id(pseudonym_id)
{
}


AddModel::~AddModel() throw()
{
}


Action *AddModel::clone() const
{
    return new AddModel(
        m_path_ident, m_filename, m_conflict_policy,
        m_register_pseudonym, m_pseudonym_id);
}


void AddModel::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *parent = m_path_ident.get_parent(context);
    std::string name = m_path_ident.get_leaf_name(context);

    // Apply ALIAS_ON_CONFLICT policy if applicable
    if(m_conflict_policy == ALIAS_ON_CONFLICT &&
       context->aliasing_policy_applied<ModelObject>(parent, name,
                                                     m_pseudonym_id,
                                                     m_register_pseudonym))
        return;

    ModelObject *ptr = new ModelObject(m_filename);

    try
    {
        ptr->set_name(name);
        parent->attach(ptr, m_conflict_policy);
    }
    catch(...)
    {
        delete ptr;
        throw;
    }

    if( m_register_pseudonym )
        context->register_pseudonym(ptr->get_object_id(), m_pseudonym_id);
#endif
}


void AddModel::save(SerializationInterface &ar) const
{
    ar << m_path_ident << m_filename << m_conflict_policy 
       << m_register_pseudonym << m_pseudonym_id;
}

void AddModel::load(DeserializationInterface &ar)
{
    ar >> m_path_ident >> m_filename >> m_conflict_policy 
       >> m_register_pseudonym >> m_pseudonym_id;
}
