/*
 * Copyright Staffan Gimåker 2007-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_ADD_MODEL_HH_INCLUDED
#define PEEKABOT_ADD_MODEL_HH_INCLUDED


#include <string>
#include <exception>
#include "../Types.hh"
#include "../Action.hh"
#include "../PathIdentifier.hh"


namespace peekabot
{
    class AddModel : public Action
    {
        PEEKABOT_SERIALIZABLE

    public:
        AddModel(
            const PathIdentifier &path_ident,
            const std::string &filename,
            NameConflictPolicy conflict_policy,
            bool register_pseudonym,
            ObjectID pseudonym_id) throw();

        virtual ~AddModel() throw();

        virtual Action *clone() const;

        virtual void execute(
            ServerExecutionContext *context)
            const throw(std::exception);

    private:
        AddModel() throw();

    protected:
        virtual void save(SerializationInterface &ar) const;

        virtual void load(DeserializationInterface &ar);

    private:
        PathIdentifier m_path_ident;
        std::string m_filename;
        NameConflictPolicy m_conflict_policy;
        bool m_register_pseudonym;
        ObjectID m_pseudonym_id;
    };
}


// PEEKABOT_REGISTER_SERIALIZABLE 41


#endif // PEEKABOT_ADD_MODEL_HH_INCLUDED
