/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "AddOccupancyGrid2D.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../OccupancyGrid2D.hh"
#endif


using namespace peekabot;


AddOccupancyGrid2D::AddOccupancyGrid2D() throw()
{
}


AddOccupancyGrid2D::AddOccupancyGrid2D(
    const PathIdentifier &path_ident,
    float cell_size,
    const RGBColor &unoccupied_color,
    const RGBColor &occupied_color,
    NameConflictPolicy conflict_policy,
    bool register_pseudonym,
    ObjectID pseudonym_id) throw()
    : m_path_ident(path_ident),
      m_cell_size(cell_size),
      m_unoccupied_color(unoccupied_color),
      m_occupied_color(occupied_color),
      m_conflict_policy(conflict_policy),
      m_register_pseudonym(register_pseudonym),
      m_pseudonym_id(pseudonym_id)
{
}


AddOccupancyGrid2D::~AddOccupancyGrid2D() throw()
{
}


Action *AddOccupancyGrid2D::clone() const
{
    return new AddOccupancyGrid2D(
        m_path_ident, m_cell_size, m_unoccupied_color, m_occupied_color,
        m_conflict_policy, m_register_pseudonym, m_pseudonym_id);
}


void AddOccupancyGrid2D::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *parent = m_path_ident.get_parent(context);
    std::string name = m_path_ident.get_leaf_name(context);

    // Apply ALIAS_ON_CONFLICT policy if applicable
    if(m_conflict_policy == ALIAS_ON_CONFLICT &&
       context->aliasing_policy_applied<OccupancyGrid2D>(parent, name,
                                                         m_pseudonym_id,
                                                         m_register_pseudonym))
        return;
    
    OccupancyGrid2D *ptr = new OccupancyGrid2D(
        m_cell_size, m_unoccupied_color, m_occupied_color);

    try
    {
        ptr->set_name(name);
        parent->attach(ptr, m_conflict_policy);
    }
    catch(...)
    {
        delete ptr;
        throw;
    }

    if( m_register_pseudonym )
        context->register_pseudonym(ptr->get_object_id(), m_pseudonym_id);
#endif
}


void AddOccupancyGrid2D::save(SerializationInterface &ar) const
{
    ar << m_path_ident << m_conflict_policy 
       << m_register_pseudonym << m_pseudonym_id;
    ar << m_cell_size << m_unoccupied_color << m_occupied_color;
}


void AddOccupancyGrid2D::load(DeserializationInterface &ar)
{
    ar >> m_path_ident >> m_conflict_policy 
       >> m_register_pseudonym >> m_pseudonym_id;
    ar >> m_cell_size >> m_unoccupied_color >> m_occupied_color;
}
