/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_ADD_OCCUPANCY_GRID_2D_HH_INCLUDED
#define PEEKABOT_ADD_OCCUPANCY_GRID_2D_HH_INCLUDED


#include <string>
#include <exception>
#include <vector>
#include "../Types.hh"
#include "../Action.hh"
#include "../Vector.hh"
#include "../PathIdentifier.hh"


namespace peekabot
{
    class AddOccupancyGrid2D : public Action
    {
        PEEKABOT_SERIALIZABLE

    public:
        AddOccupancyGrid2D(
            const PathIdentifier &path_ident,
            float cell_size,
            const RGBColor &unoccupied_color,
            const RGBColor &occupied_color,
            NameConflictPolicy conflict_policy,
            bool register_pseudonym,
            ObjectID pseudonym_id) throw();

        virtual ~AddOccupancyGrid2D() throw();

        virtual Action *clone() const;

        virtual void execute(
            ServerExecutionContext *context)
            const throw(std::exception);

    private:
        AddOccupancyGrid2D() throw();

    protected:
        virtual void save(SerializationInterface &ar) const;

        virtual void load(DeserializationInterface &ar);

    private:
        PathIdentifier m_path_ident;
        float m_cell_size;
        RGBColor m_unoccupied_color;
        RGBColor m_occupied_color;
        NameConflictPolicy m_conflict_policy;
        bool m_register_pseudonym;
        ObjectID m_pseudonym_id;
    };
}


// PEEKABOT_REGISTER_SERIALIZABLE 56


#endif // PEEKABOT_ADD_OCCUPANCY_GRID_2D_HH_INCLUDED
