/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_ASSIGN_HH_INCLUDED
#define PEEKABOT_ASSIGN_HH_INCLUDED


#include <string>
#include <stdexcept>

#include "../Types.hh"
#include "../Action.hh"
#include "../PathIdentifier.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#endif


namespace peekabot
{
    template<class ObjectType>
    class Assign : public Action
    {
        PEEKABOT_SERIALIZABLE

    public:
        Assign(
            const PathIdentifier &path_ident,
            ObjectID pseudonym_id) throw()
            : m_path_ident(path_ident),
              m_pseudonym_id(pseudonym_id) {}

        virtual ~Assign() throw() {}

        virtual Action *clone() const
        {
            return new Assign(m_path_ident, m_pseudonym_id);
        }

        virtual void execute(
            ServerExecutionContext *context)
            const throw(std::exception)
        {
#ifdef __PEEKABOT_SERVER
            SceneObject *obj = m_path_ident.get_leaf(context);

            // Check type
            if( !dynamic_cast<ObjectType *>(obj) )
                throw std::runtime_error(
                    "Type assertion failed");

            context->register_pseudonym(obj->get_object_id(), m_pseudonym_id);
#endif
        }

    private:
        Assign() throw() {}

    protected:
        virtual void save(SerializationInterface &ar) const
        {
            ar << m_path_ident << m_pseudonym_id;
        }

        virtual void load(DeserializationInterface &ar)
        {
            ar >> m_path_ident >> m_pseudonym_id;
        }

    private:
        PathIdentifier m_path_ident;
        ObjectID m_pseudonym_id;
    };
}


#endif // PEEKABOT_ASSIGN_HH_INCLUDED
