/*
 * Copyright Staffan Gimåker 2007-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "GetTransformation.hh"
#include "../MatrixIO.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../GenericResult.hh"
#  include "ActionResultContainer.hh"
#endif


using namespace peekabot;


GetTransformation::GetTransformation() throw()
{
}


GetTransformation::GetTransformation(
    uint32_t request_id, 
    ObjectID target, 
    CoordinateSystem coord_sys) throw()
    : m_request_id(request_id),
      m_target(target),
      m_coord_sys(coord_sys)
{
}


GetTransformation::GetTransformation(const GetTransformation &x) throw()    
    : m_request_id(x.m_request_id),
      m_target(x.m_target),
      m_coord_sys(x.m_coord_sys)
{
}


GetTransformation::~GetTransformation() throw()
{
}


Action *GetTransformation::clone() const
{
    return new GetTransformation(*this);
}


void GetTransformation::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target);

    if( !ptr )
        throw std::runtime_error(
            "Failed to get transformation, target object not found.");

    Matrix4ru transformation;

    switch( m_coord_sys )
    {
        case PARENT_COORDINATES:
            transformation = ptr->get_transformation();
            break;

        case WORLD_COORDINATES:
            transformation = ptr->get_mtow();
            break;

        default:
            throw std::runtime_error(
                "Failed to get transformation: unsupported coordinate system");
            break;
    }

    context->post_response( 
        new ActionResultContainer(
            m_request_id,
            boost::shared_ptr<ActionResult>(
                new GenericResult<Matrix4ru>(transformation) ) ) );
#endif
}


void GetTransformation::save(SerializationInterface &ar) const
{
    ar << m_request_id << m_target << m_coord_sys;
}

void GetTransformation::load(DeserializationInterface &ar)
{
    ar >> m_request_id >> m_target >> m_coord_sys;
}
