/*
 * Copyright Anders Boberg 2006-2007.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_MOVE_OBJECT_RELATIVE_HH_INCLUDED
#define PEEKABOT_MOVE_OBJECT_RELATIVE_HH_INCLUDED


#include "../Types.hh"
#include "../Action.hh"
#include "../Matrix.hh"


namespace peekabot
{
    class MoveObjectRelative : public Action
    {
        PEEKABOT_SERIALIZABLE

    public:
        MoveObjectRelative(
            ObjectID target, real_unit x, real_unit y, real_unit z,
            transformation_target::CoordinateSystem system,
            ObjectID camera = 0) throw();

        virtual Action *clone() const;

        virtual void execute(
            ServerExecutionContext *context) 
            const throw(std::exception);

    private:
        MoveObjectRelative() throw();

    protected:
        virtual void save(SerializationInterface &ar) const;

        virtual void load(DeserializationInterface &ar);

    private:
        // The target object to move
        ObjectID m_target;

        // The amount to move along each axis
        real_unit m_x, m_y, m_z;
        
        // The coordinate system to use
        transformation_target::CoordinateSystem m_system;
        
        // The camera in whose view plane the transformation takes place
        ObjectID m_camera;
    };
}


// PEEKABOT_REGISTER_SERIALIZABLE 4


#endif // PEEKABOT_MOVE_OBJECT_RELATIVE_HH_INCLUDED
