/*
 * Copyright Anders Boberg 2006.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>

#include "PanCamera.hh"
#include "../Matrix.hh"
#include "../Vector.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../SceneObject.hh"
#  include "../CameraObject.hh"
#endif


using namespace peekabot;


PanCamera::PanCamera() throw()
{
}


PanCamera::PanCamera(
    ObjectID target, real_unit delta_x, real_unit delta_y) throw()
    : m_target(target),
      m_delta_x(delta_x), m_delta_y(delta_y)
{
}


Action *PanCamera::clone() const
{
    return new PanCamera(m_target, m_delta_x, m_delta_y);
}


void PanCamera::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    CameraObject *ptr = dynamic_cast<CameraObject *>(
        context->get_object(m_target));

    if( !ptr )
        throw std::runtime_error(
            "Object is not a camera or wasn't found.");
    
    Matrix4ru trans_matrix = ptr->get_transformation();

    trans_matrix(3) -= (m_delta_x * ptr->get_fov() *
                        -trans_matrix(1) - m_delta_y *
                        ptr->get_fov() * trans_matrix(2));
       
    ptr->set_transformation(trans_matrix);
#endif
}


void PanCamera::save(SerializationInterface &ar) const
{
    ar << m_target << m_delta_x << m_delta_y;
}

void PanCamera::load(DeserializationInterface &ar)
{
    ar >> m_target >> m_delta_x >> m_delta_y;
}
