/*
 * Copyright Anders Boberg 2006-2007.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_ROTATE_OBJECT_HH_INCLUDED
#define PEEKABOT_ROTATE_OBJECT_HH_INCLUDED


#include "../Types.hh"
#include "../Action.hh"
#include "../Matrix.hh"
#include "../Vector.hh"


namespace peekabot
{
    class RotateObject : public Action
    {
        PEEKABOT_SERIALIZABLE

    public:
        RotateObject(
            ObjectID target, real_unit x, real_unit y, real_unit z,
            transformation_target::CoordinateSystem axis_system,
            const Vector4ru &center_point,
            transformation_target::CoordinateSystem pivot_system) throw();

        virtual Action *clone() const;

        virtual void execute(
            ServerExecutionContext *context) 
            const throw(std::exception);

    private:
        RotateObject() throw();

    protected:
        virtual void save(SerializationInterface &ar) const;

        virtual void load(DeserializationInterface &ar);

    private:
        // The target object to move
        ObjectID m_target;

        // The amount to move along each axis
        real_unit m_x, m_y, m_z;
        
        // The coordinate system to use for axes
        transformation_target::CoordinateSystem m_axis_system;
     
        // The center point for the rotation
        Vector4ru m_center_point;

        // The coordinate system for the pivot
        transformation_target::CoordinateSystem m_pivot_system;
    };
}


// PEEKABOT_REGISTER_SERIALIZABLE 9


#endif // PEEKABOT_ROTATE_OBJECT_HH_INCLUDED
