/*
 * Copyright Anders Boberg 2006.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetColor.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#endif


using namespace peekabot;


SetColor::SetColor() throw()
{
}


SetColor::SetColor(ObjectID target_id,
                   RGBColor color, bool recursive) throw()
    : m_target_id(target_id),
      m_color(color),
      m_recursive(recursive)
{
}


SetColor::~SetColor() throw()
{
}


Action *SetColor::clone() const
{
    return new SetColor(m_target_id, m_color, m_recursive);
}


void SetColor::execute(
    ServerExecutionContext* context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target_id);

    if( !ptr )
        throw std::runtime_error(
            "Failed to set color, target object not found");

    ptr->set_color(m_color, m_recursive);
#endif
}


void SetColor::save(SerializationInterface &ar) const
{
    ar << m_target_id << m_color << m_recursive;
}

void SetColor::load(DeserializationInterface &ar)
{
    ar >> m_target_id >> m_color >> m_recursive;
}
