/*
 * Copyright Anders Boberg 2006.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetFOV.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../SceneObject.hh"
#  include "../CameraObject.hh"
#endif


using namespace peekabot;


SetFOV::SetFOV() throw()
{
}


SetFOV::SetFOV(
    ObjectID target, real_unit fov) throw()
    : m_target(target),
      m_fov(fov)
{
}


Action *SetFOV::clone() const
{
    return new SetFOV(m_target, m_fov);
}


void SetFOV::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    CameraObject *ptr = dynamic_cast<CameraObject *>(
        context->get_object(m_target));

    if( !ptr )
        throw std::runtime_error(
            "Failed to set FOV - object not found, or of non-camera type");

    if( m_fov <= 0 || m_fov >= 180 )
        throw std::domain_error(
            "Failed to set FOV - FOV must be in the range (0,180)");
    
    if(m_fov < ptr->get_fov() - 0.0001 || m_fov > ptr->get_fov() + 0.0001)
        ptr->set_fov(m_fov);
#endif
}


void SetFOV::save(SerializationInterface &ar) const
{
    ar << m_target << m_fov;
}

void SetFOV::load(DeserializationInterface &ar)
{
    ar >> m_target >> m_fov;
}
