/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetGridCentralAngle.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../GridObject.hh"
#endif


using namespace peekabot;


SetGridCentralAngle::SetGridCentralAngle() throw()
{
}


SetGridCentralAngle::SetGridCentralAngle(
    ObjectID target_id, float central_angle) throw()
    : m_target_id(target_id),
      m_central_angle(central_angle)
{
}


SetGridCentralAngle::~SetGridCentralAngle() throw()
{
}


Action *SetGridCentralAngle::clone() const
{
    return new SetGridCentralAngle(m_target_id, m_central_angle);
}


void SetGridCentralAngle::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target_id);

    if( !ptr )
        throw std::runtime_error("Target object not found");

    GridObject *grid_ptr = dynamic_cast<GridObject *>(ptr);

    if( !grid_ptr )
        throw std::runtime_error("Object is of non-grid type");

    grid_ptr->set_central_angle(m_central_angle);
#endif
}


void SetGridCentralAngle::save(SerializationInterface &ar) const
{
    ar << m_target_id << m_central_angle;
}


void SetGridCentralAngle::load(DeserializationInterface &ar)
{
    ar >> m_target_id >> m_central_angle;
}
