/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetGridSegments.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../GridObject.hh"
#endif


using namespace peekabot;


SetGridSegments::SetGridSegments() throw()
{
}


SetGridSegments::SetGridSegments(
    ObjectID target_id, uint32_t segment_count) throw()
    : m_target_id(target_id),
      m_segment_count(segment_count)
{
}


SetGridSegments::~SetGridSegments() throw()
{
}


Action *SetGridSegments::clone() const
{
    return new SetGridSegments(m_target_id, m_segment_count);
}


void SetGridSegments::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target_id);

    if( !ptr )
        throw std::runtime_error("Target object not found");

    GridObject *grid_ptr = dynamic_cast<GridObject *>(ptr);

    if( !grid_ptr )
        throw std::runtime_error("Object is of non-grid type");

    grid_ptr->set_segment_count(m_segment_count);
#endif
}


void SetGridSegments::save(SerializationInterface &ar) const
{
    ar << m_target_id << m_segment_count;
}


void SetGridSegments::load(DeserializationInterface &ar)
{
    ar >> m_target_id >> m_segment_count;
}
