/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetGridType.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../GridObject.hh"
#endif


using namespace peekabot;


SetGridType::SetGridType() throw()
{
}


SetGridType::SetGridType(
    ObjectID target_id, GridType type) throw()
    : m_target_id(target_id),
      m_type(type)
{
}


SetGridType::~SetGridType() throw()
{
}


Action *SetGridType::clone() const
{
    return new SetGridType(m_target_id, m_type);
}


void SetGridType::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target_id);

    if( !ptr )
        throw std::runtime_error("Target object not found");

    GridObject *grid_ptr = dynamic_cast<GridObject *>(ptr);

    if( !grid_ptr )
        throw std::runtime_error("Object is of non-grid type");

    grid_ptr->set_grid_type(m_type);
#endif
}


void SetGridType::save(SerializationInterface &ar) const
{
    ar << m_target_id << m_type;
}


void SetGridType::load(DeserializationInterface &ar)
{
    ar >> m_target_id >> m_type;
}
