/*
 * Copyright Staffan Gimåker 2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/math/fpclassify.hpp>

#include "SetJointMin.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../Joint.hh"
#endif


using namespace peekabot;


SetJointMin::SetJointMin(ObjectID id, float val) throw()
    : m_id(id), m_val(val)
{
}


SetJointMin::SetJointMin() throw()
{
}


SetJointMin::SetJointMin(const SetJointMin &other) throw()
    : m_id(other.m_id),
      m_val(other.m_val)
{
}


SetJointMin::~SetJointMin() throw()
{
}


Action *SetJointMin::clone() const
{
    return new SetJointMin(*this);
}


void SetJointMin::execute(
    ServerExecutionContext *context)
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    Joint *ptr = dynamic_cast<Joint *>(context->get_object(m_id));

    if( !ptr )
        throw std::runtime_error(
            "Failed to set joint min, target object not "
            "found or of non-joint type");

    if( !boost::math::isfinite(m_val) )
        throw std::logic_error(
            "Failed to set DOF min value: value must not "
            "be infinity or NaN");

    ptr->set_min_value(m_val);
#endif
}


void SetJointMin::save(SerializationInterface &ar) const
{
    ar << m_id << m_val;
}


void SetJointMin::load(DeserializationInterface &ar)
{
    ar >> m_id >> m_val;
}
