/*
 * Copyright Staffan Gimåker 2007-2009.
 * Copyright Anders Boberg 2007.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/math/fpclassify.hpp>

#include "SetJointValue.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../Joint.hh"
#endif


using namespace peekabot;


SetJointValue::SetJointValue(
    ObjectID dof_id,
    real_unit dof_value,
    bool user) throw()
    : m_dof_id(dof_id),
      m_dof_value(dof_value),
      m_user_value(user)
{
}


SetJointValue::SetJointValue() throw()
{

}


SetJointValue::SetJointValue(const SetJointValue &dof) throw()
    : m_dof_id(dof.m_dof_id),
      m_dof_value(dof.m_dof_value)
{
}


SetJointValue::~SetJointValue() throw()
{
}


Action *SetJointValue::clone() const
{
    return new SetJointValue(*this);
}


void SetJointValue::execute(
    ServerExecutionContext *context)
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    Joint *ptr = dynamic_cast<Joint *>(context->get_object(m_dof_id));

    if( !ptr )
        throw std::runtime_error(
            "Failed to set joint value, target object not "
            "found or of non-joint type");

    if( !boost::math::isfinite(m_dof_value) )
        throw std::logic_error(
            "Failed to set DOF value: value must not "
            "be infinity or NaN");

    if(m_user_value)
        ptr->set_user_value(m_dof_value);
    else
        ptr->set_value(m_dof_value);
#endif
}


void SetJointValue::save(SerializationInterface &ar) const
{
    ar << m_dof_id << m_dof_value << m_user_value;
}

void SetJointValue::load(DeserializationInterface &ar)
{
    ar >> m_dof_id >> m_dof_value >> m_user_value;
}
