/*
 * Copyright Staffan Gimåker 2007-2009.
 * Copyright Anders Boberg 2007.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_SET_JOINT_VALUE_HH_INCLUDED
#define PEEKABOT_SET_JOINT_VALUE_HH_INCLUDED


#include <stdexcept>
#include "../Types.hh"
#include "../Action.hh"


namespace peekabot
{
    /**
     * \brief Set the value of the given joint.
     */
    class SetJointValue : public Action
    {
        PEEKABOT_SERIALIZABLE

    public:
        SetJointValue(
            ObjectID dof_id,
            real_unit dof_value,
            bool user = false) throw();

        virtual ~SetJointValue() throw();

        virtual Action *clone() const;

        virtual void execute(
            ServerExecutionContext *context)
            const throw(std::exception);

    private:
        SetJointValue() throw();

        SetJointValue(const SetJointValue &dof) throw();

    protected:
        virtual void save(SerializationInterface &ar) const;

        virtual void load(DeserializationInterface &ar);

    private:
        ObjectID m_dof_id;

        real_unit m_dof_value;

        bool m_user_value;
    };
}


// PEEKABOT_REGISTER_SERIALIZABLE 33


#endif // PEEKABOT_SET_JOINT_VALUE_HH_INCLUDED
