/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetLabelAlignment.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../Label.hh"
#endif


using namespace peekabot;


SetLabelAlignment::SetLabelAlignment() throw()
{
}


SetLabelAlignment::SetLabelAlignment(
    ObjectID target_id, TextAlignment alignment) throw()
    : m_target_id(target_id),
      m_alignment(alignment)
{
}


SetLabelAlignment::~SetLabelAlignment() throw()
{
}


Action *SetLabelAlignment::clone() const
{
    return new SetLabelAlignment(m_target_id, m_alignment);
}


void SetLabelAlignment::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target_id);

    if( !ptr )
        throw std::runtime_error("Target object not found");

    Label *lbl = dynamic_cast<Label *>(ptr);
    if( !lbl )
        throw std::runtime_error("Target is of non-label type");

    lbl->set_alignment(m_alignment);
#endif
}


void SetLabelAlignment::save(SerializationInterface &ar) const
{
    ar << m_target_id << m_alignment;
}


void SetLabelAlignment::load(DeserializationInterface &ar)
{
    ar >> m_target_id >> m_alignment;
}
