/*
 * Copyright Anders Boberg 2006.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetLayer.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#endif


using namespace peekabot;


SetLayer::SetLayer() throw()
{
}


SetLayer::SetLayer(ObjectID target_id, uint32_t layer, bool recursive) throw()
    : m_target_id(target_id),
      m_layer(layer),
      m_recursive(recursive)
{
}


SetLayer::~SetLayer() throw()
{
}


Action *SetLayer::clone() const
{
    return new SetLayer(m_target_id, m_layer, m_recursive);
}


void SetLayer::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target_id);

    if( !ptr )
        throw std::runtime_error(
            "Failed to set layer, target object not found");

    if(m_recursive)
    {
        set_recursively(ptr);
    }
    else
    {
        if(ptr->get_layer() != m_layer)
            ptr->set_layer(m_layer);
    }
#endif
}

#ifdef __PEEKABOT_SERVER
void SetLayer::set_recursively(SceneObject *ptr) const
{
    if(ptr->get_layer() != m_layer)
        ptr->set_layer(m_layer);

    SceneObject::ChildContainer &children = ptr->get_children();
    for(SceneObject::ChildContainer::iterator it = children.begin();
        it != children.end(); ++it)
        set_recursively(*it);
}
#endif


void SetLayer::save(SerializationInterface &ar) const
{
    ar << m_target_id << m_layer << m_recursive;
}

void SetLayer::load(DeserializationInterface &ar)
{
    ar >> m_target_id >> m_layer >> m_recursive;
}
