/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetLineStyle.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../LineBased.hh"
#endif


using namespace peekabot;


SetLineStyle::SetLineStyle() throw()
{
}


SetLineStyle::SetLineStyle(ObjectID target_id, LineStyle style, int factor) throw()
    : m_target_id(target_id),
      m_style(style),
      m_factor((uint32_t)factor)
{
}


SetLineStyle::~SetLineStyle() throw()
{
}


Action *SetLineStyle::clone() const
{
    return new SetLineStyle(m_target_id, m_style, m_factor);
}


void SetLineStyle::execute(
    ServerExecutionContext *context)
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target_id);

    if( !ptr )
        throw std::runtime_error("Target object not found");

    LineBased *line_based = dynamic_cast<LineBased *>(ptr);

    if( !line_based )
        throw std::runtime_error("Target not line based");

    line_based->set_line_style(m_style, m_factor);
#endif
}


void SetLineStyle::save(SerializationInterface &ar) const
{
    ar << m_target_id << m_style << m_factor;
}


void SetLineStyle::load(DeserializationInterface &ar)
{
    ar >> m_target_id >> m_style >> m_factor;
}
