/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetLineWidth.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../LineBased.hh"
#endif


using namespace peekabot;


SetLineWidth::SetLineWidth() throw()
{
}


SetLineWidth::SetLineWidth(ObjectID target_id, float width) throw()
    : m_target_id(target_id),
      m_width(width)
{
}


SetLineWidth::~SetLineWidth() throw()
{
}


Action *SetLineWidth::clone() const
{
    return new SetLineWidth(m_target_id, m_width);
}


void SetLineWidth::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target_id);

    if( !ptr )
        throw std::runtime_error("Target object not found");

    LineBased *line_based = dynamic_cast<LineBased *>(ptr);

    if( !line_based )
        throw std::runtime_error("Target not line based");
    
    line_based->set_line_width(m_width);
#endif
}


void SetLineWidth::save(SerializationInterface &ar) const
{
    ar << m_target_id << m_width;
}


void SetLineWidth::load(DeserializationInterface &ar)
{
    ar >> m_target_id >> m_width;
}
