/*
 * Copyright Anders Boberg 2006.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetMetaName.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../SceneObject.hh"
#endif


using namespace peekabot;


SetMetaName::SetMetaName() throw()
{
}


SetMetaName::SetMetaName(ObjectID target, std::string name) throw()
    : m_target(target), 
      m_meta_name(name)
{
}


Action *SetMetaName::clone() const
{
    return new SetMetaName(m_target, m_meta_name);
}


void SetMetaName::execute(ServerExecutionContext *context) const
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target);

    if( !ptr )
        throw std::runtime_error(
            "Failed to set meta name, target object not found");

    if( ptr->get_meta_name() != m_meta_name )
    {
        ptr->set_meta_name(m_meta_name);
    }
#endif
}


void SetMetaName::save(SerializationInterface &ar) const
{
    ar << m_target << m_meta_name;
}

void SetMetaName::load(DeserializationInterface &ar)
{
    ar >> m_target >> m_meta_name;
}
