/*
 * Copyright Anders Boberg 2006.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetName.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../SceneObject.hh"
#endif


using namespace peekabot;


SetName::SetName() throw()
{
}


SetName::SetName(ObjectID target, std::string name) throw()
    : m_target(target), 
      m_name(name)
{
}


Action *SetName::clone() const
{
    return new SetName(m_target, m_name);
}


void SetName::execute(ServerExecutionContext *context) const
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target);

    if( !ptr )
        throw std::runtime_error(
            "Failed to set name, target object not found");

    if( ptr->get_name() != m_name )
    {
        ptr->set_name(m_name);
    }
#endif
}


void SetName::save(SerializationInterface &ar) const
{
    ar << m_target << m_name;
}

void SetName::load(DeserializationInterface &ar)
{
    ar >> m_target >> m_name;
}
