/*
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetOpacity.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#endif


using namespace peekabot;


SetOpacity::SetOpacity() throw()
{
}


SetOpacity::SetOpacity(ObjectID target_id, const Opacity &opacity) throw()
    : m_target_id(target_id),
      m_opacity(opacity)
{
}


SetOpacity::~SetOpacity() throw()
{
}


Action *SetOpacity::clone() const
{
    return new SetOpacity(m_target_id, m_opacity);
}


void SetOpacity::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target_id);

    if( !ptr )
        throw std::runtime_error(
            "Failed to set opacity, target object not found");

    ptr->set_opacity(m_opacity);
#endif
}


void SetOpacity::save(SerializationInterface &ar) const
{
    ar << m_target_id << m_opacity;
}

void SetOpacity::load(DeserializationInterface &ar)
{
    ar >> m_target_id >> m_opacity;
}
