/*
 * Copyright Anders Boberg 2006.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <cassert>
#include <stdexcept>
#include "SetOrthographic.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../SceneObject.hh"
#  include "../CameraObject.hh"
#endif


using namespace peekabot;


SetOrthographic::SetOrthographic() throw()
{
}


SetOrthographic::SetOrthographic(ObjectID target, bool orthographic) throw()
    : m_target(target),
      m_orthographic(orthographic)
{
}


Action *SetOrthographic::clone() const
{
    return new SetOrthographic(m_target, m_orthographic);
}


void SetOrthographic::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    CameraObject *ptr = dynamic_cast<CameraObject *>(
        context->get_object(m_target));

    if( !ptr )
        throw std::runtime_error(
            "Camera not found, or of non-camera type");
    
    if(m_orthographic != ptr->is_orthographic())
        ptr->set_orthographic(m_orthographic);
#endif
}


void SetOrthographic::save(SerializationInterface &ar) const
{
    ar << m_target << m_orthographic;
}

void SetOrthographic::load(DeserializationInterface &ar)
{
    ar >> m_target >> m_orthographic;
}
