/*
 * Copyright Staffan Gimåker 2007-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_SET_POSE_HH_INCLUDED
#define PEEKABOT_SET_POSE_HH_INCLUDED


#include <stdexcept>
#include "../Types.hh"
#include "../Action.hh"
#include "../Vector.hh"


namespace peekabot
{
    /**
     * \brief An action that merges the procedure for setting an object's 
     * position and orientation into one easy to use package.
     */
    class SetPose : public Action
    {
        PEEKABOT_SERIALIZABLE

    public:
        SetPose(
            ObjectID target, 
            const Vector3ru &pos, 
            real_unit yaw,
            real_unit roll,
            real_unit pitch,
            CoordinateSystem coord_sys) throw();

        virtual ~SetPose() throw();

        virtual Action *clone() const;

        virtual void execute(
            ServerExecutionContext *context) 
            const throw(std::exception);

    private:
        SetPose() throw();

        SetPose(const SetPose &action) throw();

    protected:
        virtual void save(SerializationInterface &ar) const;

        virtual void load(DeserializationInterface &ar);

    private:
        ObjectID m_target;
        
        Vector3ru m_pos;

        real_unit m_yaw, m_roll, m_pitch;

        CoordinateSystem m_coord_sys;
    };
}


// PEEKABOT_REGISTER_SERIALIZABLE 34


#endif // PEEKABOT_SET_POSE_HH_INCLUDED
