/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_SET_ROTATION_HH_INCLUDED
#define PEEKABOT_SET_ROTATION_HH_INCLUDED


#include <stdexcept>
#include "../Types.hh"
#include "../Action.hh"


namespace peekabot
{
    class SetRotation : public Action
    {
        PEEKABOT_SERIALIZABLE

    public:
        SetRotation(
            ObjectID target,
            real_unit yaw,
            real_unit roll,
            real_unit pitch,
            CoordinateSystem coord_sys) throw();

        virtual ~SetRotation() throw();

        virtual Action *clone() const;

        virtual void execute(
            ServerExecutionContext *context)
            const throw(std::exception);

    private:
        SetRotation() throw();

        SetRotation(const SetRotation &action) throw();

    protected:
        virtual void save(SerializationInterface &ar) const;

        virtual void load(DeserializationInterface &ar);

    private:
        ObjectID m_target;

        real_unit m_yaw, m_roll, m_pitch;

        CoordinateSystem m_coord_sys;
    };
}


// PEEKABOT_REGISTER_SERIALIZABLE 87


#endif // PEEKABOT_SET_ROTATION_HH_INCLUDED
