/*
 * Copyright Staffan Gimåker 2006-2009.
 * Copyright Anders Boberg 2007.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/math/fpclassify.hpp>
#include <stdexcept>

#include "SetScale.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../SceneObject.hh"
#  include "../ScalableObject.hh"
#endif


using namespace peekabot;


SetScale::SetScale() throw()
{
}


SetScale::SetScale(
    ObjectID id, real_unit x_scale, real_unit y_scale, real_unit z_scale,
    bool relative) 
    throw()
    : m_id(id), m_x_scale(x_scale), m_y_scale(y_scale), m_z_scale(z_scale),
      m_relative(relative)
{
}


SetScale::~SetScale() throw()
{
}


Action *SetScale::clone() const
{
    return new SetScale(m_id, m_x_scale, m_y_scale, m_z_scale);
}


void SetScale::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    ScalableObject *ptr = dynamic_cast<ScalableObject *>(
        context->get_object(m_id));

    // Check for infinity and NaN
    if( !boost::math::isfinite(m_x_scale) ||
        !boost::math::isfinite(m_y_scale) ||
        !boost::math::isfinite(m_z_scale) )
            throw std::logic_error(
                "Failed to set scale: scale "
                "cannot contain infinity or NaN");

    if( ptr )
    {
        if(m_relative)
        {        
            ptr->set_scale(ptr->get_x_scale()*m_x_scale,
                           ptr->get_y_scale()*m_y_scale,
                           ptr->get_z_scale()*m_z_scale);
        }
        else
        {
            ptr->set_scale(m_x_scale, m_y_scale, m_z_scale);
        }
    }
    else
        throw std::runtime_error(
            "Scaling failed - object not found or non-scalable.");
#endif
}


void SetScale::save(SerializationInterface &ar) const
{
    ar << m_id << m_x_scale << m_y_scale << m_z_scale << m_relative;
}

void SetScale::load(DeserializationInterface &ar)
{
    ar >> m_id >> m_x_scale >> m_y_scale >> m_z_scale >> m_relative;
}
