/*
 * Copyright Staffan Gimåker 2007-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/scoped_array.hpp>
#include "SetSensorData.hh"
#include "../Bytesex.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../serialization/ChunkedBufferAdapter.hh"
#  include "../SensorObject.hh"
#endif


using namespace peekabot;


SetSensorData::SetSensorData(
    ObjectID target, const ChunkedBuffer &data) throw()
    : m_target(target),
      m_data(data),
      m_data_is_big_endian(PEEKABOT_BYTE_ORDER == PEEKABOT_BIG_ENDIAN)
{
}


SetSensorData::SetSensorData() throw()
    : m_data(1024)
{
}


SetSensorData::SetSensorData(const SetSensorData &action) throw()
    : m_target(action.m_target),
      m_data(action.m_data),
      m_data_is_big_endian(action.m_data_is_big_endian)
{
}


SetSensorData::~SetSensorData() throw()
{
}


Action *SetSensorData::clone() const
{
    return new SetSensorData(*this);
}


void SetSensorData::execute(
    ServerExecutionContext *context) const
{
#ifdef __PEEKABOT_SERVER
    SensorObject *sensor = context->get_object<SensorObject>(m_target);

    if( !sensor )
        throw std::runtime_error(
            "Failed to set sensor data, target object not found or of wrong type");

    ChunkedBuffer data_copy(m_data);
    ChunkedBufferAdapter adapter(data_copy);
    DeserializationInterface buf(adapter, m_data_is_big_endian);
    sensor->set_sensor_data(buf);
#endif
}


void SetSensorData::save(SerializationInterface &ar) const
{
    uint32_t n = m_data.get_size();
    ar << m_target << m_data_is_big_endian << n;

    if( n > 0 )
    {
        boost::scoped_array<uint8_t> data(new uint8_t[n]);
        m_data.peek(data.get(), n);
        ar.save_binary(data.get(), n);
    }
}


void SetSensorData::load(DeserializationInterface &ar)
{
    uint32_t n;
    ar >> m_target >> m_data_is_big_endian >> n;

    if( n > 0 )
    {
        boost::scoped_array<uint8_t> data(new uint8_t[n]);
        ar.load_binary(data.get(), n);
        m_data.clear();
        m_data.write(data.get(), n);
    }
}

