/*
 * Copyright Staffan Gimåker 2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/math/fpclassify.hpp>

#include "SetSliderAxis.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../Slider.hh"
#endif


using namespace peekabot;


SetSliderAxis::SetSliderAxis(
    ObjectID target,
    float x, float y, float z,
    CoordinateSystem coord_sys)
    : m_target(target),
      m_x(x), m_y(y), m_z(z),
      m_coord_sys(coord_sys)
{
}


SetSliderAxis::SetSliderAxis() throw()
{
}


SetSliderAxis::SetSliderAxis(const SetSliderAxis &other) throw()
    : m_target(other.m_target),
      m_x(other.m_x), m_y(other.m_y), m_z(other.m_z),
      m_coord_sys(other.m_coord_sys)
{
}


SetSliderAxis::~SetSliderAxis() throw()
{
}


Action *SetSliderAxis::clone() const
{
    return new SetSliderAxis(*this);
}


void SetSliderAxis::execute(
    ServerExecutionContext *context)
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    Slider *ptr = dynamic_cast<Slider *>(context->get_object(m_target));

    if( !ptr )
        throw std::runtime_error(
            "Failed to set slider axis, target object not found or "
            "of non-slider type");

    // Check for infinity and NaN
    if( !boost::math::isfinite(m_x) ||
        !boost::math::isfinite(m_y) ||
        !boost::math::isfinite(m_z) )
        throw std::logic_error(
                "Failed to set slider axis: axis "
                "cannot contain infinity or NaN");

    ptr->set_axis(Vector3f(m_x, m_y, m_z), m_coord_sys);
#endif
}


void SetSliderAxis::save(SerializationInterface &ar) const
{
    ar << m_target << m_x << m_y << m_z << m_coord_sys;
}

void SetSliderAxis::load(DeserializationInterface &ar)
{
    ar >> m_target >> m_x >> m_y >> m_z >> m_coord_sys;
}
