/*
 * Copyright Anders Boberg 2006.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetVisibility.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#endif


using namespace peekabot;


SetVisibility::SetVisibility() throw()
{
}


SetVisibility::SetVisibility(ObjectID target_id, bool visible) throw()
    : m_target_id(target_id),
      m_visible(visible)
{
}


SetVisibility::~SetVisibility() throw()
{
}


Action *SetVisibility::clone() const
{
    return new SetVisibility(m_target_id, m_visible);
}


void SetVisibility::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target_id);

    if( !ptr )
        throw std::runtime_error("Object not found");

    if( ptr->is_hidden() == m_visible )
        ptr->set_hidden(!m_visible);
#endif
}


void SetVisibility::save(SerializationInterface &ar) const
{
    ar << m_target_id << m_visible;
}

void SetVisibility::load(DeserializationInterface &ar)
{
    ar >> m_target_id >> m_visible;
}
