/*
 * Copyright Anders Boberg 2006-2007.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <cassert>
#include "SetZoomDistance.hh"
#include "../Matrix.hh"
#include "../Vector.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../SceneObject.hh"
#  include "../CameraObject.hh"
#endif


using namespace peekabot;


SetZoomDistance::SetZoomDistance() throw()
{
}


SetZoomDistance::SetZoomDistance(
    ObjectID target, float zoom_distance, bool relative) throw()
    : m_target(target),
      m_zoom_distance(zoom_distance),
      m_relative(relative)
{
}


Action *SetZoomDistance::clone() const
{
    return new SetZoomDistance(m_target, m_zoom_distance, m_relative);
}


void SetZoomDistance::execute(
    ServerExecutionContext *context) 
    const throw(std::exception)
{
#ifdef __PEEKABOT_SERVER
    CameraObject *ptr = dynamic_cast<CameraObject *>(
        context->get_object(m_target));

    if( !ptr )
        throw std::runtime_error(
            "Failed to set zoom distance - object not found or of non-camera type");
    if(m_relative)
    {
        real_unit zoom = ptr->get_zoom_distance() * m_zoom_distance;
        zoom = (zoom < 0.01 ? 0.01 : zoom);
        ptr->set_zoom_distance(ptr->get_zoom_distance() + m_zoom_distance);
    }
    else
        ptr->set_zoom_distance(m_zoom_distance);
#endif
}


void SetZoomDistance::save(SerializationInterface &ar) const
{
    ar << m_target << m_zoom_distance << m_relative;
}

void SetZoomDistance::load(DeserializationInterface &ar)
{
    ar >> m_target >> m_zoom_distance >> m_relative;
}
