/*
 * Copyright Anders Boberg 2007.
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_VIEWPORT_MOVE_HH_INCLUDED
#define PEEKABOT_VIEWPORT_MOVE_HH_INCLUDED


#include "../Types.hh"
#include "../Action.hh"
#include "../Matrix.hh"


namespace peekabot
{
    class ViewportMove : public Action
    {
        PEEKABOT_SERIALIZABLE

    public:
        ViewportMove(ObjectID target, real_unit dx, real_unit dy,
                     transformation_target::Axis axes,
                     transformation_target::CoordinateSystem system,
                     ObjectID camera = 0) throw();

        virtual Action *clone() const;

        virtual void execute(
            ServerExecutionContext *context) 
            const throw(std::exception);

    private:
        ViewportMove() throw();

    protected:
        virtual void save(SerializationInterface &ar) const;

        virtual void load(DeserializationInterface &ar);

    private:
        // The target object to move
        ObjectID m_target;

        // The amount to move along each axis
        real_unit m_dx, m_dy;
        
        // Which axes to transform along
        transformation_target::Axis m_axes;

        // The coordinate system to use
        transformation_target::CoordinateSystem m_system;
        
        // The camera in whose view plane the transformation takes place
        ObjectID m_camera;
    };
}


// PEEKABOT_REGISTER_SERIALIZABLE 52


#endif // PEEKABOT_VIEWPORT_MOVE_HH_INCLUDED
