/*
 * Copyright Anders Boberg 2007.
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_VIEWPORT_ROTATE_HH_INCLUDED
#define PEEKABOT_VIEWPORT_ROTATE_HH_INCLUDED

#include <set>
#include "../Types.hh"
#include "../Action.hh"
#include "../Matrix.hh"
#include "../Vector.hh"


namespace peekabot
{
    class ViewportRotate : public Action
    {
        PEEKABOT_SERIALIZABLE

    public:
        typedef std::set<ObjectID> ObjectSet;

        ViewportRotate(
            ObjectSet targets, real_unit dx, real_unit dy,
            transformation_target::Axis axes,
            transformation_target::CoordinateSystem axis_system,
            transformation_target::RotationCenterpoint pivot,
            ObjectID camera) throw();

        virtual Action *clone() const;

        virtual void execute(
            ServerExecutionContext *context)
            const throw(std::exception);

    private:
        ViewportRotate() throw();

    protected:
        virtual void save(SerializationInterface &ar) const;

        virtual void load(DeserializationInterface &ar);

    private:
        // The target objects to move
        ObjectSet m_targets;

        // The amount cursor moved in viewport
        real_unit m_dx, m_dy;

        ObjectID m_camera;

        // The axes around which the transformation should take place
        transformation_target::Axis m_axes;

        // The coordinate system to use for axes
        transformation_target::CoordinateSystem m_axis_system;

        // The center point for the rotation
        Vector4ru m_center_point;

        // The pivot of rotation
        transformation_target::RotationCenterpoint m_pivot;
    };
}


// PEEKABOT_REGISTER_SERIALIZABLE 53


#endif // PEEKABOT_VIEWPORT_ROTATE_HH_INCLUDED
