/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "CameraProxy.hh"
#include "../PeekabotClient.hh"

#include "../../actions/AddCamera.hh"
#include "../../actions/AssignCamera.hh"
#include "../../actions/SetFOV.hh"
#include "../../actions/SetZoomDistance.hh"
#include "../../actions/SetOrthographic.hh"


using namespace peekabot;
using namespace peekabot::client;


//
// ------------------ CameraProxyBase implementation -------------------
//


CameraProxyBase::CameraProxyBase()
{
}


CameraProxyBase::CameraProxyBase(const CameraProxyBase &p)
    : ObjectProxyBase(p)
{
}

//

DelayedDispatch CameraProxyBase::set_fov(float fov_deg)
{
    return DelayedDispatch(
        get_client_impl(), new SetFOV(get_object_id(), fov_deg));
}


DelayedDispatch CameraProxyBase::set_zoom_distance(
    float zoom_distance, bool relative)
{
    return DelayedDispatch(
        get_client_impl(),
        new SetZoomDistance(get_object_id(), zoom_distance, relative));
}


DelayedDispatch CameraProxyBase::set_orthographic(bool orthographic)
{
    return DelayedDispatch(
        get_client_impl(), new SetOrthographic(get_object_id(), orthographic));
}


//
// ------------------ CameraProxy implementation -------------------
//


CameraProxy::CameraProxy()
{
}


CameraProxy::CameraProxy(const CameraProxyBase &p)
    : CameraProxyBase(p)
{
}


CameraProxy &CameraProxy::operator=(const CameraProxy &p) throw()
{
    return *this = (CameraProxyBase &)p;
}


CameraProxy &CameraProxy::operator=(const CameraProxyBase &p) throw()
{
    unchecked_assign(unchecked_get_client_impl(p), get_pseudonym(p));
    return *this;
}


DelayedDispatch CameraProxy::assign(const ObjectProxyBase &p)
{
    unchecked_assign(get_client_impl(p), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignCamera(PathIdentifier(get_object_id(p)), get_object_id()));
}


DelayedDispatch CameraProxy::assign(
    PeekabotClient &client,
    const std::string &path)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignCamera(PathIdentifier(path), get_object_id()));
}


DelayedDispatch CameraProxy::assign(
    const ObjectProxyBase &parent, 
    const std::string &rel_path)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(), new AssignCamera(
            PathIdentifier(get_object_id(parent), rel_path), get_object_id()));
}


DelayedDispatch CameraProxy::add(
    PeekabotClient &client,
    const std::string &path,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddCamera(PathIdentifier(path), 
                      conflict_policy, true, get_object_id()));
}


DelayedDispatch CameraProxy::add(
    const ObjectProxyBase &parent, 
    const std::string &name,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddCamera(PathIdentifier(get_object_id(parent), name), 
                      conflict_policy, true, get_object_id()));
}
