/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "CircleProxy.hh"
#include "../PeekabotClient.hh"

#include "../../actions/AddCircle.hh"
#include "../../actions/AssignCircle.hh"


using namespace peekabot;
using namespace peekabot::client;


//
// ------------------ CircleProxyBase implementation -------------------
//


CircleProxyBase::CircleProxyBase()
{
}


CircleProxyBase::CircleProxyBase(const CircleProxyBase &p)
    : ObjectProxyBase(p),
      ScalableProxyBase(p),
      LineBasedProxyBase(p)
{
}


//
// ------------------ CircleProxy implementation -------------------
//


CircleProxy::CircleProxy()
{
}


CircleProxy::CircleProxy(const CircleProxyBase &p)
    : ObjectProxyBase(p),
      CircleProxyBase(p)
{
}


CircleProxy &CircleProxy::operator=(const CircleProxy &p) throw()
{
    return *this = (CircleProxyBase &)p;
}


CircleProxy &CircleProxy::operator=(const CircleProxyBase &p) throw()
{
    unchecked_assign(unchecked_get_client_impl(p), get_pseudonym(p));
    return *this;
}


DelayedDispatch CircleProxy::assign(const ObjectProxyBase &p)
{
    unchecked_assign(get_client_impl(p), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignCircle(PathIdentifier(get_object_id(p)), get_object_id()));
}


DelayedDispatch CircleProxy::assign(
    PeekabotClient &client,
    const std::string &path)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(), 
        new AssignCircle(PathIdentifier(path), get_object_id()));
}


DelayedDispatch CircleProxy::assign(
    const ObjectProxyBase &parent, 
    const std::string &rel_path)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(), new AssignCircle(
            PathIdentifier(get_object_id(parent), rel_path), get_object_id()));
}


DelayedDispatch CircleProxy::add(
    PeekabotClient &client,
    const std::string &path,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddCircle(PathIdentifier(path), conflict_policy, 
                      true, get_object_id()));
}


DelayedDispatch CircleProxy::add(
    const ObjectProxyBase &parent, 
    const std::string &name,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddCircle(PathIdentifier(get_object_id(parent), name), 
                      conflict_policy, true, get_object_id()));
}
