/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "CubeProxy.hh"
#include "../PeekabotClient.hh"

#include "../../actions/AddCube.hh"
#include "../../actions/AssignCube.hh"


using namespace peekabot;
using namespace peekabot::client;


//
// ------------------ CubeProxyBase implementation -------------------
//


CubeProxyBase::CubeProxyBase()
{
}


CubeProxyBase::CubeProxyBase(const CubeProxyBase &p)
    : ObjectProxyBase(p),
      ScalableProxyBase(p)
{
}


//
// ------------------ CubeProxy implementation -------------------
//


CubeProxy::CubeProxy()
{
}


CubeProxy::CubeProxy(const CubeProxyBase &p)
    : ObjectProxyBase(p),
      CubeProxyBase(p)
{
}


CubeProxy &CubeProxy::operator=(const CubeProxy &p) throw()
{
    return *this = (CubeProxyBase &)p;
}


CubeProxy &CubeProxy::operator=(const CubeProxyBase &p) throw()
{
    unchecked_assign(unchecked_get_client_impl(p), get_pseudonym(p));
    return *this;
}


DelayedDispatch CubeProxy::assign(const ObjectProxyBase &p)
{
    unchecked_assign(get_client_impl(p), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignCube(PathIdentifier(get_object_id(p)), get_object_id()));
}


DelayedDispatch CubeProxy::assign(
    PeekabotClient &client,
    const std::string &path)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignCube(PathIdentifier(path), get_object_id()));
}


DelayedDispatch CubeProxy::assign(
    const ObjectProxyBase &parent, 
    const std::string &rel_path)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignCube(
            PathIdentifier(get_object_id(parent), rel_path), get_object_id()));
}


DelayedDispatch CubeProxy::add(
    PeekabotClient &client,
    const std::string &path,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddCube(PathIdentifier(path), conflict_policy, 
                    true, get_object_id()));
}


DelayedDispatch CubeProxy::add(
    const ObjectProxyBase &parent, 
    const std::string &name,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddCube(PathIdentifier(get_object_id(parent), name), 
                      conflict_policy, true, get_object_id()));
}
