/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "CylinderProxy.hh"
#include "../PeekabotClient.hh"

#include "../../actions/AddCylinder.hh"
#include "../../actions/AssignCylinder.hh"


using namespace peekabot;
using namespace peekabot::client;


//
// ------------------ CylinderProxyBase implementation -------------------
//


CylinderProxyBase::CylinderProxyBase()
{
}


CylinderProxyBase::CylinderProxyBase(const CylinderProxyBase &p)
    : ObjectProxyBase(p),
      ScalableProxyBase(p)
{
}


//
// ------------------ CylinderProxy implementation -------------------
//


CylinderProxy::CylinderProxy()
{
}


CylinderProxy::CylinderProxy(const CylinderProxyBase &p)
    : ObjectProxyBase(p),
      CylinderProxyBase(p)
{
}


CylinderProxy &CylinderProxy::operator=(const CylinderProxy &p) throw()
{
    return *this = (CylinderProxyBase &)p;
}


CylinderProxy &CylinderProxy::operator=(const CylinderProxyBase &p) throw()
{
    unchecked_assign(unchecked_get_client_impl(p), get_pseudonym(p));
    return *this;
}


DelayedDispatch CylinderProxy::assign(const ObjectProxyBase &p)
{
    unchecked_assign(get_client_impl(p), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignCylinder(PathIdentifier(get_object_id(p)), get_object_id()));
}


DelayedDispatch CylinderProxy::assign(
    PeekabotClient &client,
    const std::string &path)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignCylinder(PathIdentifier(path), get_object_id()));
}


DelayedDispatch CylinderProxy::assign(
    const ObjectProxyBase &parent, 
    const std::string &rel_path)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(), new AssignCylinder(
            PathIdentifier(get_object_id(parent), rel_path), get_object_id()));
}


DelayedDispatch CylinderProxy::add(
    PeekabotClient &client,
    const std::string &path,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddCylinder(PathIdentifier(path), conflict_policy, 
                        true, get_object_id()));
}


DelayedDispatch CylinderProxy::add(
    const ObjectProxyBase &parent, 
    const std::string &name,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddCylinder(PathIdentifier(get_object_id(parent), name), 
                      conflict_policy, true, get_object_id()));
}
