/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "GroupProxy.hh"
#include "../PeekabotClient.hh"

#include "../../actions/AddGroup.hh"
#include "../../actions/AssignGroup.hh"


using namespace peekabot;
using namespace peekabot::client;


//
// ------------------ GroupProxyBase implementation -------------------
//


GroupProxyBase::GroupProxyBase()
{
}


GroupProxyBase::GroupProxyBase(const GroupProxyBase &p)
    : ObjectProxyBase(p)
{
}


//
// ------------------ GroupProxy implementation -------------------
//


GroupProxy::GroupProxy()
{
}


GroupProxy::GroupProxy(const GroupProxyBase &p)
    : GroupProxyBase(p)
{
}


GroupProxy &GroupProxy::operator=(const GroupProxy &p) throw()
{
    return *this = (GroupProxyBase &)p;
}


GroupProxy &GroupProxy::operator=(const GroupProxyBase &p) throw()
{
    unchecked_assign(unchecked_get_client_impl(p), get_pseudonym(p));
    return *this;
}


DelayedDispatch GroupProxy::assign(const ObjectProxyBase &p)
{
    unchecked_assign(get_client_impl(p), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignGroup(PathIdentifier(get_object_id(p)), get_object_id()));
}


DelayedDispatch GroupProxy::assign(
    PeekabotClient &client,
    const std::string &path)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignGroup(PathIdentifier(path), get_object_id()));
}


DelayedDispatch GroupProxy::assign(
    const ObjectProxyBase &parent, 
    const std::string &rel_path)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(), new AssignGroup(
            PathIdentifier(get_object_id(parent), rel_path), get_object_id()));
}


DelayedDispatch GroupProxy::add(
    PeekabotClient &client,
    const std::string &path,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddGroup(PathIdentifier(path), conflict_policy, 
                     true, get_object_id()));
}


DelayedDispatch GroupProxy::add(
    const ObjectProxyBase &parent, 
    const std::string &name,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddGroup(PathIdentifier(get_object_id(parent), name), 
                      conflict_policy, true, get_object_id()));
}
