/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "JointProxy.hh"
#include "../PeekabotClient.hh"

#include "../../actions/AssignJoint.hh"
#include "../../actions/SetJointValue.hh"
#include "../../actions/SetJointMin.hh"
#include "../../actions/SetJointMax.hh"
#include "../../actions/SetJointOffset.hh"


using namespace peekabot;
using namespace peekabot::client;


//
// ------------------ JointProxyBase implementation -------------------
//


JointProxyBase::JointProxyBase()
{
}


JointProxyBase::JointProxyBase(const JointProxyBase &p)
    : ObjectProxyBase(p)
{
}


DelayedDispatch JointProxyBase::set_dof(float val) throw()
{
    return DelayedDispatch(
        get_client_impl(),
        new SetJointValue(get_object_id(), val));
}


DelayedDispatch JointProxyBase::set_min(float min)
{
    return DelayedDispatch(
        get_client_impl(),
        new SetJointMin(get_object_id(), min));
}


DelayedDispatch JointProxyBase::set_max(float max)
{
    return DelayedDispatch(
        get_client_impl(),
        new SetJointMax(get_object_id(), max));
}


DelayedDispatch JointProxyBase::set_offset(float offset)
{
    return DelayedDispatch(
        get_client_impl(),
        new SetJointOffset(get_object_id(), offset));
}


//
// ------------------ JointProxy implementation -------------------
//


JointProxy::JointProxy()
{
}


JointProxy::JointProxy(const JointProxyBase &p)
    : JointProxyBase(p)
{
}


JointProxy &JointProxy::operator=(const JointProxy &p) throw()
{
    return *this = (JointProxyBase &)p;
}


JointProxy &JointProxy::operator=(const JointProxyBase &p) throw()
{
    unchecked_assign(unchecked_get_client_impl(p), get_pseudonym(p));

    return *this;
}


DelayedDispatch JointProxy::assign(const ObjectProxyBase &p)
{
    unchecked_assign(get_client_impl(p), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignJoint(PathIdentifier(get_object_id(p)), get_object_id()));
}


DelayedDispatch JointProxy::assign(
    PeekabotClient &client, const std::string &path)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignJoint(PathIdentifier(path), get_object_id()));
}


DelayedDispatch JointProxy::assign(
    const ObjectProxyBase &parent,
    const std::string &rel_path)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignJoint(
            PathIdentifier(get_object_id(parent), rel_path), get_object_id()));
}
