/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "LabelProxy.hh"
#include "../PeekabotClient.hh"

#include "../../actions/AddLabel.hh"
#include "../../actions/AssignLabel.hh"
#include "../../actions/SetLabelText.hh"
#include "../../actions/SetLabelAlignment.hh"


using namespace peekabot;
using namespace peekabot::client;


//
// ------------------ LabelProxyBase implementation -------------------
//


LabelProxyBase::LabelProxyBase()
{
}


LabelProxyBase::LabelProxyBase(const LabelProxyBase &p)
    : ObjectProxyBase(p),
      ScalableProxyBase(p)
{
}

//

DelayedDispatch LabelProxyBase::set_text(const std::string &text)
{
    return DelayedDispatch(
        get_client_impl(), new SetLabelText(get_object_id(), text));
}

DelayedDispatch LabelProxyBase::set_alignment(TextAlignment alignment)
{
    return DelayedDispatch(
        get_client_impl(), new SetLabelAlignment(get_object_id(), alignment));
}


//
// ------------------ LabelProxy implementation -------------------
//


LabelProxy::LabelProxy()
{
}


LabelProxy::LabelProxy(const LabelProxyBase &p)
    : ObjectProxyBase(p),
      LabelProxyBase(p)
{
}


LabelProxy &LabelProxy::operator=(const LabelProxy &p) throw()
{
    return *this = (LabelProxyBase &)p;
}


LabelProxy &LabelProxy::operator=(const LabelProxyBase &p) throw()
{
    unchecked_assign(unchecked_get_client_impl(p), get_pseudonym(p));
    return *this;
}


DelayedDispatch LabelProxy::assign(const ObjectProxyBase &p)
{
    unchecked_assign(get_client_impl(p), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignLabel(PathIdentifier(get_object_id(p)), get_object_id()));
}


DelayedDispatch LabelProxy::assign(
    PeekabotClient &client, const std::string &path)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(), new AssignLabel(PathIdentifier(path), get_object_id()));
}


DelayedDispatch LabelProxy::assign(
    const ObjectProxyBase &parent, 
    const std::string &rel_path)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(), new AssignLabel(
            PathIdentifier(get_object_id(parent), rel_path), get_object_id()));
}


DelayedDispatch LabelProxy::add(
    PeekabotClient &client,
    const std::string &path,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddLabel(PathIdentifier(path), 
                     conflict_policy, true, get_object_id()));
}


DelayedDispatch LabelProxy::add(
    const ObjectProxyBase &parent, 
    const std::string &name,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddLabel(PathIdentifier(get_object_id(parent), name), 
                     conflict_policy, true, get_object_id()));
}
