/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "OccupancyGrid2DProxy.hh"
#include "../PeekabotClient.hh"

#include "../../actions/AddOccupancyGrid2D.hh"
#include "../../actions/AssignOccupancyGrid2D.hh"
#include "../../actions/SetOccupancyGrid2DCells.hh"


using namespace peekabot;
using namespace peekabot::client;


//
// ------------------ OccupancySet2D implementation -------------------
//


void OccupancySet2D::set_cell(float x, float y, float belief)
{
    m_cells.push_back(std::make_pair(Vector2f(x,y), belief));
}


void OccupancySet2D::clear()
{
    m_cells.clear();
}

std::size_t OccupancySet2D::size() const
{
    return m_cells.size();
}


bool OccupancySet2D::empty() const
{
    return m_cells.empty();
}


//
// ------------------ OccupancyGrid2DProxyBase implementation -------------------
//


OccupancyGrid2DProxyBase::OccupancyGrid2DProxyBase()
{
}


OccupancyGrid2DProxyBase::OccupancyGrid2DProxyBase(OccupancyGrid2DProxyBase &p)
    : ObjectProxyBase(p)
{
}

//

DelayedDispatch OccupancyGrid2DProxyBase::set_cells(
    const std::vector<std::pair<Vector2f, float> > &cells)
{
    return DelayedDispatch(
        get_client_impl(),
        new SetOccupancyGrid2DCells(get_object_id(), cells));
}


DelayedDispatch OccupancyGrid2DProxyBase::set_cell(
    float x, float y, float belief)
{
    std::vector<std::pair<Vector2f, float> > cells;
    cells.push_back(std::make_pair(Vector2f(x,y), belief));

    return set_cells(cells);
}


DelayedDispatch OccupancyGrid2DProxyBase::set_cells(
    const OccupancySet2D &cells)
{
    return DelayedDispatch(
        get_client_impl(),
        new SetOccupancyGrid2DCells(get_object_id(), cells.m_cells));
}

//
// ------------------ OccupancyGrid2DProxy implementation -------------------
//


OccupancyGrid2DProxy::OccupancyGrid2DProxy()
{
}


OccupancyGrid2DProxy::OccupancyGrid2DProxy(OccupancyGrid2DProxy &p)
    : OccupancyGrid2DProxyBase(p)
{
}


OccupancyGrid2DProxy &OccupancyGrid2DProxy::operator=(
    const OccupancyGrid2DProxy &p) throw()
{
    return *this = (OccupancyGrid2DProxyBase &)p;
}


OccupancyGrid2DProxy &OccupancyGrid2DProxy::operator=(const 
    OccupancyGrid2DProxyBase &p) throw()
{
    unchecked_assign(unchecked_get_client_impl(p), get_pseudonym(p));
    return *this;
}


DelayedDispatch OccupancyGrid2DProxy::assign(const ObjectProxyBase &p)
{
    unchecked_assign(get_client_impl(p), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignOccupancyGrid2D(
            PathIdentifier(get_object_id(p)), get_object_id()));
}


DelayedDispatch OccupancyGrid2DProxy::assign(
    PeekabotClient &client,
    const std::string &path)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignOccupancyGrid2D(PathIdentifier(path), get_object_id()));
}


DelayedDispatch OccupancyGrid2DProxy::assign(
    const ObjectProxyBase &parent, 
    const std::string &rel_path)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(), new AssignOccupancyGrid2D(
            PathIdentifier(get_object_id(parent), rel_path), get_object_id()));
}


DelayedDispatch OccupancyGrid2DProxy::add(
    PeekabotClient &client,
    const std::string &path,
    float cell_size,
    float unoccupied_r, float unoccupied_g, float unoccupied_b,
    float occupied_r, float occupied_g, float occupied_b, 
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddOccupancyGrid2D(
            PathIdentifier(path), cell_size,
            RGBColor(unoccupied_r,unoccupied_g,unoccupied_b),
            RGBColor(occupied_r,occupied_g,occupied_b),
            conflict_policy, true, get_object_id()));
}


DelayedDispatch OccupancyGrid2DProxy::add(
    const ObjectProxyBase &parent, 
    const std::string &name,
    float cell_size,
    float unoccupied_r, float unoccupied_g, float unoccupied_b,
    float occupied_r, float occupied_g, float occupied_b, 
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddOccupancyGrid2D(
            PathIdentifier(get_object_id(parent), name), cell_size,
            RGBColor(unoccupied_r,unoccupied_g,unoccupied_b),
            RGBColor(occupied_r,occupied_g,occupied_b),
            conflict_policy, true, get_object_id()));
}
