/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "PolygonProxy.hh"
#include "../PeekabotClient.hh"

#include "../../actions/AddPolygon.hh"
#include "../../actions/AssignPolygon.hh"


using namespace peekabot;
using namespace peekabot::client;


//
// ------------------ PolygonProxyBase implementation -------------------
//


PolygonProxyBase::PolygonProxyBase()
{
}


PolygonProxyBase::PolygonProxyBase(const PolygonProxyBase &p)
    : ObjectProxyBase(p),
      ScalableProxyBase(p),
      VertexBasedProxyBase(p)
{
}


//
// ------------------ PolygonProxy implementation -------------------
//


PolygonProxy::PolygonProxy()
{
}


PolygonProxy::PolygonProxy(const PolygonProxyBase &p)
    : ObjectProxyBase(p),
      PolygonProxyBase(p)
{
}


PolygonProxy &PolygonProxy::operator=(const PolygonProxy &p) throw()
{
    return *this = (PolygonProxyBase &)p;
}


PolygonProxy &PolygonProxy::operator=(const PolygonProxyBase &p) throw()
{
    unchecked_assign(unchecked_get_client_impl(p), get_pseudonym(p));
    return *this;
}


DelayedDispatch PolygonProxy::assign(const ObjectProxyBase &p)
{
    unchecked_assign(get_client_impl(p), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignPolygon(PathIdentifier(get_object_id(p)), get_object_id()));
}


DelayedDispatch PolygonProxy::assign(
    PeekabotClient &client,
    const std::string &path)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignPolygon(PathIdentifier(path), get_object_id()));
}


DelayedDispatch PolygonProxy::assign(
    const ObjectProxyBase &parent, 
    const std::string &rel_path)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(), new AssignPolygon(
            PathIdentifier(get_object_id(parent), rel_path), get_object_id()));
}


DelayedDispatch PolygonProxy::add(
    PeekabotClient &client,
    const std::string &path,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddPolygon(PathIdentifier(path), 
                       conflict_policy, 
                       true, get_object_id()));
}


DelayedDispatch PolygonProxy::add(
    const ObjectProxyBase &parent, 
    const std::string &name,
    NameConflictPolicy conflict_policy)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AddPolygon(PathIdentifier(get_object_id(parent), name), 
                       conflict_policy, 
                       true, get_object_id()));
}
