/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "ScalableProxy.hh"
#include "../PeekabotClient.hh"

#include "../../actions/AssignScalable.hh"
#include "../../actions/SetScale.hh"


using namespace peekabot;
using namespace peekabot::client;


//
// ------------------ ScalableProxyBase implementation -------------------
//

ScalableProxyBase::ScalableProxyBase()
{
}


ScalableProxyBase::ScalableProxyBase(const ScalableProxyBase &p)
    : ObjectProxyBase(p)
{
}


DelayedDispatch ScalableProxyBase::set_scale(
    float x_scale, float y_scale, float z_scale) throw()
{
    return DelayedDispatch(
        get_client_impl(),
        new SetScale(get_object_id(), x_scale, y_scale, z_scale));
}


//
// ------------------ ScalableProxy implementation -------------------
//


ScalableProxy::ScalableProxy()
{
}

ScalableProxy::ScalableProxy(const ScalableProxyBase &p)
    : ObjectProxyBase(p),
      ScalableProxyBase(p)
{
}


ScalableProxy &ScalableProxy::operator=(const ScalableProxy &p) throw()
{
    return *this = (ScalableProxyBase &)p;
}


ScalableProxy &ScalableProxy::operator=(const ScalableProxyBase &p) throw()
{
    unchecked_assign(p);
    return *this;
}


DelayedDispatch ScalableProxy::assign(const ObjectProxyBase &p)
{
    unchecked_assign(unchecked_get_client_impl(p), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignScalable(PathIdentifier(get_object_id(p)), get_object_id()));
}


DelayedDispatch ScalableProxy::assign(
    PeekabotClient &client,
    const std::string &path)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignScalable(PathIdentifier(path), get_object_id()));
}


DelayedDispatch ScalableProxy::assign(
    const ObjectProxyBase &parent,
    const std::string &rel_path)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignScalable(
            PathIdentifier(get_object_id(parent), rel_path), get_object_id()));
}
