/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef PEEKABOT_CLIENT_SENSOR_PROXY_HH_INCLUDED
#define PEEKABOT_CLIENT_SENSOR_PROXY_HH_INCLUDED


#include "../../Deprecated.hh"
#include "ObjectProxy.hh"


namespace peekabot
{
    class ChunkedBuffer;

    namespace client
    {
        /**
         * \brief Base class for SensorProxy.
         *
         * \deprecated The sensor concept was deprecated in 0.6. Support will 
         * be removed in 0.7.
         */
        class SensorProxyBase : public ObjectProxyBase
        {
        public:
            class SensorData
            {
                friend class SensorProxyBase;

            public:
                SensorData();

                ~SensorData();

                void write(float x) throw();

                void write(float x[], size_t n) throw();

                void write(double x) throw();

                void write(double x[], size_t n) throw();

                void write(bool x) throw();

                void write(bool x[], size_t n) throw();

                void clear() throw();
                
            private:
                ChunkedBuffer *m_buf;
            };

            SensorProxyBase();

            SensorProxyBase(const SensorProxyBase &p);

            /**
             * \brief Send and update the sensor in the server with the data
             * contained in the given SensorData object.
             *
             * How the sensor data is interpreted and displayed depends on the 
             * actual server-side type of the sensor. See \ref sensor_types for 
             * documentation on the available sensor types and in what format 
             * they expect the data to be.
             */
            DelayedDispatch update(const SensorData &reading);
        } PEEKABOT_DEPRECATED;


        /**
         * \brief Proxy class used to manipulate sensors, and update their 
         * sensor readings, in particular.
         *
         * The sensor proxy accumulates sensor data locally in a 
         * SenorProxyBase::SensorData object. When all needed data has been 
         * accumulated to update the sensor, use update() to send the data and 
         * update the sensor.
         *
         * Usage example:
         * \code
         * peekabot::SensorProxy p;
         * p.assign(client, "root.laser_scanner");
         * float d[361];
         * ...
         * peekabot::SensorProxy::SensorData reading;
         * reading.write(d, 361);
         * p.update(reading); // Send data and update the sensor
         * \endcode
         *
         * \deprecated The sensor concept was deprecated in 0.6. Support will 
         * be removed in 0.7.
         */
        class SensorProxy : public SensorProxyBase
        {
        public:
            SensorProxy();

            SensorProxy(const SensorProxyBase &p);

            SensorProxy &operator=(const SensorProxy &p) throw();

            SensorProxy &operator=(const SensorProxyBase &p) throw();

            /**
             * \copydoc ScalableProxy::assign(const ObjectProxyBase&)
             */
            DelayedDispatch assign(const ObjectProxyBase &p);

            /**
             * \copydoc ObjectProxy::assign(PeekabotClient&,const std::string&)
             */
            DelayedDispatch assign(
                PeekabotClient &client,
                const std::string &path);

            /**
             * \copydoc 
             * ObjectProxy::assign(const ObjectProxyBase&,const std::string&)
             */
            DelayedDispatch assign(
                const ObjectProxyBase &parent, 
                const std::string &rel_path);
        } PEEKABOT_DEPRECATED;
    }
}


#endif // PEEKABOT_CLIENT_SENSOR_PROXY_HH_INCLUDED
