/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include "VertexBasedProxy.hh"
#include "../PeekabotClient.hh"

#include "../../actions/AssignVertexBased.hh"
#include "../../actions/SetVertices.hh"


using namespace peekabot;
using namespace peekabot::client;


//
// ------------------ VertexSet implementation --------------------
//

void VertexSet::add_vertex(
    float x, float y, float z) throw()
{
    m_verts.push_back(Vector3f(x,y,z));
}

void VertexSet::clear() throw()
{
    m_verts.clear();
}

size_t VertexSet::size() const throw()
{
    return m_verts.size();
}

bool VertexSet::empty() const throw()
{
    return m_verts.empty();
}


//
// ------------------ VertexBasedProxyBase implementation -------------------
//

VertexBasedProxyBase::VertexBasedProxyBase()
{
}


VertexBasedProxyBase::VertexBasedProxyBase(const VertexBasedProxyBase &p)
    : ObjectProxyBase(p)
{
}

//

DelayedDispatch VertexBasedProxyBase::set_vertices(
    const VertexSet &vertices)
{
    return DelayedDispatch(
        get_client_impl(),
        new SetVertices(get_object_id(), vertices.m_verts));
}


DelayedDispatch VertexBasedProxyBase::add_vertices(
    const VertexSet &vertices)
{
    return DelayedDispatch(
        get_client_impl(),
        new SetVertices(get_object_id(), vertices.m_verts, true));
}


DelayedDispatch VertexBasedProxyBase::add_vertex(float x, float y, float z)
{
    VertexSet vs;
    vs.add_vertex(x,y,z);
    return add_vertices(vs);
}


DelayedDispatch VertexBasedProxyBase::clear_vertices()
{
    VertexSet empty_vs;
    return set_vertices(empty_vs);
}


//
// ------------------ VertexBasedProxy implementation -------------------
//


VertexBasedProxy::VertexBasedProxy()
{
}

VertexBasedProxy::VertexBasedProxy(const VertexBasedProxyBase &p)
    : ObjectProxyBase(p),
      VertexBasedProxyBase(p)
{
}


VertexBasedProxy &VertexBasedProxy::operator=(
    const VertexBasedProxy &p) throw()
{
    return *this = (VertexBasedProxyBase &)p;
}


VertexBasedProxy &VertexBasedProxy::operator=(
    const VertexBasedProxyBase &p) throw()
{
    unchecked_assign(unchecked_get_client_impl(p), get_pseudonym(p));
    return *this;
}

DelayedDispatch VertexBasedProxy::assign(const ObjectProxyBase &p)
{
    unchecked_assign(get_client_impl(p), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignVertexBased(PathIdentifier(get_object_id(p)), 
                              get_object_id()));
}

DelayedDispatch VertexBasedProxy::assign(
    PeekabotClient &client,
    const std::string &path)
{
    unchecked_assign(get_client_impl(client), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignVertexBased(PathIdentifier(path), get_object_id()));
}

DelayedDispatch VertexBasedProxy::assign(
    const ObjectProxyBase &parent, 
    const std::string &rel_path)
{
    unchecked_assign(get_client_impl(parent), allocate_pseudonym());

    return DelayedDispatch(
        get_client_impl(),
        new AssignVertexBased(
            PathIdentifier(get_object_id(parent), rel_path), get_object_id()));
}
