/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>

#include "CollapsableGroup.hh"

using namespace peekabot;
using namespace gui;

CollapsableGroup::CollapsableGroup(int x, int y, int w, int h, const char *l)
    : Fl_Group(x, y, w, h, l),
      m_original_h(h),
      m_fit(true),
      m_open(true)
{
    box(FL_NO_BOX);
    labeltype(FL_NO_LABEL);
    m_frame = new Fl_Box(x, y + 10, w, h - 10);
    resizable(0);
    m_frame->box(FL_PLASTIC_DOWN_FRAME);

    m_collapse_button = new Fl_Button(x + 10, y, w - 20, 20, l);
    m_collapse_button->callback(collapse_callback);
    m_collapse_button->clear_visible_focus();

    end();
}

void CollapsableGroup::set_open(bool open_group)
{  
    if(open_group == false && m_open == true)
    {
        for(int i = 0; i < children(); ++i)
        {
            Fl_Widget *w = child(i);
            if(w != m_collapse_button && w != m_frame)
            {
                w->hide();
                w->deactivate();
            }
        }

        size(w(), 25);
        m_frame->size(w(), 15);
    }
    else if(open_group == true && m_open == false)
    {
        for(int i = 0; i < children(); ++i)
        {
            Fl_Widget *w = child(i);
            if(w != m_collapse_button && w != m_frame)
            {
                w->show();
                w->activate();
            }
        }

        size(w(), m_original_h);
        m_frame->size(w(), m_original_h - 10);
    }
    
    m_open = open_group;
}


void CollapsableGroup::collapse_callback(Fl_Widget *w, void *data)
{
    // Cast the parent and toggle the open/closed status.
    CollapsableGroup *g = static_cast<CollapsableGroup*>(w->parent());
    g->set_open(!g->is_open());
    g->parent()->redraw();
}
