/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FL/Fl_Color_Chooser.H>

#include "ColorTrigger.hh"
#include "../Types.hh"
#include "UserInterface.hh"
#include "UpdateActions.hh"
#include "../SceneObject.hh"
#include "../SceneTreePtr.hh"

using namespace peekabot;
using namespace peekabot::gui;


ColorTrigger::ColorTrigger(Fl_Color_Chooser* color_chooser)
    : m_color_chooser(color_chooser)
{
}


ColorTrigger::~ColorTrigger()
{
    SceneTreePtr ptr = TheSceneTree::acquire();
    while( !m_objects.empty() )
        detach_from(*m_objects.begin(), false);
}


void ColorTrigger::on_color_set(SceneObject *object)
{
    // Holds the averaged color values
    RGBColor color;

    int count = m_objects.size();
    for(std::set<SceneObject *>::const_iterator it = m_objects.begin();
        it != m_objects.end(); ++it)
    {
        RGBColor tmp = (*it)->get_color();        
        color.r += tmp.r;
        color.g += tmp.g;
        color.b += tmp.b;
    }
    color.r /= count;
    color.g /= count;
    color.b /= count;
    
    TheUserInterface::instance().post_action(
        m_color_chooser, new UpdateColor(m_color_chooser, color.r, color.g, color.b));
    
}


void ColorTrigger::attach_to(SceneObject *object)
{
    // Add object to the set of registered objects
    if( m_objects.insert(object).second )
    {
        // Register triggers
        object->m_on_color_set.connect(
            boost::bind(&ColorTrigger::on_color_set, this, _1));
        
        object->m_on_subtree_detached.connect(
            boost::bind(&ColorTrigger::detach_from, this, _1, true));
        
        on_color_set(object);
    }
}

void ColorTrigger::detach_from(SceneObject *object, bool update_widget)
{
    // Erase object from set of registered objects
    if( m_objects.erase(object) > 0 )
    {
        // Deregister triggers
        object->m_on_color_set.disconnect(
            boost::bind(&ColorTrigger::on_color_set, this, _1));
        
        object->m_on_subtree_detached.disconnect(
            boost::bind(&ColorTrigger::detach_from, this, _1, true));

        if( update_widget )
            on_color_set(object);
    }
}

