/**
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <boost/bind.hpp>

#include "CommandRepository.hh"
#include "UserInterfaceUtilities.hh"
#include "Manipulator.hh"

using namespace peekabot;
using namespace gui;

CommandRepository::CommandRepository()
{
    m_command_map[CMD_SET_NAVIGATION_MODE] = boost::bind(
        &Manipulator::set_mode, TheUIUtilities::instance().get_manipulator(),
        NAVIGATE_MODE);
    m_command_map[CMD_SET_MOVE_MODE] = boost::bind(
        &Manipulator::set_mode, TheUIUtilities::instance().get_manipulator(),
        MOVE_MODE);   
    m_command_map[CMD_SET_ROTATE_MODE] = boost::bind(
        &Manipulator::set_mode, TheUIUtilities::instance().get_manipulator(),
        ROTATE_MODE);

    m_command_map[CMD_TOGGLE_MOVE_MODE] = boost::bind(
        &Manipulator::toggle_mode, TheUIUtilities::instance().get_manipulator(),
        MOVE_MODE);   
    m_command_map[CMD_TOGGLE_ROTATE_MODE] = boost::bind(
        &Manipulator::toggle_mode, TheUIUtilities::instance().get_manipulator(),
        ROTATE_MODE);



    m_command_map[CMD_SET_CAMERA_COORDINATES] = boost::bind(
        &Manipulator::set_system, TheUIUtilities::instance().get_manipulator(),
        transformation_target::CAMERA_COORDINATES);    
    m_command_map[CMD_SET_LOCAL_COORDINATES] = boost::bind(
        &Manipulator::set_system, TheUIUtilities::instance().get_manipulator(),
        transformation_target::LOCAL_COORDINATES);
    m_command_map[CMD_SET_WORLD_COORDINATES] = boost::bind(
        &Manipulator::set_system, TheUIUtilities::instance().get_manipulator(),
        transformation_target::WORLD_COORDINATES);
    m_command_map[CMD_SET_PARENT_COORDINATES] = boost::bind(
        &Manipulator::set_system, TheUIUtilities::instance().get_manipulator(),
        transformation_target::PARENT_COORDINATES);


    m_command_map[CMD_TOGGLE_X_AXIS] = boost::bind(
        &Manipulator::toggle_axis, TheUIUtilities::instance().get_manipulator(),
        transformation_target::X_AXIS);
    m_command_map[CMD_TOGGLE_Y_AXIS] = boost::bind(
        &Manipulator::toggle_axis,
        TheUIUtilities::instance().get_manipulator(),
        transformation_target::Y_AXIS);
    m_command_map[CMD_TOGGLE_Z_AXIS] = boost::bind(
        &Manipulator::toggle_axis, TheUIUtilities::instance().get_manipulator(),
        transformation_target::Z_AXIS); 

    m_command_map[CMD_SET_X_AXIS] = boost::bind(
        &Manipulator::set_axes, TheUIUtilities::instance().get_manipulator(),
        transformation_target::X_AXIS);
    m_command_map[CMD_SET_Y_AXIS] = boost::bind(
        &Manipulator::set_axes,
        TheUIUtilities::instance().get_manipulator(),
        transformation_target::Y_AXIS);
    m_command_map[CMD_SET_Z_AXIS] = boost::bind(
        &Manipulator::set_axes, TheUIUtilities::instance().get_manipulator(),
        transformation_target::Z_AXIS); 
     
    m_command_map[CMD_CLEAR_SELECTION] = boost::bind(
        &UserInterfaceUtilities::clear_selection,
        &TheUIUtilities::instance());
    m_command_map[CMD_DELETE_OBJECT] = boost::bind(
        &UserInterfaceUtilities::delete_selected,
        &TheUIUtilities::instance());
    m_command_map[CMD_FULLSCREEN] = boost::bind(
        &UserInterfaceUtilities::do_fullscreen,
        &TheUIUtilities::instance(), static_cast<Viewport*>(0));

    m_command_map[CMD_SET_LOCAL_PIVOT] = boost::bind(
        &Manipulator::set_pivot, TheUIUtilities::instance().get_manipulator(),
        transformation_target::LOCAL_CENTER);
    m_command_map[CMD_SET_PARENT_PIVOT] = boost::bind(
        &Manipulator::set_pivot, TheUIUtilities::instance().get_manipulator(),
        transformation_target::PARENT_CENTER);
    m_command_map[CMD_SET_WORLD_PIVOT] = boost::bind(
        &Manipulator::set_pivot, TheUIUtilities::instance().get_manipulator(),
        transformation_target::WORLD_CENTER);
    m_command_map[CMD_SET_AVERAGE_PIVOT] = boost::bind(
        &Manipulator::set_pivot, TheUIUtilities::instance().get_manipulator(),
        transformation_target::AVERAGE_CENTER);

    // Abort should probably be implemented as a separate function in
    // the future, if features such as undo are implemented.
    m_command_map[CMD_ABORT] = boost::bind(
        &Manipulator::set_mode, TheUIUtilities::instance().get_manipulator(),
        NAVIGATE_MODE);
}

CommandRepository::NullaryCommand &CommandRepository::get_command(
    Command command) throw(std::runtime_error)
{
    CommandMap::iterator it = m_command_map.find(command);
    if(it == m_command_map.end())
        throw(std::runtime_error("Error! Invalid command requested "
                                 "from CommandRepository!"));
    return it->second;
}
