/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#if 0
#include <FL/Fl_Slider.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Pack.H>

#include "UserInterface.hh"
#include "UserInterfaceUtilities.hh"
#include "UpdateActions.hh"
#include "DOFGroup.hh"
#include "../DOF.hh"
#include "../RobotObject.hh"
#include "../actions/SetDOFValue.hh"
#include "../renderer/Renderer.hh"
#include "../IDTranslationTable.hh"

using boost::posix_time::ptime;
using boost::posix_time::microsec_clock;
using namespace peekabot;
using namespace renderer;
using namespace gui;

DOFGroup::DOFGroup(int x, int y) : PropertyGroup(x, y, 230,175, "DOF")
{
    hide();
    
    begin();
    m_dof_pack = new Fl_Pack(x + 2, y + 20, 226, 74);
    m_dof_pack->labeltype(FL_NORMAL_LABEL);
    m_dof_pack->type(FL_VERTICAL);
    //m_dof_pack->box(FL_DOWN_BOX);
    m_dof_pack->end();
    m_dof_pack->show();

    end();
    set_open(false);
    show();

    m_trigger = new DOFTrigger(this);

    set_properties(TheUIUtilities::instance().get_selection());
}


DOFGroup::~DOFGroup()
{
    delete m_trigger;
}

void DOFGroup::set_properties(const std::set<ObjectID> &objects)
{
    if(objects.size() == 1)
    {
        // Lock the scene tree to make sure nothing is modified while we build.
        SceneTreePtr ptr = TheSceneTree::acquire();

        // Deregister our trigger from the previously selected object.
        m_trigger->deregister();
        m_dof_pack->clear();

        // Register for the new object.
        SceneObject* selected = ptr->get_object(*objects.begin(),
                                                GUI_SOURCE_ID);
        RobotObject* robo;
        DOF* dof;

        // If the newly selected object was a RobotObject,
        // we'll be interested in any DOFObject children.
        if((robo = dynamic_cast<RobotObject*>(selected)) != 0)
        {
              activate();
              // This will cause the trigger to search recursively
              // for DOFs in the subtree beneath the robot.
              m_trigger->on_child_attached(robo);
        }
        // If the user selected a DOFObject, only the DOF control
        // for that particular object will be shown.
        else if((dof = dynamic_cast<DOF*>(selected)) != 0)
        {
            activate();
            m_trigger->register_dof(dof);
        }
        else
            deactivate();
    }
    else
        deactivate();
}

void DOFGroup::clear_updates()
{
    TheUserInterface::instance().remove_updates(this);

    Fl_Widget* const *children = m_dof_pack->array();
    for(int i = 0; i < m_dof_pack->children(); ++i)
    {
        static_cast<DOFControl*>(children[i])->clear_updates();
    }
}

void DOFGroup::add_dof_control(const std::string& name, ObjectID dof,
                               real_unit min, real_unit max,
                               real_unit user_value, real_unit value)
{
    DOFControl *control = new DOFControl(
        m_dof_pack->x()+2, m_dof_pack->y()+2, m_dof_pack->w()-4, 70, name.c_str());
    m_dof_pack->add(control);
    control->init(dof, min, max, user_value, value);
    m_dof_pack->resize(x()+2, y()+20, w()-4, h()-22);
    redraw();
 
    // Lock the scene tree to make sure nothing is modified while we build.
    SceneTreePtr ptr = TheSceneTree::acquire();
    m_trigger->register_control(dof, control, control->get_sliders());    
}

DOFGroup::DOFControl::DOFControl(int x, int y, int w, int h, const char* l)
    : Fl_Group(x, y, w, h, l),
      m_last_action_time(microsec_clock::local_time())
{    
    hide();
    m_user_slider = new Fl_Slider(x + 10, y + 20, w - 20, 20, 0);
    m_user_slider->type(FL_HORIZONTAL);
    m_value_slider = new Fl_Slider(x + 10, y + 40, w - 20, 20, 0);
    m_value_slider->type(FL_HORIZONTAL);
    m_value_slider->deactivate();
    end();
    resizable(0);
    align(FL_ALIGN_INSIDE | FL_ALIGN_CENTER | FL_ALIGN_TOP);
    labeltype(FL_NORMAL_LABEL);
    box(FL_DOWN_BOX);
    show();
}

DOFGroup::DOFControl::~DOFControl()
{
}

void DOFGroup::DOFControl::init(ObjectID dof, real_unit min, real_unit max,
                                real_unit user_value, real_unit value)
{
    m_dof = dof;
    m_user_slider->range(min, max);
    m_user_slider->step((max-min)/100);
    m_user_slider->value(user_value);
    m_user_slider->callback(enter_value_callback, this);
    m_value_slider->range(min, max);
    m_value_slider->value(value);
}

void DOFGroup::DOFControl::enter_value_callback(
    Fl_Widget *w, void* data)
{
    Fl_Slider *slider = static_cast<Fl_Slider*>(w);
    DOFControl *control = static_cast<DOFControl*>(data);
    
    // Check to see that at least one action interval has passed
    // since our last visit here, else bail out.
    ptime current_time = microsec_clock::local_time();
    if(current_time < control->m_last_action_time +
       TheUserInterface::instance().get_action_interval())
        return;
    control->m_last_action_time = current_time;
    
    TheUIUtilities::instance().post_action(
        new SetDOFValue(control->m_dof, slider->value(), true));   
}

std::pair<Fl_Slider*, Fl_Slider*> DOFGroup::DOFControl::get_sliders()
{
    return std::make_pair(m_user_slider, m_value_slider);
}

void DOFGroup::DOFControl::clear_updates()
{
    // Scrap any incoming updates for the sliders
    TheUserInterface::instance().remove_updates(m_user_slider);
    TheUserInterface::instance().remove_updates(m_value_slider);   
}

DOFGroup::DOFTrigger::~DOFTrigger()
{
}

void DOFGroup::DOFTrigger::register_dof(DOF *dof)
{
    class AddDOF : public UpdateAction
    {
    public:
        AddDOF(DOFGroup* group, const std::string & name, ObjectID dof,
               real_unit min, real_unit max, real_unit user_value,
               real_unit value)
            : m_group(group), m_name(name), m_dof(dof), m_min(min), m_max(max),
              m_user_value(user_value), m_value(value) {}
        
        virtual void execute()
        {
            m_group->add_dof_control(m_name, m_dof, m_min, m_max, m_user_value,
                                     m_value);
        }

        virtual bool replaces() { return false; }

    private:
        DOFGroup *m_group;
        std::string m_name;
        ObjectID m_dof;
        
        real_unit m_min;
        real_unit m_max;
        real_unit m_user_value;
        real_unit m_value;
    };

    dof->register_trigger(ON_DELETE, this);
    dof->register_trigger(ON_DOF_USER_VALUE_SET, this);
    dof->register_trigger(ON_DOF_VALUE_SET, this);
    dof->register_trigger(ON_DOF_DOMAIN_SET, this);
    
    TheUserInterface::instance().post_action(
        m_dof_group, new AddDOF(m_dof_group, dof->get_name(),
                                dof->get_object_id(), dof->get_min_value(),
                                dof->get_max_value(), dof->get_user_value(),
                                dof->get_value()));
}

void DOFGroup::DOFTrigger::register_control(
    ObjectID object, DOFControl *control,
    std::pair<Fl_Slider*, Fl_Slider*> sliders)
{
    DOFMap::iterator it = m_dof_map.find(object);
    if(it == m_dof_map.end())
    {
        m_dof_map[object] = control;
        m_slider_map[object] = sliders;      
    }
}

void DOFGroup::DOFTrigger::trigger(StimuliType stimuli,
                                   SceneObject *object)
{
    switch(stimuli)
    {
        case ON_DELETE:
            on_delete(object);
            break;
        case ON_DOF_VALUE_SET:
            on_dof_value_set(object);
            break;
        case ON_DOF_USER_VALUE_SET:
            on_dof_user_value_set(object);
            break;
        case ON_DOF_DOMAIN_SET:
            on_dof_domain_set(object);
            break;
        case ON_CHILD_ATTACHED:
            on_child_attached(object);
        default:
            break;
    }    
}

void DOFGroup::DOFTrigger::on_delete(SceneObject *object)
{
    ObjectID id = object->get_object_id();

    // If the object's in the dof map, it means it's a dof object
    // and that its dof control should be removed.
    DOFMap::iterator it = m_dof_map.find(id);
    if(it != m_dof_map.end())
    {
        TheUserInterface::instance().post_action(
            it->second, new DeleteWidget(it->second));
    }
}

void DOFGroup::DOFTrigger::on_deregister(StimuliType stimuli,
                                         SceneObject *object)
    throw()
{
    ObjectID id = object->get_object_id();
    if(stimuli == ON_DOF_VALUE_SET)
    {
        m_dof_map.erase(id);
        m_slider_map.erase(id);
        NodeID ghost = m_ghost_map[id];
        TheRenderer::instance().remove_node(ghost);
        m_ghost_map.erase(id);
    }
}

void DOFGroup::DOFTrigger::on_dof_value_set(SceneObject *object)
{
    ObjectID id = object->get_object_id();

    // Verify that the object is in the dof map and send update.
    SliderMap::iterator it = m_slider_map.find(id);
    if(it != m_slider_map.end())
    {
        DOF *dof = static_cast<DOF*>(object);
        TheUserInterface::instance().post_action(
            it->second.second, new UpdateValuator(it->second.second,
                                                  dof->get_value()));
    }
}

void DOFGroup::DOFTrigger::on_dof_user_value_set(SceneObject *object)
{
    ObjectID id = object->get_object_id();
    NodeID ghost;

    if(m_ghost_map.find(id) == m_ghost_map.end())
    {
        NodeID orig_id = TheIDTranslationTable::instance().get_node_id(id);
        SceneObject *parent = object->get_parent();
        // What to do if object has no parent?
        if(parent == 0)
            return;
        NodeID parent_id = TheIDTranslationTable::instance().get_node_id(
            parent->get_object_id());

        ghost = TheRenderer::instance().clone_node(orig_id, parent_id);
        m_ghost_map[id] = ghost;
        TheRenderer::instance().set_node_opacity(ghost, Opacity(false,0.7));
        TheRenderer::instance().set_node_visibility(ghost, false);        
    }
    else
        ghost = m_ghost_map[id];

    DOF *dof = static_cast< DOF* >(object);
    Matrix4ru matrix = dof->get_full_transform(dof->get_user_value());

    TheRenderer::instance().set_node_transformation(
        ghost, matrix);
}

void DOFGroup::DOFTrigger::on_dof_domain_set(SceneObject *object)
{
    ObjectID id = object->get_object_id();

    // Verify that the object is in the dof map and send update.
    SliderMap::iterator it = m_slider_map.find(id);
    if(it != m_slider_map.end())
    {
        DOF *dof = static_cast<DOF*>(object);
        TheUserInterface::instance().post_action(
            it->second.first, new UpdateValuatorDomain(it->second.first,
                                                       dof->get_min_value(),
                                                       dof->get_max_value()));
        TheUserInterface::instance().post_action(
            it->second.second, new UpdateValuatorDomain(it->second.second,
                                                        dof->get_min_value(),
                                                        dof->get_max_value()));
    }
}

void DOFGroup::DOFTrigger::on_child_attached(SceneObject *object)
{
    DOF *dof = dynamic_cast<DOF*>(object);
    // If it's a DOF object, register for the dof modification stimuli
    if(dof != 0)
    {
        register_dof(dof);
    }
    
    // Recurse for children.
    SceneObject::ChildContainer &children = object->get_children();

    for( SceneObject::ChildContainer::iterator it = children.begin();
         it != children.end(); ++it )
    {
        on_child_attached((*it));
    }
}
#endif

