/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_GUI_DOFGROUP_HH
#define __PEEKABOT_GUI_DOFGROUP_HH

#if 0

#include <set>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <FL/Fl_Group.H>

#include "PropertyGroup.hh"
#include "../renderer/IDTypes.hh"
#include "../SceneTreePtr.hh"

class Fl_Slider;
class Fl_Pack;
class Fl_Button;

namespace peekabot
{

    class DOF;

namespace gui
{
    
    /** \brief A group of controls for manipulating degrees of freedom of a
     * robot.
     *
     * This group is available for both \c DOF objects (\c JointDOF and \c TrackDOF),
     * in which case it contains controls for manipulating only the DOF in question,
     * and for \c RobotObject, in which case it will contain controls for each
     * DOF object that is a child of the selected robot.
     */
class DOFGroup : public PropertyGroup
{   
public:
    
    /** \brief Creates a new \c DOFGroup object.
     *
     * Allocates and adds the sub widgets and fills them with properties by
     * creating and registering the \c DOFTrigger.
     */
    DOFGroup(int x, int y);

    /** \brief Deletes the trigger allocated for this group.
     */
    virtual ~DOFGroup();

    /** \brief Virtual method for filling this group with DOF information from an object.
     *
     * This method registers a DOFTrigger with the currently selected
     * object in order to fill the input fields with DOF information. When
     * the selection changes it deregisters triggers for any object that has
     * been deselected.
     *
     * This method is called by the \c PropertyVisitor upon creation, and
     * whenever the selection changes.
     *
     */
    virtual void set_properties(const std::set<ObjectID> &objects);

    /** \brief Removes all pending updates for this group.
     */
    void clear_updates();

    void add_dof_control(const std::string& name, ObjectID dof, real_unit min,
                         real_unit max, real_unit user_value, real_unit value);


    class DOFControl : public Fl_Group
    {
    public:
        DOFControl(int x, int y, int w, int h, const char* l = 0);
        virtual ~DOFControl();
        
        void init(ObjectID dof, real_unit min, real_unit max,
                  real_unit user_value, real_unit value);

        static void enter_value_callback(
            Fl_Widget *w, void* data);
        
        std::pair<Fl_Slider*, Fl_Slider*> get_sliders();
        
        void clear_updates();
    private:
        Fl_Slider *m_user_slider;
        Fl_Slider *m_value_slider;

        boost::posix_time::ptime m_last_action_time;

        ObjectID m_dof;        
    };

    class DOFTrigger : public ObjectTrigger
    {
    public:
        DOFTrigger(DOFGroup* group) : m_dof_group(group){}
        virtual ~DOFTrigger();
        typedef std::map<ObjectID, DOFControl*> DOFMap;
        typedef std::map<ObjectID, std::pair<Fl_Slider*, Fl_Slider*> > SliderMap;
        typedef std::map<ObjectID, NodeID> GhostMap;

        virtual void trigger(StimuliType stimuli, SceneObject *object);

        void on_delete(SceneObject* object);
        void on_dof_user_value_set(SceneObject* object);
        void on_dof_value_set(SceneObject* object);
        void on_dof_domain_set(SceneObject* object);
        void on_child_attached(SceneObject* object);

        virtual void on_deregister(StimuliType stimuli, SceneObject *object)
            throw();
        
        void register_dof(peekabot::DOF *object);
        void register_control(ObjectID object, DOFControl *control,
                              std::pair<Fl_Slider*, Fl_Slider*> sliders);

    private:
        DOFGroup *m_dof_group;
        DOFMap m_dof_map;
        SliderMap m_slider_map;
        GhostMap m_ghost_map;
        

    };

private:
    void find_dofs(SceneObject* object);

    Fl_Pack *m_dof_pack;
    
    Fl_Button *m_set_button;
    Fl_Button *m_reset_button;

    DOFTrigger* m_trigger;
};

}
}
#endif

#endif //__PEEKABOT_GUI_DOFGROUP_HH
