/**
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_GUI_DECORATOR_HH
#define __PEEKABOT_GUI_DECORATOR_HH


#include <map>
#include <set>

#include "../Types.hh"
#include "GUITypes.hh"
#include "../renderer/IDTypes.hh"


namespace peekabot
{
    namespace gui
    {
        class UserInterfaceUtilities;

        /**
         * \brief Class responsible for adding extra renderings effects and
         * decorations to the scene rendered in the viewports.
         *
         * Such extra decorations include outlines for selected objects,
         * lines to show the coordinate system used when moving, etc.
         */
        class Decorator
        {
        public:
            Decorator(UserInterfaceUtilities &uiu);

            virtual ~Decorator();

        private:
            void selection_changed(ObjectID id, bool selected) throw();

            void mode_changed(ManipulatorMode mode) throw();

            void coordinate_system_changed(CoordinateSystem coord_sys) throw();

            void pivot_changed(RotationCenterpoint pivot) throw();

            void axes_changed(Axis axes) throw();

        private:
            UserInterfaceUtilities &m_uiu;        
        };

    }
}


#endif // __PEEKABOT_GUI_DECORATOR_HH
