/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "UserInterfaceUtilities.hh"
#include "Frame.hh"
#include "PropertyFrame.hh"
#include "ViewFrame.hh"
#include "BrowserFrame.hh"
#include "ConsoleFrame.hh"
#include "Viewport.hh"
#include "Workspace.hh"

#include <iostream>

using namespace peekabot;
using namespace gui;


///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
////////////////////////   Frame methods //  //////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////




Frame::Frame(int x, int y, int w, int h, char* l) : 
    Fl_Scroll(x, y, w, h, l), m_selected(false)
{
    end();
    box(FL_DOWN_BOX);
    Fl_Menu_Button *menu = new Fl_Menu_Button(Fl_Scroll::x()+1, Fl_Scroll::y(), 30, 25);
    menu->color((Fl_Color)220);
    menu->add("View", 0 , 0);
    menu->add("Property inspector", 0 , 0);
    menu->add("Tree browser", 0 , 0);
    menu->add("Console", 0 , 0);
    menu->callback(menu_callback, this);
    add(menu);

}

int Frame::handle(int msg)
{
    if(msg == FL_MOVE)
    {
        // If this frame is already selected, leave the handling to the base
        // class.
        /*        if(m_selected)
                  return Fl_Scroll::handle(msg);*/

        // If it wasn't, we mark this frame as selected and do nothing further.
        dynamic_cast<FrameSelector*>(parent())->select_frame(this);
        return Fl_Scroll::handle(msg);
    }
    else
        return Fl_Scroll::handle(msg);
}

void Frame::draw()
{
    Fl_Scroll::draw();
    if(m_selected)
    {
        Fl_Color c = fl_color();
        fl_color(FL_BLACK);
        fl_rect(x(), y(), w(), h());
        fl_rect(x()+1, y()+1, w()-2, h()-2);
        fl_color(c);
    }
}

void Frame::menu_callback(Fl_Widget* w, void *data)
{
    Frame* frame = reinterpret_cast<Frame*>(data);
    const Fl_Menu_Item *item = reinterpret_cast<Fl_Menu_Button*>(w)->mvalue();
                     
    // Hack to determine which menu item we chose... better hope there will
    // never be more than one menu option with the same initial letter.
    switch(*item->text)
    {
        case 'P':
            // If the selected frame type is the same as the current,
            // we do nothing.
            if(frame->frame_type() != Frame::PROPERTY_FRAME)
                new PropertyFrame(frame);
            break;
        case 'V':
            if(frame->frame_type() != Frame::VIEW_FRAME)
                new ViewFrame(frame);
            break;
        case 'T':
            if(frame->frame_type() != Frame::BROWSER_FRAME)
                new BrowserFrame(frame);
            break;
        case 'C':
            if(frame->frame_type() != Frame::CONSOLE_FRAME)
                new ConsoleFrame(frame);
            break;
        default:
            break;
    }
}

void Frame::selected(bool selected)
{
    m_selected = selected;
}

bool Frame::is_selected() const
{
    return m_selected;
}
