/*
 * Copyright Anders Boberg 2006.
 * Copyright Staffan Gimåker 2007, 2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "IconManager.hh"
#include "Configuration.hh"

#include <FL/Fl_PNG_Image.H>

using namespace peekabot;
using namespace peekabot::gui;


IconManager::IconManager() throw()
{
    m_icon_dir = TheConfiguration::instance().get<fs::path>(
        "share_path", fs::get_pkgdata_path()) / "images";
}

IconManager::~IconManager() throw()
{
    ImageMap::iterator it;

    for(it = m_image_map.begin(); it != m_image_map.end(); ++it)
    {
        delete it->second;
        m_image_map.erase(it);
    }
}


Fl_Image* IconManager::get_icon(std::string filename) throw()
{
    ImageMap::iterator it = m_image_map.find(filename);
    if(it == m_image_map.end())
    {
        Fl_Image* image = new Fl_PNG_Image(
            (m_icon_dir / filename).string().c_str());

        m_image_map[filename] = image;
    }

    return m_image_map[filename];
}

