/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FL/Fl_Menu_Item.H>
#include <boost/format.hpp>
#include "../actions/LoadScene.hh"
#include "MainMenu.hh"
#include "Workspace.hh"
#include "UserInterfaceUtilities.hh"
#include "UserInterface.hh"
#include "ViewFrame.hh"


using namespace peekabot;
using namespace gui;

MainMenu::MainMenu(int x, int y, int w, int h, char* l) :
    Fl_Menu_Bar(x, y, w, h, l)
{
    const Fl_Menu_Item menu[] = {
        {"File", 0, 0, 0, FL_SUBMENU, FL_NORMAL_LABEL, 0, 14, 0},
        {"Load Scene", 0, &MainMenu::load_callback, 0, 0,
         FL_NORMAL_LABEL, 0, 14, 0},
        {"Quit", 0, &MainMenu::quit_callback, 0, 0,
         FL_NORMAL_LABEL, 0, 14, 0},
        {0,0,0,0,0,0,0,0,0},

        /*{"Edit", 0, 0, 0, 64, FL_NORMAL_LABEL, 0, 14, 0},
        {"Save snapshot", 0, 0, 0, FL_MENU_INACTIVE,
         FL_NORMAL_LABEL, 0, 14, 0},
         {0,0,0,0,0,0,0,0,0},*/

        {"Help", 0,  0, 0, 64, FL_NORMAL_LABEL, 0, 14, 0},
        {"About", 0, &MainMenu::about_callback, 0, 0,
         FL_NORMAL_LABEL, 0, 14, 0},
        {0,0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0,0}};
    
    copy(menu);
}

void MainMenu::quit_callback(Fl_Widget* w, void* data)
{
    TheUserInterface::instance().quit();
    //TheUserInterface::instance().get_main_window()->handle(FL_CLOSE);
}

void MainMenu::save_snapshot_callback(Fl_Widget *w, void* data)
{
    //    MainMenu *menu = static_cast<MainMenu*>(data);
    
    Workspace* active = TheUIUtilities::instance().active_workspace();

    if(active != 0)
    {
        ViewFrame* frame = dynamic_cast<ViewFrame*>(
            active->get_selected_frame());
        
        if(frame == 0)
        {
            fl_alert("Please select a ViewFrame first.");
            return;
        }
        else
        {
            std::string filename = fl_input(
                "Enter filename for snapshot:", "snapshot.tga");
            
            if(!filename.empty())
            {
                /// \todo Fix the snapshot feature
                //TheRenderer::instance().save_snapshot(filename);
                frame->redraw_viewport();
            }
        }
    }
}


void MainMenu::load_callback(Fl_Widget *w, void* data)
{
    //MainMenu *menu = static_cast<MainMenu*>(data);
    
    const char* tmp = fl_input(
        "File name:", "");
    if(tmp == 0)
        return;
    
    TheUIUtilities::instance().post_action(
        new LoadScene(ROOT_OBJECT_ID, tmp, AUTO_ENUMERATE_ON_CONFLICT));
}

void MainMenu::about_callback(Fl_Widget *w, void* data)
{
    fl_message(
        (boost::format(
            "peekabot %1%\n\n"
            "Copyright Anders Boberg and "
            "Staffan Gimker\n") % version::PEEKABOT_VERSION_STRING).str().c_str());
}
