/**
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_GUI_MANIPULATOR_HH
#define __PEEKABOT_GUI_MANIPULATOR_HH

#include "GUITypes.hh"
#include "../Types.hh"

#include <boost/signal.hpp>

namespace peekabot
{
    namespace gui
    {
        class Viewport;
        
        class Manipulator
        {
        public:
            typedef boost::signal<void (ManipulatorMode)> ModeSignal;
            typedef boost::signal<void (transformation_target::CoordinateSystem)> SystemSignal;
            typedef boost::signal<void (transformation_target::Axis)> AxisSignal;
            typedef boost::signal<void (transformation_target::RotationCenterpoint)> PivotSignal;

            Manipulator();

            void set_mode(ManipulatorMode mode) throw();

            void toggle_mode(ManipulatorMode mode) throw();

            ManipulatorMode get_mode() const throw();

            
            void set_system(transformation_target::CoordinateSystem system);

            transformation_target::CoordinateSystem get_system() const throw();


            void set_axes(transformation_target::Axis axes);

            void toggle_axis(transformation_target::Axis axis);

            transformation_target::Axis get_axes() const throw();

            void set_pivot(transformation_target::RotationCenterpoint pivot);

            transformation_target::RotationCenterpoint get_pivot() const throw();

            void viewport_click(Viewport *vp, int x, int y, int button,
                                int state);
            void viewport_drag(Viewport *vp, int dx, int dy, int button,
                               int state);

            void viewport_move(Viewport *vp, int dx, int dy, int state);
            
            void viewport_scroll(Viewport *vp, int delta, int state);

            ModeSignal & get_mode_signal();
            SystemSignal & get_system_signal();
            AxisSignal & get_axis_signal();
            PivotSignal & get_pivot_signal();

        private:
            int select(Viewport *vp, int x, int y, int button, int state);
            int navigate_mode(Viewport *vp, int dx, int dy, int button,
                             int state);
            int move_mode(Viewport *vp, int dx, int dy, int button,
                             int state);
            int rotate_mode(Viewport *vp, int dx, int dy, int button,
                             int state);

            void rotate_system_option()
            {
                int system = m_system << 1;
                if(system > transformation_target::PARENT_COORDINATES)
                    set_system(transformation_target::CAMERA_COORDINATES);
                else
                    set_system((transformation_target::CoordinateSystem)(system));
            }

            ManipulatorMode m_mode;
            transformation_target::CoordinateSystem m_system;
            transformation_target::Axis m_axes;
            transformation_target::RotationCenterpoint m_pivot;
            
            ModeSignal m_mode_signal;
            SystemSignal m_system_signal;
            AxisSignal m_axis_signal;
            PivotSignal m_pivot_signal;
        };
    }
}

#endif //__PEEKABOT_GUI_MANIPULATOR_HH
