/*
 * Copyright Anders Boberg 2006-2007.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FL/Fl_Tabs.H>

#include "../SceneTreePtr.hh"
#include "MoveRotateGroup.hh"
#include "UserInterface.hh"
#include "UserInterfaceUtilities.hh"
#include "../Types.hh"
#include "TransformationInput.hh"
#include "CoordinateTrigger.hh"
#include "../SceneObject.hh"
#include <algorithm>

using namespace peekabot;
using namespace peekabot::gui;
using namespace peekabot::transformation_target;


MoveRotateGroup::MoveRotateGroup(int x, int y) :
    PropertyGroup(x, y, 230, 175, "Transform")
{
    begin();
    Fl_Tabs* o = new Fl_Tabs(x, y + 25, 230, 150);
    o->box(FL_PLASTIC_UP_BOX);
       
    {
        Fl_Group* o = new Fl_Group(x, y + 45, 230, 125, "Move");
        o->color(fl_rgb_color(255));
        {
            Fl_Group* o = new Fl_Group(x + 110, y + 60, 100, 95,
                                       "Coordinates");
            o->box(FL_PLASTIC_THIN_UP_BOX);
            o->align(FL_ALIGN_TOP|FL_ALIGN_INSIDE);
            {
                m_world_coordinates = new Fl_Round_Button(
                    x + 110, y + 80, 60, 20, "World");
                m_world_coordinates->type(102);
                m_world_coordinates->down_box(FL_ROUND_DOWN_BOX);
                m_world_coordinates->callback(
                    coordinate_choice_callback, this);
            }
            {
                m_parent_coordinates = new Fl_Round_Button(
                    x + 110, y + 95, 60, 20, "Parent");
                m_parent_coordinates->type(102);
                m_parent_coordinates->value(1);
                m_parent_coordinates->down_box(FL_ROUND_DOWN_BOX);
                m_parent_coordinates->callback(
                    coordinate_choice_callback, this);
            }
            {
                m_local_coordinates = new Fl_Round_Button(
                    x + 110, y + 110, 60, 20, "Local");
                m_local_coordinates->type(102);
                m_local_coordinates->down_box(FL_ROUND_DOWN_BOX);
                m_local_coordinates->callback(
                    coordinate_choice_callback, this);
            }
            {
                m_relative_coordinates = new Fl_Check_Button(
                    x + 110, y + 130, 70, 25, "Relative");
                m_relative_coordinates->down_box(FL_DOWN_BOX);
                m_relative_coordinates->callback(
                    relative_coordinates_callback, this);
            }
            o->end();
        }
        {
            m_x_coordinate = new MoveInput(x + 25, y + 60,
                                                   60, 25, "X");
            m_x_coordinate->color(FL_LIGHT1);
            m_x_coordinate->labeltype(FL_ENGRAVED_LABEL);
            m_x_coordinate->labelcolor(FL_BACKGROUND_COLOR);
            m_x_coordinate->minimum(-10000);
            m_x_coordinate->maximum(10000);
            m_x_coordinate->step(0.01);
            m_x_coordinate->set_group(this);
            m_x_coordinate->set_coordinate(X_COORDINATE);
        }
        {
            m_y_coordinate = new MoveInput(x + 25, y + 95,
                                                   60, 25, "Y");
            m_y_coordinate->color(FL_LIGHT1);
            m_y_coordinate->labeltype(FL_ENGRAVED_LABEL);
            m_y_coordinate->labelcolor(FL_BACKGROUND_COLOR);
            m_y_coordinate->minimum(-10000);
            m_y_coordinate->maximum(10000);
            m_y_coordinate->step(0.01);
            m_y_coordinate->set_group(this);
            m_y_coordinate->set_coordinate(Y_COORDINATE);
        }
        {
            m_z_coordinate = new MoveInput(x + 25, y + 130,
                                                   60, 25, "Z");
            m_z_coordinate->color(FL_LIGHT1);
            m_z_coordinate->labeltype(FL_ENGRAVED_LABEL);
            m_z_coordinate->labelcolor(FL_BACKGROUND_COLOR);
            m_z_coordinate->minimum(-10000);
            m_z_coordinate->maximum(10000);
            m_z_coordinate->step(0.01);
            m_z_coordinate->set_group(this);
            m_z_coordinate->set_coordinate(Z_COORDINATE);
        }
        o->end();
        o->show();
    }
    {
        Fl_Group* o = new Fl_Group(x, y + 45, 230, 130, "Rotate");
        o->hide();
        o->color(FL_WHITE);
        {
            Fl_Group* o = new Fl_Group(x + 90, y + 70, 65, 75,
                                       "Pivot");
            o->box(FL_PLASTIC_THIN_UP_BOX);
            o->align(FL_ALIGN_TOP|FL_ALIGN_INSIDE);
            {
                m_parent_center = new Fl_Round_Button(
                    x + 90, y + 85, 65, 20, "Parent");
                m_parent_center->type(102);
                m_parent_center->down_box(FL_ROUND_DOWN_BOX);
                m_parent_center->callback(
                    rotational_system_callback, this);
            }
            {
                m_local_center = new Fl_Round_Button(
                    x + 90, y + 100, 55, 25, "Local");
                m_local_center->type(102);
                m_local_center->value(1);
                m_local_center->down_box(FL_ROUND_DOWN_BOX);
                m_local_center->callback(
                    rotational_system_callback, this);
            }
            {
                m_world_center = new Fl_Round_Button(
                    x + 90, y + 120, 55, 20, "World");
                m_world_center->type(102);
                m_world_center->value(0);
                m_world_center->down_box(FL_ROUND_DOWN_BOX);  
                m_world_center->callback(
                    rotational_system_callback, this);              
            }

            o->end();
        }
        {
            m_x_axis = new RotateInput(x + 25, y + 60,
                                       60, 25, "X");
            m_x_axis->color(FL_LIGHT1);
            m_x_axis->labeltype(FL_ENGRAVED_LABEL);
            m_x_axis->labelcolor(FL_BACKGROUND_COLOR);
            m_x_axis->minimum(-10000);
            m_x_axis->maximum(10000);
            m_x_axis->step(0.1);
            m_x_axis->set_axis(X_AXIS);
            m_x_axis->set_group(this);
        }
        {
            m_y_axis = new RotateInput(x + 25, y + 95,
                                       60, 25, "Y");
            m_y_axis->color(FL_LIGHT1);
            m_y_axis->labeltype(FL_ENGRAVED_LABEL);
            m_y_axis->labelcolor(FL_BACKGROUND_COLOR);
            m_y_axis->minimum(-10000);
            m_y_axis->maximum(10000);
            m_y_axis->step(0.1);
            m_y_axis->set_axis(Y_AXIS);
            m_y_axis->set_group(this);
        }
        {
            m_z_axis = new RotateInput(x + 25, y + 130,
                                       60, 25, "Z");
            m_z_axis->color(FL_LIGHT1);
            m_z_axis->labeltype(FL_ENGRAVED_LABEL);
            m_z_axis->labelcolor(FL_BACKGROUND_COLOR);
            m_z_axis->minimum(-10000);
            m_z_axis->maximum(10000);
            m_z_axis->step(0.1);
            m_z_axis->set_axis(Z_AXIS);
            m_z_axis->set_group(this);
        }
        {
            Fl_Group* o = new Fl_Group(x + 160, y + 70,
                                       65, 75, "Axis");
            o->box(FL_PLASTIC_THIN_UP_BOX);
            o->align(FL_ALIGN_TOP|FL_ALIGN_INSIDE);
            {
                m_parent_axis = new Fl_Round_Button(
                    x + 160, y + 85, 65, 20, "Parent");
                m_parent_axis->type(102);
                m_parent_axis->value(0);
                m_parent_axis->down_box(FL_ROUND_DOWN_BOX);
                m_parent_axis->callback(
                    rotational_system_callback, this);
            }
            {
                m_local_axis = new Fl_Round_Button(
                    x + 160, y + 100, 65, 25, "Local");
                m_local_axis->type(102);
                m_local_axis->value(1);
                m_local_axis->down_box(FL_ROUND_DOWN_BOX);
                m_local_axis->callback(
                    rotational_system_callback, this);
            }
            {
                m_world_axis = new Fl_Round_Button(
                    x + 160, y + 120, 65, 20, "World");
                m_world_axis->type(102);
                m_world_axis->down_box(FL_ROUND_DOWN_BOX);
                m_world_axis->callback(
                    rotational_system_callback, this);
            }
            o->end();
        }
        o->end();
    }
    //o->end();

    end();
    set_open(false);
    show();


    m_trigger = new CoordinateTrigger(m_x_coordinate, m_y_coordinate,
                                      m_z_coordinate);

    set_properties(TheUIUtilities::instance().get_selection());
}

MoveRotateGroup::~MoveRotateGroup()
{
    delete m_trigger;
}

void MoveRotateGroup::set_properties(const std::set<ObjectID> & objects)
{
    if(!objects.empty())
    {     
        activate();

        // Reset rotation values if the selection has changed.
        if(objects != m_objects)
        {
            m_x_axis->value(0);
            m_y_axis->value(0);
            m_z_axis->value(0);
        }


        // If we have multiple objects, only world coordinates are allowed.
        if(objects.size() > 1)
        {
            // Set the controls allowed
            m_world_coordinates->value(1);
            m_parent_coordinates->deactivate();
            m_parent_coordinates->value(0);
            m_local_coordinates->deactivate();
            m_local_coordinates->value(0);   
        }
        else
        {
            // Set allowed controls
            m_parent_coordinates->activate();
            m_local_coordinates->activate();
        }

        // This will either set each coordinate input box to 0,
        // or fill them with coordinate information, depending on whether
        // the button is checked or not.
        m_relative_coordinates->do_callback();

        // Lock the scene tree to make sure nothing is modified while we build.
        SceneTreePtr ptr = TheSceneTree::acquire();

        // Lock the selection set as well
        //Lock selection_lock(TheUIUtilities::instance().get_selection_mutex());

        // Make sure all objects in the set are still in the scene...
        std::set<ObjectID>::const_iterator it;        
        for(it = objects.begin(); it != objects.end(); ++it)
        {
            if(!ptr->get_object(*it))
            {
                TheUIUtilities::instance().deselect_object(*it);
                return;
            }
        }

        // Deregister triggers
        deregister_triggers(objects, ptr);        

        // Register new ones
        register_triggers(objects, ptr);
        
        m_objects = objects;

    }
    else     // If the selection is empty, we just deactivate the widget.
        deactivate();
}

// Fills the coordinate boxes with appropriate values, based on the
// coordinate system used.
void MoveRotateGroup::coordinate_choice_callback(Fl_Widget* w, void* data)
{
    // Determine which coordinate system was selected.
    transformation_target::CoordinateSystem system;
    MoveRotateGroup *this_group = reinterpret_cast<MoveRotateGroup*>(data);
    if(w == this_group->m_local_coordinates)
    {
        system = transformation_target::LOCAL_COORDINATES;
        this_group->m_x_coordinate->clear();
        this_group->m_y_coordinate->clear();
        this_group->m_z_coordinate->clear();
    }
    else if(w == this_group->m_parent_coordinates)
    {
        system = transformation_target::PARENT_COORDINATES;
    }
    else //(w == this_group->m_world_coordinates)
    {
        system = transformation_target::WORLD_COORDINATES;
    }

    // Lock the scene tree while we modify the trigger.
    SceneTreePtr ptr = TheSceneTree::acquire();
    
    this_group->m_trigger->set_coordinate_system(system);
}

void MoveRotateGroup::rotational_system_callback(Fl_Widget* w, void* data)
{
    MoveRotateGroup *this_group = reinterpret_cast<MoveRotateGroup*>(data);

    this_group->m_x_axis->clear();
    this_group->m_y_axis->clear();
    this_group->m_z_axis->clear();
}

void MoveRotateGroup::relative_coordinates_callback(Fl_Widget* w, void* data)
{
    Fl_Check_Button *relative_button = reinterpret_cast<Fl_Check_Button*>(w);
    MoveRotateGroup *this_group = reinterpret_cast<MoveRotateGroup*>(data);
    
    // If we uncheck the relative coordinates check box, we force the
    // coordinate system selection callback to be called for whichever
    // coordinate system was previously selected. This fills the value
    // fields with fresh data.
    if(relative_button->value() == 0)
    {
        if(this_group->m_world_coordinates->value() == 1)
            this_group->m_world_coordinates->do_callback();
        else if(this_group->m_local_coordinates->value() == 1)
            this_group->m_local_coordinates->do_callback();
        else if(this_group->m_parent_coordinates->value() == 1)
            this_group->m_parent_coordinates->do_callback();
    }
    else if (relative_button->value() == 1)
    {
        this_group->m_x_coordinate->clear();
        this_group->m_y_coordinate->clear();
        this_group->m_z_coordinate->clear();
    }
}


void MoveRotateGroup::deregister_triggers(const std::set<ObjectID> &objects,
                                          SceneTreePtr &ptr)
{
    // The set difference m_objects - objects gives all objects that
    // were previously selected, but are no longer.
    std::set<ObjectID> tmp;
    std::insert_iterator<std::set<ObjectID> > result(tmp, tmp.begin());

    std::insert_iterator<std::set<ObjectID> > end =
        std::set_difference(m_objects.begin(), m_objects.end(),
                            objects.begin(), objects.end(), result);

    // Deregister triggers for all objects that have been deselected.
    for(std::set<ObjectID>::iterator it = tmp.begin();
        it != tmp.end(); ++it)
    {
        SceneObject *object = ptr->get_object(*it);
        if(object)
            m_trigger->detach_from(object);
    }
}

void MoveRotateGroup::register_triggers(const std::set<ObjectID> &objects,
                                        SceneTreePtr &ptr)
{
    // The set difference objects - m_objects gives all objects that
    // weren't previously selected, but are now.
    std::set<ObjectID> tmp;
    std::insert_iterator<std::set<ObjectID> > result(tmp, tmp.begin());

    std::insert_iterator<std::set<ObjectID> > end =
        std::set_difference(objects.begin(), objects.end(),
                            m_objects.begin(), m_objects.end(), result);

    // Register triggers for all newly selected objects.
    for(std::set<ObjectID>::iterator it = tmp.begin();
        it != tmp.end(); ++it)
    {
        SceneObject *object = ptr->get_object(*it);
        if(object)
            m_trigger->attach_to(object);
    }
}

void MoveRotateGroup::clear_updates()
{
    // Scrap any incoming updates for this group's widgets
    TheUserInterface::instance().remove_updates(m_x_coordinate);
    TheUserInterface::instance().remove_updates(m_y_coordinate);
    TheUserInterface::instance().remove_updates(m_z_coordinate);
    
}

