/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FL/Fl_Input.H>

#include "NameTrigger.hh"
#include "UserInterface.hh"
#include "UpdateActions.hh"
#include "../SceneObject.hh"
#include "../SceneTreePtr.hh"

using namespace peekabot;
using namespace peekabot::gui;


NameTrigger::NameTrigger(Fl_Input* name_field, Fl_Input* meta_name_field)
    : m_name_field(name_field), m_meta_name_field(meta_name_field)
{
}

NameTrigger::~NameTrigger()
{
    SceneTreePtr ptr = TheSceneTree::acquire();
    while( !m_objects.empty() )
        detach_from(*m_objects.begin());
}


void NameTrigger::on_name_set(SceneObject *object)
{
    std::string name = object->get_name();
    
    TheUserInterface::instance().post_action(
        m_name_field, new UpdateInput(m_name_field, name));    
}

void NameTrigger::on_meta_name_set(SceneObject *object)
{
    std::string meta_name = object->get_meta_name();
    
    TheUserInterface::instance().post_action(
        m_meta_name_field, new UpdateInput(m_meta_name_field, meta_name));        
}

void NameTrigger::attach_to(SceneObject *object)
{
    // Add object to the set of registered objects
    if( object && m_objects.insert(object).second )
    {
        // Register triggers
        object->m_on_name_set.connect(
            boost::bind(&NameTrigger::on_name_set, this, _1));
        
        object->m_on_meta_name_set.connect(
            boost::bind(&NameTrigger::on_meta_name_set, this, _1));
        
        object->m_on_subtree_detached.connect(
            boost::bind(&NameTrigger::detach_from, this, _1));
        
        on_name_set(object);
        on_meta_name_set(object);
    }
}

void NameTrigger::detach_from(SceneObject *object)
{
    // Erase object from set of registered objects
    if( object && m_objects.erase(object) > 0 )
    {
        // Deregister triggers
        object->m_on_name_set.disconnect(
            boost::bind(&NameTrigger::on_name_set, this, _1));
        
        object->m_on_meta_name_set.disconnect(
            boost::bind(&NameTrigger::on_meta_name_set, this, _1));
        
        object->m_on_subtree_detached.disconnect(
            boost::bind(&NameTrigger::detach_from, this, _1));
    }
}

