/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_GUI_PROPERTYGROUP_HH
#define __PEEKABOT_GUI_PROPERTYGROUP_HH

#include <set>

#include "CollapsableGroup.hh"
#include "../Types.hh"

namespace peekabot
{
    class SceneObject;

    namespace gui
    {
        class PropertyTrigger;


        /** \brief Interface for property groups.
         *
         * Establishes a common interface for how groups of properties in a \c PropertyFrame should retrieve information from a \c SceneObject.
         *
         * Inherits graphical representation and controls from the \c CollapsableGroup class.
         *
         */



        class PropertyGroup : public CollapsableGroup
        {
        public:
            /** \brief Standard FLTK constructor.
             *
             * \param x The x-coordinate of the top left corner of the group.
             * \param y The y-coordinate of the top left corner of the group.
             * \param w The width of this group.
             * \param h The height of this group.
             * \param l The widget label (for FLTK compatibility)
             */
            PropertyGroup(int x, int y, int w, int h, char* l = 0);
        
            /** \brief Abstract method for filling a \c PropertyGroup with the properties of (an) object(s).
             *
             * This method is called by the \c PropertyVisitor as a new \c SceneObject is visited, or when the selection set changes in some way.
             *
             */
            virtual void set_properties(const std::set<ObjectID> &objects)=0;
        
        protected:
            /** \brief The set of object this object was created with.
             */
            std::set<ObjectID> m_objects;
        };

    }
}
#endif //__PEEKABOT_GUI_PROPERTYGROUP_HH
