/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FL/Fl_Value_Input.H>

#include "ScaleTrigger.hh"
#include "UserInterface.hh"
#include "UpdateActions.hh"
#include "../ScalableObject.hh"
#include "../SceneObject.hh"
#include "../SceneTreePtr.hh"

using namespace peekabot;
using namespace peekabot::gui;

ScaleTrigger::ScaleTrigger(
    Fl_Value_Input* x_scale, 
    Fl_Value_Input* y_scale,
    Fl_Value_Input* z_scale)
    : m_x_scale(x_scale), m_y_scale(y_scale), m_z_scale(z_scale)
{
}

ScaleTrigger::~ScaleTrigger()
{
    SceneTreePtr ptr = TheSceneTree::acquire();
    while( !m_objects.empty() )
        detach_from(*m_objects.begin());
}

void ScaleTrigger::on_scale_set(ScalableObject* sobject)
{
    TheUserInterface::instance().post_action(
        m_x_scale, new UpdateValuator(m_x_scale, sobject->get_x_scale()));
    
    TheUserInterface::instance().post_action(
        m_y_scale, new UpdateValuator(m_y_scale, sobject->get_y_scale()));
    
    TheUserInterface::instance().post_action(
        m_z_scale, new UpdateValuator(m_z_scale, sobject->get_z_scale()));
   
}

void ScaleTrigger::attach_to(SceneObject *object)
{
    // Add object to the set of registered objects
    if( m_objects.insert(object).second )
    {
        ScalableObject *sobject = dynamic_cast<ScalableObject *>(object);
        assert( sobject );

        // Register triggers
        sobject->m_on_scale_set.connect(
            boost::bind(&ScaleTrigger::on_scale_set, this, _1));
        
        object->m_on_subtree_detached.connect(
            boost::bind(&ScaleTrigger::detach_from, this, _1));
        
        on_scale_set(sobject);
    }
}

void ScaleTrigger::detach_from(SceneObject *object)
{
    // Erase object from set of registered objects
    if( m_objects.erase(object) > 0 )
    {
        ScalableObject *sobject = dynamic_cast<ScalableObject *>(object);
        assert( sobject );

        // Deregister triggers
        sobject->m_on_scale_set.disconnect(
            boost::bind(&ScaleTrigger::on_scale_set, this, _1));
        
        object->m_on_subtree_detached.disconnect(
            boost::bind(&ScaleTrigger::detach_from, this, _1));
    }
}

