/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_GUI_SCALE_TRIGGER_HH
#define __PEEKABOT_GUI_SCALE_TRIGGER_HH


#include <set>


class Fl_Value_Input;

namespace peekabot
{

class SceneObject;
class ScalableObject;

namespace gui
{


    /** \brief Trigger for updating the scale fields in a\c ScaleGroup.
     *
     * This trigger is registered when a \c ScaleGroup is created in a \c
     * PropertyFrame. It will contain pointers to the scale fields that it
     * is registered for.
     *
     * It is registered for the ON_SCALE_SET stimulus.
     *
     * When triggered it generates update events and posts them on the GUI update
     * queue via \c UserInterface::post_event().
     *
     */

class ScaleTrigger
{

public:
    /** \brief This constructor simply sets the pointers to x-, y- and z-scale
     * factor fields.
     */
    ScaleTrigger(Fl_Value_Input* x_scale, 
                 Fl_Value_Input* y_scale,
                 Fl_Value_Input* z_scale);

    ~ScaleTrigger();

    void on_scale_set(ScalableObject *sobject);

    void attach_to(SceneObject *object);

    void detach_from(SceneObject *object);

private:
    /** \brief Pointers to the scale fields that the trigger is registered for.
     */
    Fl_Value_Input *m_x_scale, *m_y_scale, *m_z_scale;

    /** \brief A set storing all objects the trigger is registered with.
     */
    std::set<SceneObject *> m_objects;
};


}
}

#endif //__PEEKABOT_GUI_SCALE_TRIGGER_HH
