/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_GUI_TRANSFORMATION_INPUT_HH
#define __PEEKABOT_GUI_TRANSFORMATION_INPUT_HH

#include <FL/Fl_Value_Input.H>
#include <boost/date_time/posix_time/posix_time.hpp>

#include "../Types.hh"

namespace peekabot
{

namespace gui
{
    class PropertyGroup;

    class TransformationInput : public Fl_Value_Input
    {
    public:
        TransformationInput(int x, int y, int w, int h, char* l = 0);
        virtual int handle(int msg);
        void set_group(PropertyGroup* group);
        
        static void enter_value_callback(
            TransformationInput *me, void* data);
        
        void clear();
    protected:
        real_unit m_last_value;

        virtual void on_drag(real_unit delta) = 0;
        virtual void on_release(real_unit delta) = 0;

        PropertyGroup* m_group;
        boost::posix_time::ptime m_last_action_time;
    };
    
    class MoveInput : public TransformationInput
    {
    public:
        MoveInput(int x, int y, int w, int h, char* l = 0);
        void set_coordinate(transformation_target::Coordinate coordinate);
    protected:
        virtual void on_drag(real_unit delta);
        virtual void on_release(real_unit delta);
        transformation_target::Coordinate m_coordinate;
    };

    class RotateInput : public TransformationInput
    {
    public:
        RotateInput(int x, int y, int w, int h, char* l = 0);
        void set_axis(transformation_target::Axis axis);
    protected:
        virtual void on_drag(real_unit delta);
        virtual void on_release(real_unit delta);
        transformation_target::Axis m_axis;
    };

    class ScaleInput : public TransformationInput
    {
    public:
        ScaleInput(int x, int y, int w, int h, char* l = 0);
        
        void set_xyz_links(ScaleInput *x, ScaleInput *y, ScaleInput *z);
        void set_uniform(bool uniform);
    protected:
        virtual void on_drag(real_unit delta);
        virtual void on_release(real_unit delta);

        ScaleInput* m_links[3];     

        bool m_uniform;
    };

    
}
}

#endif // __PEEKABOT_GUI_TRANSFORMATION_INPUT_HH
