/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "UpdateActions.hh"

#include <FL/Fl_Input.H>
#include <FL/Fl_Valuator.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Color_Chooser.H>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>

#include "ViewFrame.hh"

using namespace peekabot;
using namespace gui;

bool UpdateAction::replaces()
{
    return true;
}

void DeleteWidget::execute()
{
    Fl_Group *parent = m_widget->parent();
    if(parent != 0)
    {
        parent->remove(m_widget);
        parent->redraw();
    }

    Fl::delete_widget(m_widget);
}


void UpdateInput::execute()
{
    m_widget->value(m_string.c_str());
}

void UpdateValuator::execute()
{
    m_widget->value(m_value);
}

void UpdateValuatorDomain::execute()
{
    m_widget->range(m_min, m_max);
}

void UpdateButton::execute()
{
    m_widget->value(m_value);
}

void UpdateColor::execute()
{
    m_widget->rgb(m_r, m_g, m_b);
}

void UpdateCamera::execute()
{
    m_widget->add_camera(m_camera_id, m_camera_name);
}

bool UpdateCamera::replaces()
{
    return false;
}

void RemoveCamera::execute()
{
    m_widget->remove_camera(m_camera_id);
}

bool RemoveCamera::replaces()
{
    return false;
}
