/*
 * Copyright Anders Boberg 2006-2007.
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//////////////////////   ViewFrame methods   //////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////

#include <boost/format.hpp>

#include <FL/fl_draw.H>

#include "ViewFrame.hh"
#include "Viewport.hh"
#include "Workspace.hh"
#include "UserInterfaceUtilities.hh"
#include "UserInterface.hh"
#include "IconManager.hh"
#include "CameraPropertyTrigger.hh"
#include "CameraTrigger.hh"
#include "../actions/SetOrthographic.hh"
#include "../actions/SetFOV.hh"
#include "../actions/SetOrientation.hh"
#include "../SceneTree.hh"
#include "../SceneTreePtr.hh"


#include <set>
#include <iostream>
#include <ctype.h>

using namespace peekabot;
using namespace gui;

ViewFrame::ViewFrame(int x, int y, int w, int h, char* l) :
    Frame(x, y, w, h, l)
{
    init(x, y, w, h);
}

void ViewFrame::init(int x, int y, int w, int h)
{
    // Weird hack. For some reason, the parent of the Viewport has to be
    // hidden as it is created, or the Viewport won't show up until the
    // window is hidden and shown again.
    //hide();

    m_camera_selection_menu = new Fl_Choice(x + 30, y, 120, 25);

    
    // Lock the scene tree to make sure nothing is modified while we
    // register triggers.
    {
        //SceneTreePtr ptr = TheSceneTree::acquire();
        
        m_camera_trigger = new CameraTrigger(this);
        
        // This causes the trigger to traverse the scene tree,
        // searching for cameras and registering itself for
        // every node.
        /*m_camera_trigger->on_child_attached(
          ptr->get_root(SourceID(SourceID::GUI_ID, SourceID::SERVER))); */
    }
    
    m_camera_selection_menu->box(FL_UP_BOX);
    
    add(m_camera_selection_menu);

    m_viewport = new Viewport(x + 5, y + 30,
                              w - 10, h - 35);  
       
    add(m_viewport);
    type(0); // No scroll bars

    
    // Create the visibility button group
    {
        int xx = x + 61;
        int yy = y - 30;
        Fl_Group* o = new Fl_Group(xx + 89, yy+30, 127, 25);
        o->box(FL_PLASTIC_UP_FRAME);
        
        m_layer_buttons[0] = new Fl_Button(xx + 90, yy + 31, 15, 12, "1");
        m_layer_buttons[0]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[0]->selection_color(FL_GRAY0);
        m_layer_buttons[0]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[0]->labelsize(8);
        m_layer_buttons[0]->value(1);
        m_layer_buttons[0]->clear_visible_focus();
        m_layer_buttons[0]->callback(layer_button_callback, this);
        

        m_layer_buttons[4] = new Fl_Button(xx + 90, yy + 42, 15, 12, "5");
        m_layer_buttons[4]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[4]->selection_color(FL_GRAY0);
        m_layer_buttons[4]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[4]->labelsize(8);
        m_layer_buttons[4]->clear_visible_focus();
        m_layer_buttons[4]->callback(layer_button_callback, this);    
        

        m_layer_buttons[1] = new Fl_Button(xx + 105, yy + 31, 15, 12, "2");
        m_layer_buttons[1]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[1]->selection_color(FL_GRAY0);
        m_layer_buttons[1]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[1]->labelsize(8);
        m_layer_buttons[1]->clear_visible_focus();
        m_layer_buttons[1]->callback(layer_button_callback, this);    
        
        
        m_layer_buttons[5] = new Fl_Button(xx + 105, yy + 42, 15, 12, "6");
        m_layer_buttons[5]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[5]->selection_color(FL_GRAY0);
        m_layer_buttons[5]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[5]->labelsize(8);
        m_layer_buttons[5]->clear_visible_focus();
        m_layer_buttons[5]->callback(layer_button_callback, this);    

        
        m_layer_buttons[2] = new Fl_Button(xx + 120, yy + 31, 15, 12, "3");
        m_layer_buttons[2]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[2]->selection_color(FL_GRAY0);
        m_layer_buttons[2]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[2]->labelsize(8);
        m_layer_buttons[2]->clear_visible_focus();
        m_layer_buttons[2]->callback(layer_button_callback, this);    

        
        m_layer_buttons[6] = new Fl_Button(xx + 120, yy + 42, 15, 12, "7");
        m_layer_buttons[6]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[6]->selection_color(FL_GRAY0);
        m_layer_buttons[6]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[6]->labelsize(8);
        m_layer_buttons[6]->clear_visible_focus();
        m_layer_buttons[6]->callback(layer_button_callback, this);    

        
        m_layer_buttons[3] = new Fl_Button(xx + 135, yy + 31, 15, 12, "4");
        m_layer_buttons[3]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[3]->selection_color(FL_GRAY0);
        m_layer_buttons[3]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[3]->labelsize(8);
        m_layer_buttons[3]->clear_visible_focus();
        m_layer_buttons[3]->callback(layer_button_callback, this);    

        
        m_layer_buttons[7] = new Fl_Button(xx + 135, yy + 42, 15, 12, "8");
        m_layer_buttons[7]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[7]->selection_color(FL_GRAY0);
        m_layer_buttons[7]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[7]->labelsize(8);
        m_layer_buttons[7]->clear_visible_focus();
        m_layer_buttons[7]->callback(layer_button_callback, this);    

        
        
        m_layer_buttons[8] = new Fl_Button(xx + 155, yy + 31, 15, 12, "9");
        m_layer_buttons[8]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[8]->selection_color(FL_GRAY0);
        m_layer_buttons[8]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[8]->labelsize(8);
        m_layer_buttons[8]->clear_visible_focus();
        m_layer_buttons[8]->callback(layer_button_callback, this);    

        
        m_layer_buttons[12] = new Fl_Button(xx + 155, yy + 42, 15, 12, "13");
        m_layer_buttons[12]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[12]->selection_color(FL_GRAY0);
        m_layer_buttons[12]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[12]->labelsize(8);
        m_layer_buttons[12]->clear_visible_focus();
        m_layer_buttons[12]->callback(layer_button_callback, this);    

        
        m_layer_buttons[9] = new Fl_Button(xx + 170, yy + 31, 15, 12, "10");
        m_layer_buttons[9]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[9]->selection_color(FL_GRAY0);
        m_layer_buttons[9]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[9]->labelsize(8);
        m_layer_buttons[9]->clear_visible_focus();
        m_layer_buttons[9]->callback(layer_button_callback, this);    

        
        m_layer_buttons[13] = new Fl_Button(xx + 170, yy + 42, 15, 12, "14");
        m_layer_buttons[13]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[13]->selection_color(FL_GRAY0);
        m_layer_buttons[13]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[13]->labelsize(8);
        m_layer_buttons[13]->clear_visible_focus();
        m_layer_buttons[13]->callback(layer_button_callback, this);    

        
        m_layer_buttons[10] = new Fl_Button(xx + 185, yy + 31, 15, 12, "11");
        m_layer_buttons[10]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[10]->selection_color(FL_GRAY0);
        m_layer_buttons[10]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[10]->labelsize(8);
        m_layer_buttons[10]->clear_visible_focus();
        m_layer_buttons[10]->callback(layer_button_callback, this);    

        
        m_layer_buttons[14] = new Fl_Button(xx + 185, yy + 42, 15, 12, "15");
        m_layer_buttons[14]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[14]->selection_color(FL_GRAY0);
        m_layer_buttons[14]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[14]->labelsize(8);
        m_layer_buttons[14]->clear_visible_focus();
        m_layer_buttons[14]->callback(layer_button_callback, this);    

        
        m_layer_buttons[11] = new Fl_Button(xx + 200, yy + 31, 15, 12, "12");
        m_layer_buttons[11]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[11]->selection_color(FL_GRAY0);
        m_layer_buttons[11]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[11]->labelsize(8);
        m_layer_buttons[11]->clear_visible_focus();
        m_layer_buttons[11]->callback(layer_button_callback, this);    

        
        m_layer_buttons[15] = new Fl_Button(xx + 200, yy + 42, 15, 12, "16");
        m_layer_buttons[15]->type(FL_TOGGLE_BUTTON);
        m_layer_buttons[15]->selection_color(FL_GRAY0);
        m_layer_buttons[15]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[15]->labelsize(8);
        m_layer_buttons[15]->clear_visible_focus();
        m_layer_buttons[15]->callback(layer_button_callback, this);    

        o->end();        
        add(o);
    }

    m_view_mode_button = new Fl_Button(x+281, y, 25, 25);
    m_view_mode_button->type(FL_TOGGLE_BUTTON);
    m_view_mode_button->clear_visible_focus();
    m_view_mode_button->image(TheIconManager::instance().get_icon(
                                  "ortho_mode.png"));
    m_view_mode_button->callback(view_mode_callback, this);
    m_view_mode_button->tooltip("Toggle orthographic mode");

    m_top_down_button = new Fl_Button(x+306, y, 25, 25);
    m_top_down_button->clear_visible_focus();
    m_top_down_button->image(TheIconManager::instance().get_icon(
                                 "topdown.png"));
    m_top_down_button->callback(top_down_callback, this);
    m_top_down_button->tooltip("Top-down view");

    m_snapshot_button = new Fl_Button(x+331, y, 25, 25);
    m_snapshot_button->clear_visible_focus();
    m_snapshot_button->image(TheIconManager::instance().get_icon(
                                 "snapshot.png"));
    m_snapshot_button->deimage(TheIconManager::instance().get_icon(
                                   "snapshot_disabled.png"));
    m_snapshot_button->callback(snapshot_callback, this);
    m_snapshot_button->tooltip("Take a snapshot");

    m_cont_snapshot_button = new Fl_Button(x+356, y, 25, 25);
    m_cont_snapshot_button->type(FL_TOGGLE_BUTTON);
    m_cont_snapshot_button->clear_visible_focus();
    m_cont_snapshot_button->image(TheIconManager::instance().get_icon(
                                      "cont_snapshot.png"));
    m_cont_snapshot_button->callback(cont_snapshot_callback, this);
    m_cont_snapshot_button->tooltip("Save all rendered frames");
    
    add(m_view_mode_button);
    add(m_top_down_button);
    add(m_snapshot_button);
    add(m_cont_snapshot_button);

    // Trigger for updating the orthographic and wireframe buttons
    m_trigger = new CameraPropertyTrigger(true, m_view_mode_button);

}

ViewFrame::ViewFrame(Frame* frame) :
    Frame(frame->x(), frame->y(), frame->w(), frame->h())
{
    init(frame->x(), frame->y(), frame->w(), frame->h());

    // Register the frame with the currently active workspace
    TheUIUtilities::instance().active_workspace()->register_view_frame(this);
    
    // Since the old frame must have been selected to use the content selection
    // menu, we want the new frame to be selected as well.
    TheUIUtilities::instance().active_workspace()->select_frame(this);

    // Remove the old frame safely.
    frame->deregister();
    frame->hide();
    Fl::delete_widget(frame);
    //  Fl::wait();

    hide();
    show();

}

ViewFrame::~ViewFrame()
{
    delete m_trigger;

    delete m_camera_trigger;

    for(size_t i = 0; i < m_cameras.size(); ++i)
        delete m_cameras[i];

}

void ViewFrame::redraw_viewport()
{
    m_viewport->redraw();
}

void ViewFrame::add_camera(ObjectID camera_id, const std::string & camera_name)
{
    // Store the camera id so we can store pointers to it to the camera
    // selection menu (which only allows a user data of void* type)
    bool not_in_list = true;
    size_t i;
    
    for(i = 0; i < m_cameras.size(); ++i)
    {
        if(*m_cameras[i] == camera_id)
        {
            not_in_list = false;

            // If the camera exists, we update the name of the camera.
            const Fl_Menu_Item *menu = m_camera_selection_menu->menu();
            for(int j = 0; j < m_camera_selection_menu->size(); ++j)
            {
                // The user_data_ member should point to the camera id.
                if(menu[j].user_data_ == m_cameras[i])
                {
                    // Fl_Menu_::replace() seems to corrupt user data, so
                    // we use remove and add.
                   m_camera_selection_menu->remove(j);

                   // Note: item labels must be unique in the menu, so
                   // if there is already an item with the same name, we append
                   // the ObjectID within parantheses.
                   if(m_camera_selection_menu->find_item(
                          camera_name.c_str()) != 0)
                   {
                       std::string camera_string = (boost::format("%1%(%2%)") %
                                                    camera_name % camera_id).str();
                       m_camera_selection_menu->add(
                           camera_string.c_str(), 0,
                           camera_selection_callback, m_cameras[i]);                       
                   }   
                   else
                   {
                       m_camera_selection_menu->add(
                           camera_name.c_str(), 0,
                           camera_selection_callback, m_cameras[i]);
                   }
                }
            }
            break;
        }
    }

    if(not_in_list) 
    {
        // i == m_cameras.size() - the index of the next element to be inserted
        m_cameras.push_back(new ObjectID(camera_id));

        // Note: item labels must be unique in the menu, so
        // if there is already an item with the same name, we append
        // the ObjectID within parantheses.
        if(m_camera_selection_menu->find_item(
               camera_name.c_str()) != 0)
        {
            std::string camera_string = (boost::format("%1%(%2%)") %
                                         camera_name % camera_id).str();
            m_camera_selection_menu->add(
                camera_string.c_str(), 0,
                camera_selection_callback, m_cameras[i]);                       
        }   
        else
        {
            m_camera_selection_menu->add(
                camera_name.c_str(), 0,
                camera_selection_callback, m_cameras[i]);
        }

        // If the size is 2, it means this is the first camera that's been added
        // to the menu, so select it.
        if(m_camera_selection_menu->size() == 2)
            m_camera_selection_menu->menu()->do_callback(
                m_camera_selection_menu);        
    }

}

void ViewFrame::remove_camera(ObjectID camera_id)
{
    // Check if the id is in the list of camera ids
    for(std::vector<ObjectID*>::iterator it = m_cameras.begin();
        it != m_cameras.end(); ++it)
    {
        // if it is, proceed to remove it from the menu.
        if(**it == camera_id)
        {
            for(int j = 0; j < m_camera_selection_menu->size() - 1; ++j)
            {
                const Fl_Menu_Item *menu = m_camera_selection_menu->menu();

                // The user_data_ member should point to the camera id.
                if(menu[j].user_data_ == *it)
                {
                    // Remove the camera.
                    m_camera_selection_menu->remove(j);
                    
                    // If the previously selected camera was removed,
                    // select a new one if possible.
                    if(m_camera_selection_menu->value() == j)
                    {
                        if(m_camera_selection_menu->size() > 1)
                        {
                            m_camera_selection_menu->value(0);
                            m_camera_selection_menu->menu()->do_callback(
                                m_camera_selection_menu);
                        }
                        else
                        {
                            m_camera_selection_menu->value(0);
                            //m_camera_selection_menu->menu()->do_callback(
                            //    m_camera_selection_menu);
                            m_viewport->set_camera(INVALID_ID);
                        }
                    }

                    break;
                }
            }

            // Remove from the list of camera ids...
            delete *it;
            m_cameras.erase(it);

            break;
        }
    }


}


void ViewFrame::camera_selection_callback(Fl_Widget* w, void * data)
{
    Fl_Choice *camera_selection_button = (Fl_Choice*)w;
    ViewFrame *view_frame = (ViewFrame*)camera_selection_button->parent();
    
    // Check to see that there is at least one camera in the camera menu.
    if(camera_selection_button->size() > 1)
    {
        const Fl_Menu_Item *menu = camera_selection_button->mvalue();
        
        // If a camera has been selected, menu points to the Fl_Menu_Item
        // corresponding to that camera.
        if(menu != 0)
        {
            //view_frame->m_viewport->hide();

            // We also need to register our trigger with the new camera,
            // and deregister it from the old one.

            
            // Lock the scene tree to make sure nothing is modified while we
            // register triggers.
            {
                SceneTreePtr ptr = TheSceneTree::acquire();

                view_frame->deregister_trigger(
                    view_frame->m_viewport->get_camera(), ptr);
                view_frame->register_trigger(
                    *static_cast<ObjectID*>(menu->user_data_), ptr);
            }
            
            view_frame->m_viewport->set_camera(
                *static_cast<ObjectID*>(menu->user_data_));

            //view_frame->m_viewport->show();
        }
        // If no camera has been selected, select the first camera in the list.
        else
        {
            camera_selection_button->value(0);

             ObjectID id = *static_cast<ObjectID*>(
                camera_selection_button->menu()->user_data_);
            // Lock the scene tree to make sure nothing is modified while we
            // register triggers.
            {
                SceneTreePtr ptr = TheSceneTree::acquire();

                view_frame->deregister_trigger(
                    view_frame->m_viewport->get_camera(), ptr);
                view_frame->register_trigger(id, ptr);
            }

            view_frame->m_viewport->set_camera(
                *static_cast<ObjectID*>(camera_selection_button->menu()->user_data_));
        }
    }
    else
        view_frame->m_viewport->set_camera(INVALID_ID);
    
    view_frame->m_viewport->redraw();
}


void ViewFrame::deregister()
{
    Workspace* ws = (Workspace*)parent();
    
    // Scrap any incoming updates for this frame's widgets
    TheUserInterface::instance().remove_updates(m_view_mode_button);

    ws->deregister_view_frame(this);
}

Frame::FrameType ViewFrame::frame_type() const
{
    return Frame::VIEW_FRAME;
}

void ViewFrame::resize(int xx, int yy, int ww, int hh)
{
    Fl_Scroll::resize(xx, yy, ww, hh);
    m_viewport->resize(xx + 5, yy + 30, ww - 10, hh - 35);    
    //   m_viewport->redraw();
}


bool ViewFrame::resizable_width(int width)
{
    return 50;
}

bool ViewFrame::resizable_height(int height)
{
    return 50;
}

Frame* ViewFrame::clone(int x, int y, int w, int h)
{
    return new ViewFrame(x, y, w, h);
}

void ViewFrame::layer_button_callback(Fl_Widget* w, void* data)
{
    ViewFrame* frame = static_cast<ViewFrame*>(data);
    
    for(unsigned int i = 0; i < NUMBER_OF_LAYERS; ++i)
    {
        frame->m_viewport->m_visible_layers[i] =
            frame->m_layer_buttons[i]->value() == 1;
    }
    
    frame->m_viewport->redraw();
}

void ViewFrame::top_down_callback(Fl_Widget* w, void* data)
{
    ViewFrame* frame = static_cast<ViewFrame*>(data);

    ObjectID camera = frame->m_viewport->get_camera();

    TheUIUtilities::instance().post_action(
        new SetOrientation(camera, Vector3ru(0,0,-1), WORLD_COORDINATES));
}

void ViewFrame::view_mode_callback(Fl_Widget* w, void* data)
{
    ViewFrame* frame = static_cast<ViewFrame*>(data);
    ObjectID camera = frame->m_viewport->get_camera();

    bool orthographic;

    if(frame->m_view_mode_button->value() == 1)
        orthographic = true;
    else
        orthographic = false;

    TheUIUtilities::instance().post_action(
        new SetOrthographic(camera, orthographic)); 
}

void ViewFrame::snapshot_callback(Fl_Widget* w, void* data)
{
    ViewFrame* frame = static_cast<ViewFrame*>(data);

    frame->m_viewport->save_snapshot();
}

void ViewFrame::cont_snapshot_callback(Fl_Widget* w, void* data)
{
    ViewFrame* frame = static_cast<ViewFrame*>(data);

    bool enabled = (bool)dynamic_cast<Fl_Button *>(w)->value();

    if( enabled )
        frame->m_snapshot_button->deactivate();
    else
        frame->m_snapshot_button->activate();

    frame->m_viewport->toggle_continual_snapshots(enabled);
}


void ViewFrame::register_trigger(ObjectID id, SceneTreePtr &ptr)
{
    SceneObject *object = ptr->get_object(id);
    if( dynamic_cast<CameraObject *>(object) )
        m_trigger->attach_to(object);
}

void ViewFrame::deregister_trigger(ObjectID id, SceneTreePtr &ptr)
{
    SceneObject *object = ptr->get_object(id);
    if( dynamic_cast<CameraObject *>(object) )
        m_trigger->detach_from(object);
}
