/*
 * Copyright Anders Boberg 2006-2007.
 * Copyright Staffan Gimåker 2007-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "NameTrigger.hh"
#include "VisibilityGroup.hh"
#include "VisibilityTrigger.hh"
#include "UserInterfaceUtilities.hh"
#include "UserInterface.hh"
#include "../SceneObject.hh"
#include "../SceneTreePtr.hh"
#include "../actions/SetOpacity.hh"
#include "../actions/SetLayer.hh"
#include "../actions/SetVisibility.hh"

using namespace peekabot;
using namespace peekabot::gui;
using namespace boost::posix_time;

VisibilityGroup::VisibilityGroup(int x, int y) :
    PropertyGroup(x, y, 230, 190, "Visibility"),
    m_last_action_time(microsec_clock::local_time())
{
    hide();
    begin();


    {
        Fl_Group* o = new Fl_Group(x + 85, y+25, 135, 70, "Layer");
        o->align(FL_ALIGN_INSIDE | FL_ALIGN_TOP);
        o->box(FL_PLASTIC_UP_FRAME);
        
        m_layer_buttons[0] = new Fl_Button(x + 90, y + 45, 15, 12, "1");
        m_layer_buttons[0]->type(FL_RADIO_BUTTON);
        m_layer_buttons[0]->selection_color(FL_GRAY0);
        m_layer_buttons[0]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[0]->labelsize(8);
        m_layer_buttons[0]->callback(set_layer_callback, this);
        

        m_layer_buttons[4] = new Fl_Button(x + 90, y + 57, 15, 12, "5");
        m_layer_buttons[4]->type(FL_RADIO_BUTTON);
        m_layer_buttons[4]->selection_color(FL_GRAY0);
        m_layer_buttons[4]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[4]->labelsize(8);
        m_layer_buttons[4]->callback(set_layer_callback, this);    
        

        m_layer_buttons[1] = new Fl_Button(x + 105, y + 45, 15, 12, "2");
        m_layer_buttons[1]->type(FL_RADIO_BUTTON);
        m_layer_buttons[1]->selection_color(FL_GRAY0);
        m_layer_buttons[1]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[1]->labelsize(8);
        m_layer_buttons[1]->callback(set_layer_callback, this);    
        
        
        m_layer_buttons[5] = new Fl_Button(x + 105, y + 57, 15, 12, "6");
        m_layer_buttons[5]->type(FL_RADIO_BUTTON);
        m_layer_buttons[5]->selection_color(FL_GRAY0);
        m_layer_buttons[5]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[5]->labelsize(8);
        m_layer_buttons[5]->callback(set_layer_callback, this);    

        
        m_layer_buttons[2] = new Fl_Button(x + 120, y + 45, 15, 12, "3");
        m_layer_buttons[2]->type(FL_RADIO_BUTTON);
        m_layer_buttons[2]->selection_color(FL_GRAY0);
        m_layer_buttons[2]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[2]->labelsize(8);
        m_layer_buttons[2]->callback(set_layer_callback, this);    

        
        m_layer_buttons[6] = new Fl_Button(x + 120, y + 57, 15, 12, "7");
        m_layer_buttons[6]->type(FL_RADIO_BUTTON);
        m_layer_buttons[6]->selection_color(FL_GRAY0);
        m_layer_buttons[6]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[6]->labelsize(8);
        m_layer_buttons[6]->callback(set_layer_callback, this);    

        
        m_layer_buttons[3] = new Fl_Button(x + 135, y + 45, 15, 12, "4");
        m_layer_buttons[3]->type(FL_RADIO_BUTTON);
        m_layer_buttons[3]->selection_color(FL_GRAY0);
        m_layer_buttons[3]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[3]->labelsize(8);
        m_layer_buttons[3]->callback(set_layer_callback, this);    

        
        m_layer_buttons[7] = new Fl_Button(x + 135, y + 57, 15, 12, "8");
        m_layer_buttons[7]->type(FL_RADIO_BUTTON);
        m_layer_buttons[7]->selection_color(FL_GRAY0);
        m_layer_buttons[7]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[7]->labelsize(8);
        m_layer_buttons[7]->callback(set_layer_callback, this);    

        
        
        m_layer_buttons[8] = new Fl_Button(x + 155, y + 45, 15, 12, "9");
        m_layer_buttons[8]->type(FL_RADIO_BUTTON);
        m_layer_buttons[8]->selection_color(FL_GRAY0);
        m_layer_buttons[8]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[8]->labelsize(8);
        m_layer_buttons[8]->callback(set_layer_callback, this);    

        
        m_layer_buttons[12] = new Fl_Button(x + 155, y + 57, 15, 12, "13");
        m_layer_buttons[12]->type(FL_RADIO_BUTTON);
        m_layer_buttons[12]->selection_color(FL_GRAY0);
        m_layer_buttons[12]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[12]->labelsize(8);
        m_layer_buttons[12]->callback(set_layer_callback, this);    

        
        m_layer_buttons[9] = new Fl_Button(x + 170, y + 45, 15, 12, "10");
        m_layer_buttons[9]->type(FL_RADIO_BUTTON);
        m_layer_buttons[9]->selection_color(FL_GRAY0);
        m_layer_buttons[9]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[9]->labelsize(8);
        m_layer_buttons[9]->callback(set_layer_callback, this);    

        
        m_layer_buttons[13] = new Fl_Button(x + 170, y + 57, 15, 12, "14");
        m_layer_buttons[13]->type(FL_RADIO_BUTTON);
        m_layer_buttons[13]->selection_color(FL_GRAY0);
        m_layer_buttons[13]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[13]->labelsize(8);
        m_layer_buttons[13]->callback(set_layer_callback, this);    

        
        m_layer_buttons[10] = new Fl_Button(x + 185, y + 45, 15, 12, "11");
        m_layer_buttons[10]->type(FL_RADIO_BUTTON);
        m_layer_buttons[10]->selection_color(FL_GRAY0);
        m_layer_buttons[10]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[10]->labelsize(8);
        m_layer_buttons[10]->callback(set_layer_callback, this);    

        
        m_layer_buttons[14] = new Fl_Button(x + 185, y + 57, 15, 12, "15");
        m_layer_buttons[14]->type(FL_RADIO_BUTTON);
        m_layer_buttons[14]->selection_color(FL_GRAY0);
        m_layer_buttons[14]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[14]->labelsize(8);
        m_layer_buttons[14]->callback(set_layer_callback, this);    

        
        m_layer_buttons[11] = new Fl_Button(x + 200, y + 45, 15, 12, "12");
        m_layer_buttons[11]->type(FL_RADIO_BUTTON);
        m_layer_buttons[11]->selection_color(FL_GRAY0);
        m_layer_buttons[11]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[11]->labelsize(8);
        m_layer_buttons[11]->callback(set_layer_callback, this);    

        
        m_layer_buttons[15] = new Fl_Button(x + 200, y + 57, 15, 12, "16");
        m_layer_buttons[15]->type(FL_RADIO_BUTTON);
        m_layer_buttons[15]->selection_color(FL_GRAY0);
        m_layer_buttons[15]->labeltype(FL_ENGRAVED_LABEL);
        m_layer_buttons[15]->labelsize(8);
        m_layer_buttons[15]->callback(set_layer_callback, this);    

        m_apply_recursively = new Fl_Check_Button(x + 90, y + 70, 20, 20,
                                                  "Set recursively");
        m_apply_recursively->down_box(FL_DOWN_BOX);
        m_apply_recursively->align(FL_ALIGN_RIGHT);

        o->end();
    }

    
    m_hidden = new Fl_Check_Button(x + 10, y + 55, 20, 20, "Hidden");
    m_hidden->down_box(FL_DOWN_BOX);
    m_hidden->callback(hide_callback, this);
    
    Fl_Box *opacity_border = new Fl_Box(x+10, y + 105, 210, 75, "Opacity");
    opacity_border->box(FL_UP_FRAME);
    opacity_border->labeltype(FL_NORMAL_LABEL);
    opacity_border->align(FL_ALIGN_INSIDE | FL_ALIGN_TOP);
    
    m_opacity_slider = new Fl_Slider(x + 15, y + 140, 200, 15);
    m_opacity_slider->type(3);
    m_opacity_slider->bounds(0, 1);
    m_opacity_slider->selection_color((Fl_Color)229);
    m_opacity_slider->callback(set_opacity_callback, this);
    
    
    m_accumulated_opacity_slider = new Fl_Slider(x + 15, y + 160, 200, 15);
    m_accumulated_opacity_slider->type(3);
    m_accumulated_opacity_slider->bounds(0, 1);
    m_accumulated_opacity_slider->selection_color((Fl_Color)229);
    m_accumulated_opacity_slider->deactivate();
    
    m_absolute_opacity = new Fl_Check_Button(x + 15, y + 120, 20, 20,
                                             "Absolute");
    m_absolute_opacity->down_box(FL_DOWN_BOX);
    m_absolute_opacity->align(FL_ALIGN_RIGHT);
    m_absolute_opacity->callback(set_opacity_callback, this);    
    
    end();
    set_open(false);
    show();

    m_trigger = new VisibilityTrigger(m_opacity_slider, m_accumulated_opacity_slider,
                                      m_absolute_opacity,
                                      m_hidden, m_layer_buttons);

    set_properties(TheUIUtilities::instance().get_selection());
}

VisibilityGroup::~VisibilityGroup()
{
    delete m_trigger;
}


void VisibilityGroup::set_properties(const std::set<ObjectID> &objects)
{
    if(!objects.empty())
    {
        activate();

        // Lock the scene tree and the selection set.
        SceneTreePtr ptr = TheSceneTree::acquire();
               
        // Deregister old triggers and register new ones...
        deregister_triggers(objects, ptr);
        register_triggers(objects, ptr);
        
        m_objects = objects; 
    }
    else
        deactivate();
}


void VisibilityGroup::deregister_triggers(const std::set<ObjectID> &objects,
                                          SceneTreePtr &ptr)
{
    std::set<ObjectID> tmp;
    std::insert_iterator<std::set<ObjectID> > result(tmp, tmp.begin());

    std::insert_iterator<std::set<ObjectID> > end =
        std::set_difference(m_objects.begin(), m_objects.end(),
                            objects.begin(), objects.end(), result);

    for(std::set<ObjectID>::iterator it = tmp.begin();
        it != tmp.end(); ++it)
    {
        SceneObject *object = ptr->get_object(*it);
        if(object)
            m_trigger->detach_from(object);
    }
}

void VisibilityGroup::register_triggers(const std::set<ObjectID> &objects,
                                        SceneTreePtr &ptr)
{
    std::set<ObjectID> tmp;
    std::insert_iterator<std::set<ObjectID> > result(tmp, tmp.begin());

    std::insert_iterator<std::set<ObjectID> > end =
        std::set_difference(objects.begin(), objects.end(),
                            m_objects.begin(), m_objects.end(), result);

    for(std::set<ObjectID>::iterator it = tmp.begin();
        it != tmp.end(); ++it)
    {
        SceneObject *object = ptr->get_object(*it);
        if(object)
            m_trigger->attach_to(object);
    }
}

void VisibilityGroup::set_opacity_callback(Fl_Widget* w, void* data)
{
    VisibilityGroup* group = static_cast<VisibilityGroup*>(data);
    
    // Lock selection set
    Lock selection_lock(TheUIUtilities::instance().get_selection_mutex());
    std::set<ObjectID> objects = TheUIUtilities::instance().get_selection();

    
    // Check to see that at least one action interval has passed
    // since our last visit here, else bail out.
    ptime current_time = microsec_clock::local_time();
    if(current_time < group->m_last_action_time +
       TheUserInterface::instance().get_action_interval())
        return;
    group->m_last_action_time = current_time;

    Opacity opacity;

    opacity.is_absolute = group->m_absolute_opacity->value() == 1;
    opacity.opacity = (real_unit)group->m_opacity_slider->value();
    
    for(std::set<ObjectID>::iterator it = objects.begin();
        it != objects.end(); ++it)
        TheUIUtilities::instance().post_action(
            new SetOpacity(*it, opacity));
}

void VisibilityGroup::set_layer_callback(Fl_Widget* w, void* data)
{
    VisibilityGroup* group = static_cast<VisibilityGroup*>(data);
    
    // Lock selection set
    Lock selection_lock(TheUIUtilities::instance().get_selection_mutex());
    std::set<ObjectID> objects = TheUIUtilities::instance().get_selection();

    // Find out which button was pressed...
    uint32_t layer = 0;
    for(int i = 0; i < 16; ++i)
        if(group->m_layer_buttons[i] == w)
            layer = i;

    // Get the status of the set recursively checkbox
    bool recursive = group->m_apply_recursively->value() == 1;

    for(std::set<ObjectID>::iterator it = objects.begin();
        it != objects.end(); ++it)
        TheUIUtilities::instance().post_action(
            new SetLayer(*it, layer, recursive));
}

void VisibilityGroup::hide_callback(Fl_Widget* w, void* data)
{
    VisibilityGroup* group = static_cast<VisibilityGroup*>(data);
    
    // Lock selection set
    Lock selection_lock(TheUIUtilities::instance().get_selection_mutex());
    std::set<ObjectID> objects = TheUIUtilities::instance().get_selection();

    // Get the status of the hide checkbox.
    bool hide = group->m_hidden->value() == 1;

    // Apply changes to all selected objects.
    for(std::set<ObjectID>::iterator it = objects.begin();
        it != objects.end(); ++it)
        TheUIUtilities::instance().post_action(
            new SetVisibility(*it, !hide));
    
}

void VisibilityGroup::clear_updates()
{
    // Scrap any incoming updates for this group's widgets
    TheUserInterface::instance().remove_updates(m_opacity_slider);
    TheUserInterface::instance().remove_updates(m_accumulated_opacity_slider);
    TheUserInterface::instance().remove_updates(m_hidden);

    for(unsigned int i = 0; i < NUMBER_OF_LAYERS; ++i)
        TheUserInterface::instance().remove_updates(m_layer_buttons[i]);
}
