/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Frame.hh"
#include "PropertyFrame.hh"
#include "ViewFrame.hh"
#include "BrowserFrame.hh"
#include "ConsoleFrame.hh"
#include "Workspace.hh"
#include "UserInterfaceUtilities.hh"

#include <iostream>

using namespace peekabot;
using namespace gui;

FrameSelector::FrameSelector() : m_selected_frame(0) {}

Workspace::Workspace(int x, int y, int w, int h, char* l) :
    Fl_Tile(x, y, w, h, l)
{
    end();
}

Workspace::~Workspace()
{
    TheUIUtilities::instance().activate_workspace(0);
}

void FrameSelector::select_frame(Frame* frame)
{
    if(m_selected_frame != 0)
    {
        // Update the previously selected frame to erase its border
        m_selected_frame->selected(false);
        m_selected_frame->redraw();
    }

    // Select the new frame and draw its border.
    frame->selected(true);
    m_selected_frame = frame;
    frame->redraw();
}

Frame* FrameSelector::get_selected_frame() const
{
    return m_selected_frame;
}

void Workspace::register_property_frame(PropertyFrame *frame)
{
    add(frame);
    m_property_frames.insert(frame);
}

void Workspace::register_view_frame(ViewFrame *frame)
{
    add(frame);
    m_view_frames.insert(frame);
}

void Workspace::register_browser_frame(BrowserFrame *frame)
{
    add(frame);
    m_browser_frames.insert(frame);
}

void Workspace::register_console_frame(ConsoleFrame *frame)
{
    add(frame);
    m_console_frames.insert(frame);
}

void Workspace::deregister_property_frame(PropertyFrame *frame)
{
    remove(*frame);
    m_property_frames.erase(frame);
}

void Workspace::deregister_view_frame(ViewFrame *frame)
{
    remove(*frame);
    m_view_frames.erase(frame);
}

void Workspace::deregister_browser_frame(BrowserFrame *frame)
{
    remove(*frame);
    m_browser_frames.erase(frame);
}

void Workspace::deregister_console_frame(ConsoleFrame *frame)
{
    remove(*frame);
    m_console_frames.erase(frame);
}

std::set<PropertyFrame*> & Workspace::get_property_frames() 
{
    return m_property_frames;
}

std::set<ViewFrame*> & Workspace::get_view_frames() 
{
    return m_view_frames;
}

std::set<BrowserFrame*> & Workspace::get_browser_frames() 
{
    return m_browser_frames;
}

std::set<ConsoleFrame*> & Workspace::get_console_frames() 
{
    return m_console_frames;
}

