/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <stdexcept>

#include "Cullable.hh"
#include "SceneNode.hh"
#include "IDTranslator.hh"


using namespace peekabot::renderer;


namespace local
{
    static Cullable::Renderables renderables;

    static void callback_fun(Renderable *r)  throw()
    {
        renderables.push_back(r);
    }
}


Cullable::Cullable(OnScreenSize *on_screen_size)
    : m_parent(0),
      m_on_screen_size(on_screen_size)
{
    set_bounding_sphere(BoundingSphere(Vector3r(0,0,0), 0));
}

Cullable::~Cullable()
{
    // Remove any aliases registered for this cullable
    TheIDTranslator::instance().remove_alias(this);

    /*if( m_parent )
        m_parent->remove_cullable(this);*/

    if( m_on_screen_size )
    {
        delete m_on_screen_size;
        m_on_screen_size = 0;
    }
}

SceneNode *Cullable::get_parent() throw()
{
    return m_parent;
}

const SceneNode *Cullable::get_parent() const throw()
{
    return m_parent;
}


Cullable::Renderables Cullable::get_renderables() throw()
{
    get_renderables(0, &local::callback_fun);
    Renderables ret = local::renderables;
    local::renderables.clear();

    return ret;
}


const BoundingSphere &Cullable::get_bounding_sphere() throw()
{
    return m_bsphere;
}


const BoundingVolume &Cullable::get_bounding_volume() throw()
{
    return get_bounding_sphere();
}

void Cullable::set_bounding_sphere(const BoundingSphere &bsphere) throw()
{
    m_bsphere = bsphere;

    on_bv_set(this);
}

void Cullable::transform_geometry(
    const boost::function<Vector3r (const Vector4r &)> &fun)
{
    throw std::runtime_error(
        "Geometry transformations are not implemented for this type of Cullable.");
}

void Cullable::set_parent(SceneNode *parent) throw()
{
    m_parent = parent;
}
