/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_RENDERER_PRIMITIVES_MESH_CULLABLE_HH
#define __PEEKABOT_RENDERER_PRIMITIVES_MESH_CULLABLE_HH


#include <boost/shared_ptr.hpp>
#include "../Types.hh"
#include "Cullable.hh"
#include "VertexBased.hh"
#include "LineBased.hh"
#include "IndexBased.hh"



namespace peekabot
{
    namespace renderer
    {
        class Mesh;
        
        namespace primitives
        {
            class Camera;
        }

        class MeshCullable : public Cullable, 
                             public VertexBased
        {
        public:
            MeshCullable(boost::shared_ptr<Mesh> mesh) throw();

            virtual ~MeshCullable();

            virtual void get_renderables(
                primitives::Camera *camera, EnumerationCallback callback) throw();

            virtual void transform_geometry(
                const boost::function<Vector3r (const Vector4r &)> &fun) throw();

            virtual void set_vertices(
                const VertexBased::Vertices &vertices) throw();
                
            virtual void add_vertices(
                const VertexBased::Vertices &vertices) throw();

            virtual void get_vertices(VertexBased::Vertices &vertices) const throw();

            boost::shared_ptr<Mesh> get_mesh() throw();

            const boost::shared_ptr<Mesh> get_mesh() const throw();

        protected:
            virtual void calculate_bvs() throw();

            void calculate_bvs(const VertexBased::Vertices &vertices) throw();
    
        protected:
            boost::shared_ptr<Mesh> m_mesh;
        };


        class PointMeshCullable : public MeshCullable
        {
        public:
            PointMeshCullable(boost::shared_ptr<Mesh> mesh) throw();

            PointMeshCullable(const PointMeshCullable &x) throw();

            virtual ~PointMeshCullable();

            virtual PointMeshCullable *clone() const throw();
        };


        class LineMeshCullable : public MeshCullable, 
                                 public LineBased
        {
        public:
            LineMeshCullable(boost::shared_ptr<Mesh> mesh) throw();

            LineMeshCullable(const LineMeshCullable &x) throw();

            virtual ~LineMeshCullable();

            virtual LineMeshCullable *clone() const throw();

            virtual void set_line_style(
                float line_width, 
                uint16_t stipple_pattern, 
                int stipple_factor);
        };


        class TriMeshCullable : public MeshCullable,
                                public IndexBased
        {
        public:
            TriMeshCullable(boost::shared_ptr<Mesh> mesh) throw();

            TriMeshCullable(const TriMeshCullable &x) throw();

            virtual ~TriMeshCullable();

            virtual TriMeshCullable *clone() const throw();

            virtual void set_indices(const Indices &indices);

            virtual void add_indices(const Indices &indices);

            virtual void get_indices(Indices &indices) const;
        };
    }
}


#endif // __PEEKABOT_RENDERER_PRIMITIVES_MESH_CULLABLE_HH
