/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_RENDERER_OCTREE_TRAVERSER_HH
#define __PEEKABOT_RENDERER_OCTREE_TRAVERSER_HH


#include "Cullable.hh"


namespace peekabot
{
    namespace renderer
    {


        /**
         * \brief Interface class used for traversing an Octree. 
         *
         * Usually it is used to traverse the potentially visible part (often 
         * referred to as the PVS) of the Octree.
         */
        class OctreeTraverser
        {
        public:
            virtual ~OctreeTraverser() {}

            /**
             * \brief Called for every traversed cullable in the Octree.
             */
            virtual void traverse(Cullable *cullable) = 0;
        };


    }
}


#endif // __PEEKABOT_RENDERER_OCTREE_TRAVERSER_HH
