/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <GL/glew.h>
#include "PngLoader.hh"
#include <boost/xpressive/xpressive.hpp>
#include <boost/scoped_array.hpp>
#include "../png++/image.hpp"


using namespace peekabot;
using namespace peekabot::renderer;


PngLoader::Informer PngLoader::ms_informer;


boost::shared_ptr<Texture> 
PngLoader::get(const std::string &id)
{
    png::image<png::rgb_pixel> img(id);
    int32_t w = static_cast<int32_t>(img.get_width());
    int32_t h = static_cast<int32_t>(img.get_height());

    boost::scoped_array<uint8_t> data(new uint8_t[3*w*h]);

    for( int32_t j = 0; j < h; ++j )
        for( int32_t i = 0; i < w; ++i )
        {
            data[3*(w*j+i)+0] = img.get_pixel(i, j).red;
            data[3*(w*j+i)+1] = img.get_pixel(i, j).green;
            data[3*(w*j+i)+2] = img.get_pixel(i, j).blue;
        }

    return boost::shared_ptr<Texture>(
        new Texture(w, h, GL_RGB, data.get()));
}

bool PngLoader::matches(const std::string &id) throw()
{
    static const boost::xpressive::sregex match_exp = 
        boost::xpressive::sregex::compile(".*\\.png");
    return boost::xpressive::regex_match(id, match_exp);
}

