/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_RENDERER_PRIMITIVE_BATCH_HH
#define __PEEKABOT_RENDERER_PRIMITIVE_BATCH_HH


#include <boost/shared_ptr.hpp>

#include "Renderable.hh"
#include "../Types.hh"
#include "../Vector.hh"


namespace peekabot
{
    namespace renderer
    {
        class VertexBuffer;

        class PrimitiveBatch : public Renderable
        {
        public:
            PrimitiveBatch(size_t chunk_size, RenderContext &context);

            void render_point(
                const Vector3f &v, 
                const RGBColor &color) throw();

            void render_line(
                const Vector3f &v1, 
                const Vector3f &v2,
                const RGBColor &color) throw();

            void render_unlit_tri(
                const Vector3f &v1, 
                const Vector3f &v2,
                const Vector3f &v3,
                const RGBColor &color) throw();

            void render_unlit_quad(
                const Vector3f &v1, 
                const Vector3f &v2,
                const Vector3f &v3,
                const Vector3f &v4,
                const RGBColor &color) throw();
            
            void flush() throw();

            virtual void render(RenderContext &context) throw();
            
        private:
            void flush_points() throw();

            void flush_lines() throw();

            void flush_unlit_tris() throw();

        private:
            const size_t m_chunk_size;

            struct Chunk
            {
                Chunk(size_t chunk_size);

                size_t m_avail;
                boost::shared_ptr<VertexBuffer> m_vertices;
                boost::shared_ptr<VertexBuffer> m_colors;
            };

            Chunk m_points;
            Chunk m_lines;
            Chunk m_unlit_tris;

            RenderContext &m_render_context;
        };
    }
}


#endif // __PEEKABOT_RENDERER_PRIMITIVE_BATCH_HH
