/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_RENDERER_RENDER_CONTEXT_HH
#define __PEEKABOT_RENDERER_RENDER_CONTEXT_HH


#include "PrimitiveBatch.hh"


namespace peekabot
{
    namespace renderer
    {
        namespace primitives { class Camera; }

        class RenderContext
        {
        public:
            RenderContext(const primitives::Camera *camera, size_t n_batches)
                : m_camera(camera),
                  m_n_batches(n_batches),
                  m_batches(0)
            {
                if( m_n_batches )
                {
                    m_batches = new PrimitiveBatch*[m_n_batches];

                    for( size_t i = 0; i < m_n_batches; ++i )
                        m_batches[i] = new PrimitiveBatch(2048, *this);
                }
            }

            virtual ~RenderContext()
            {
                if( m_batches )
                {
                    flush();

                    for( size_t i = 0; i < m_n_batches; ++i )
                        delete m_batches[i];

                    delete[] m_batches;
                    m_batches = 0;
                }
            }

            inline const primitives::Camera *get_camera() const throw()
            {
                return m_camera;
            }

            size_t batch_count() const throw()
            {
                return m_n_batches;
            }

            PrimitiveBatch &get_batch(size_t i) throw()
            {
                return *m_batches[i];
            }

            void flush() throw()
            {
                if( m_batches )
                    for( size_t i = 0; i < m_n_batches; ++i )
                        m_batches[i]->flush();
            }

        private:
            const primitives::Camera *m_camera;

            size_t m_n_batches;
            
            PrimitiveBatch **m_batches;
        };
    }
}


#endif // __PEEKABOT_RENDERER_RENDER_CONTEXT_HH
