/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_RENDERER_SCENE_ACTION_HH
#define __PEEKABOT_RENDERER_SCENE_ACTION_HH


namespace peekabot
{
    namespace renderer
    {
        // Forward declarations
        class RenderEngine;



        /**
         * \brief Command pattern interface for making changes to the scene.
         * 
         * Albeit implementing the same design pattern as Actions,
         * to avoid confusion, these have been named commands rather than actions.
         */
        class SceneAction
        {
        public:
            virtual ~SceneAction() {}

            /**
             * \brief Carry out the encapsulated action.
             *
             * \param the_scene The scene which to execute the action on.
             * \return Implementations should return \c true they failed to execute,
             * \c false otherwise.
             */
            virtual void execute(RenderEngine *engine) = 0;
        };



    }
}

#endif // __PEEKABOT_RENDERER_SCENE_ACTION_HH
