/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <GL/glew.h>

#include "AxisTripod.hh"
#include "Camera.hh"
#include "../State.hh"
#include "../SceneNode.hh"
#include "../RenderContext.hh"
#include "../PrimitiveBatch.hh"

#include "../MeshFactory.hh"



using namespace peekabot;
using namespace peekabot::renderer;
using namespace peekabot::renderer::primitives;


namespace
{
    /*TriMesh create_axis_mesh() throw()
    {
        int cylinder_segments = 8;
        float bottom = 0;
        float top = 1;
        float r = 0.1; // radius of the axis cylinder
        float tip_r = r*1.2; // radius of the tip cone

        // Vertices...
        Vector3r *x = new Vector3r[3*cylinder_segments+1];

        x[cylinder_segments+1] = Vector3r(0,0,bottom);
        for( int i = 0; i < cylinder_segments; ++i )
        {
            // Bottom:
            float alpha = i*(3.14159*2/cylinder_segments);
            x[i+1] = Vector3r(r*cosf(alpha), r*sinf(alpha), bottom);

            // Sides:
            x[(cylinder_segments+1)+2*i+0] = Vector3r(r*cosf(alpha), r*sinf(alpha), top);
            x[(cylinder_segments+1)+2*i+1] = Vector3r(r*cosf(alpha), r*sinf(alpha), bottom);
        }

        boost::shared_ptr<VertexBuffer> vertices(new VertexBuffer);
        vertices->set_data(x, sizeof(Vector3r)*(2*(cylinder_segments+1) + 2*cylinder_segments), 
                           VertexBuffer::STATIC_DRAW);
    


        // Normals...
        //
        // Bottom:
        for( int i = 0; i <= cylinder_segments; ++i )
            x[i] = Vector3r(0, 0, -1);

        // Sides:
        for( int i = 0; i < cylinder_segments; ++i )
        {
            float alpha = i*(3.14159*2/cylinder_segments);
            x[(cylinder_segments+1)+2*i+0] = Vector3r(cosf(alpha), sinf(alpha), 0);
            x[(cylinder_segments+1)+2*i+1] = Vector3r(cosf(alpha), sinf(alpha), 0);
        }


        boost::shared_ptr<VertexBuffer> normals(new VertexBuffer);
        normals->set_data(x, sizeof(Vector3r)*((cylinder_segments+1) + 2*cylinder_segments), 
                          VertexBuffer::STATIC_DRAW);

        delete[] x;

        // Indices...
        uint32_t *inds = new uint32[3*(3*cylinder_segments)];

        // Top/bottom:
        inds[0] = 0;
        inds[1] = cylinder_segments;
        inds[2] = 1;

        for( int i = 1; i < cylinder_segments; ++i )
        {
            inds[3*i+0] = 0;
            inds[3*i+1] = i;
            inds[3*i+2] = i+1;
        }

        // Sides:
        for( int i = 0; i < cylinder_segments-1; ++i )
        {
            inds[3*(cylinder_segments+2*i)+0] = (cylinder_segments+1)+2*i+0;
            inds[3*(cylinder_segments+2*i)+1] = (cylinder_segments+1)+2*i+1;
            inds[3*(cylinder_segments+2*i)+2] = (cylinder_segments+1)+2*(i+1)+1;

            inds[3*(cylinder_segments+2*i+1)+0] = (cylinder_segments+1)+2*i+0;
            inds[3*(cylinder_segments+2*i+1)+1] = (cylinder_segments+1)+2*(i+1)+1;
            inds[3*(cylinder_segments+2*i+1)+2] = (cylinder_segments+1)+2*(i+1)+0;
        }

        inds[3*(cylinder_segments+2*(cylinder_segments-1))+0] = (cylinder_segments+1)+2*(cylinder_segments-1)+0;
        inds[3*(cylinder_segments+2*(cylinder_segments-1))+1] = (cylinder_segments+1)+2*(cylinder_segments-1)+1;
        inds[3*(cylinder_segments+2*(cylinder_segments-1))+2] = (cylinder_segments+1)+2*0+1;



        boost::shared_ptr<IndexBuffer> indices(new IndexBuffer);
        indices->set_data(inds, sizeof(uint32)*(3*(3*cylinder_segments)), 
                          IndexBuffer::STATIC_DRAW);

        delete[] inds;


        return TriMesh(
            vertices, normals, 
            boost::shared_ptr<VertexBuffer>(), 
            boost::shared_ptr<VertexBuffer>(), 
            indices, 0, 3*cylinder_segments);
            }*/
}



// ----------------------------------


AxisTripod::AxisTripod()
    : Cullable(new Cullable::OnScreenSize(64, 64))
{
    get_state().set(new statelets::Lighting(false));
    get_state().set(new statelets::Alpha(1.0f));
    // Tripods aren't pickable
    get_state().set(new statelets::Name(0));
    
    toggle_always_on_top(true);
    toggle_selection_effects(false);
}

void AxisTripod::calculate_bvs() throw()
{              
    set_bounding_sphere(
        BoundingSphere(get_parent()->get_world_pos(), 0));
}

AxisTripod *AxisTripod::clone() const throw()
{
    return new AxisTripod();
}

void AxisTripod::get_renderables(
    Camera *camera, EnumerationCallback callback) throw()
{
    callback(this);
}

void AxisTripod::render(RenderContext &context) throw()
{
    const primitives::Camera *cam = context.get_camera();

    const Vector4f &wp = get_parent()->get_world_pos();

    // v = The tripods position relative the camera used to render it
    Vector3r v = wp-cam->get_viewer_world_pos();
    // u = Camera viewing vector
    Vector3r u = cam->get_view_vector();

    // Overwrite v with the projection of v on u.
    // |v| is now the distance to the tripod along the view vector
    v = (u*v) * u;

    // We can calculate how big the tripod should be by:
    // 1) calculating how big (in meters) it should be to be rendered as 64 
    //    pixels if it was situated at depth x=near
    // 2) use similarity and the result from 1 to calculate how big it has to
    //    be given its depth of x=|v|
    float sz = (
        64/cam->get_pixels_per_meter() * // 1 
        v.norm()/cam->get_near_plane()); // 2!
    
    context.get_batch(0).render_line(
        wp, wp + get_parent()->get_mtow()(0)*sz,
        RGBColor(1,0,0));

    context.get_batch(0).render_line(
        wp, wp + get_parent()->get_mtow()(1)*sz,
        RGBColor(0,1,0));

    context.get_batch(0).render_line(
        wp, wp + get_parent()->get_mtow()(2)*sz,
        RGBColor(0,0,1));
}
