/*
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __PEEKABOT_RENDERER_PRIMITIVES_MESH_BASED_HH
#define __PEEKABOT_RENDERER_PRIMITIVES_MESH_BASED_HH


#include "../../Types.hh"
#include "../Cullable.hh"
#include "../BoundingVolumes.hh"
#include "../Mesh.hh"


namespace peekabot
{
    namespace renderer
    {
        namespace primitives
        {


            class MeshBased : public Cullable
            {
            public:
                MeshBased(uint8_t lod_count, 
                          const BoundingSphere &bsphere, 
                          real32 hysteresis) throw();

                virtual ~MeshBased();

                MeshBased(const MeshBased &x) throw();

                virtual MeshBased *clone() const throw();

                void set_lod(uint8_t lod_no, uint16_t coverage, const Mesh *mesh) throw();

                virtual void get_renderables(
                    Camera *camera, EnumerationCallback callback) throw();

                virtual void transform_geometry(
                    const boost::function<Vector3r (const Vector4r &)> &fun) throw();

            protected:
                virtual void calculate_bvs() throw();
    
            protected:
                BoundingSphere m_base_bsphere;

                struct LOD
                {
                    LOD() throw() : m_mesh(0), m_threshold(0) {}
                    ~LOD() throw() { if( m_mesh != 0 ) delete m_mesh; }

                    Mesh   *m_mesh;
                    uint16_t  m_threshold;
                };

                uint8_t m_n_lods;
                uint8_t m_curr_lod;
                // 0-100
                uint8_t m_hysteresis;

                LOD *m_lods;
            };


        }
    }
}


#endif // __PEEKABOT_RENDERER_PRIMITIVES_MESH_BASED_HH
