/*
 * Copyright Staffan Gimåker 2008.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef __PEEKABOT_SERIALIZATION_SERIALIZABLE_HH
#define __PEEKABOT_SERIALIZATION_SERIALIZABLE_HH


namespace peekabot
{
    class SerializationInterface;
    class DeserializationInterface;
    class SerializableFactory;

    namespace detail
    {
        class SerializableFriend;
    }


    class Serializable
    {
        friend class detail::SerializableFriend;

    public:
        virtual ~Serializable() {}
    
    protected:
        virtual void save(SerializationInterface &ar) const = 0;

        virtual void load(DeserializationInterface &ar) = 0;
    };


    namespace detail
    {
        class SerializableFriend
        {
        protected:
            template<class T>
            inline void save(const T &x, SerializationInterface &ar) const
            {
                x.save(ar);
            }

            template<class T>
            inline void load(T &x, DeserializationInterface &ar) const
            {
                x.load(ar);
            }
        };
    }
}

#define PEEKABOT_SERIALIZABLE \
    friend class peekabot::SerializableFactory; \
    friend class peekabot::detail::SerializableFriend; \
    friend class peekabot::SerializationInterface; \
    friend class peekabot::DeserializationInterface;


#endif // __PEEKABOT_SERIALIZATION_SERIALIZABLE_HH
